@@ -267,7 +267,7 @@ void Magick::Options::fillPattern ( const MagickCore::Image *fillPattern_ )
 		    static_cast<MagickBooleanType>(MagickTrue),
 		    &exceptionInfo );
       throwException( exceptionInfo );
-  DestroyExceptionInfo( &exceptionInfo );
+      (void) DestroyExceptionInfo( &exceptionInfo );
     }
 }
 const MagickCore::Image* Magick::Options::fillPattern ( void  ) const
@@ -324,7 +324,8 @@ std::string Magick::Options::format ( void ) const
   GetExceptionInfo(&exception);
   if ( *_imageInfo->magick != '\0' )
     magick_info = GetMagickInfo( _imageInfo->magick , &exception);
-  DestroyExceptionInfo(&exception);
+  throwException( exception );
+  (void) DestroyExceptionInfo( &exception );
   
   if (( magick_info != 0 ) && 
       ( *magick_info->description != '\0' ))
@@ -353,7 +354,7 @@ void Magick::Options::magick ( const std::string &magick_ )
     throwExceptionExplicit( OptionWarning,
 			    "Unrecognized image format",
 			    magick_.c_str() );
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo( &exception );
 }
 std::string Magick::Options::magick ( void ) const
 {
@@ -585,7 +586,7 @@ void Magick::Options::strokePattern ( const MagickCore::Image *strokePattern_ )
 		    MagickTrue,
 		    &exceptionInfo );
       throwException( exceptionInfo );
-  DestroyExceptionInfo( &exceptionInfo );
+      (void) DestroyExceptionInfo( &exceptionInfo );
     }
 }
 const MagickCore::Image* Magick::Options::strokePattern ( void  ) const
