@@ -10,14 +10,14 @@
 %                             JJ    P      22222                              %
 %                                                                             %
 %                                                                             %
-%                     Read/Write JPEG-2000 Image Format.                      %
+%                     Read/Write JPEG-2000 Image Format                       %
 %                                                                             %
 %                                John Cristy                                  %
 %                                Nathan Brown                                 %
 %                                 June 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/colorspace.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
@@ -248,6 +249,7 @@ static int BlobClose(jas_stream_obj_t *object)
 
   source=(StreamManager *) object;
   (void) CloseBlob(source->image);
+  source=(StreamManager *) RelinquishMagickMemory(source);
   return(0);
 }
 
@@ -340,6 +342,10 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
+  QuantumAny
+    pixel,
+    scale[4];
+
   register long
     i,
     x;
@@ -350,7 +356,6 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned long
     maximum_component_depth,
     number_components,
-    scale[4],
     x_step[4],
     y_step[4];
 
@@ -450,13 +455,17 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   image->compression=JPEG2000Compression;
   for (i=0; i < (long) number_components; i++)
   {
-    if ((((long) (jas_image_cmptwidth(jp2_image,components[i])*
-          jas_image_cmpthstep(jp2_image,components[i])) != (long) image->columns)) ||
-        (((long) (jas_image_cmptheight(jp2_image,components[i])*
-          jas_image_cmptvstep(jp2_image,components[i])) != (long) image->rows)) ||
+    unsigned long
+      height,
+      width;
+
+    width=(unsigned long) (jas_image_cmptwidth(jp2_image,components[i])*
+      jas_image_cmpthstep(jp2_image,components[i]));
+    height=(unsigned long) (jas_image_cmptheight(jp2_image,components[i])*
+      jas_image_cmptvstep(jp2_image,components[i]));
+    if ((width != image->columns) || (height != image->rows) ||
         (jas_image_cmpttlx(jp2_image,components[i]) != 0) ||
-        (jas_image_cmpttly(jp2_image,components[i]) != 0) ||
-        (jas_image_cmptsgnd(jp2_image,components[i]) != MagickFalse))
+        (jas_image_cmpttly(jp2_image,components[i]) != 0))
       {
         (void) jas_stream_close(jp2_stream);
         jas_image_destroy(jp2_image);
@@ -475,11 +484,6 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       jas_image_destroy(jp2_image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   maximum_component_depth=0;
   for (i=0; i < (long) number_components; i++)
   {
@@ -498,20 +502,11 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   else
     image->depth=(unsigned long) MagickMin(MAGICKCORE_QUANTUM_DEPTH,16);
   for (i=0; i < (long) number_components; i++)
-  {
-    scale[i]=1;
-    if (jas_image_cmptprec(jp2_image,components[i]) < 16)
-      scale[i]=(1U << (16-jas_image_cmptprec(jp2_image,components[i])))+1;
-  }
-  if (number_components == 1)
-    {
-      image->colors=(image->depth == 8 ? 256UL : MaxColormapSize);
-      if (AcquireImageColormap(image,image->colors) == MagickFalse)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    }
+    scale[i]=GetQuantumScale((unsigned long) jas_image_cmptprec(jp2_image,
+      components[i]));
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
     for (i=0; i < (long) number_components; i++)
@@ -522,34 +517,17 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       case 1:
       {
-        register IndexPacket
-          *indexes;
-
-        IndexPacket
-          index;
-
         /*
           Grayscale.
         */
-        indexes=GetIndexes(image);
-        if (image->depth == 8)
-          for (x=0; x < (long) image->columns; x++)
-          {
-            index=ScaleQuantumToChar(ScaleShortToQuantum((unsigned short)
-              (scale[0]*jas_matrix_getv(pixels[0],x))));
-            index=ConstrainColormapIndex(image,index);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-          }
-        else
-          for (x=0; x < (long) image->columns; x++)
-          {
-            index=(unsigned short) (scale[0]*jas_matrix_getv(pixels[0],x/
-              x_step[0]));
-            index=ConstrainColormapIndex(image,index);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-          }
+        for (x=0; x < (long) image->columns; x++)
+        {
+          pixel=(QuantumAny) jas_matrix_getv(pixels[0],x/x_step[0]);
+          q->red=ScaleAnyToQuantum(pixel,image->depth,scale[0]);
+          q->green=q->red;
+          q->blue=q->red;
+          q++;
+        }
         break;
       }
       case 3:
@@ -559,12 +537,12 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=ScaleShortToQuantum((unsigned short) (scale[0]*
-            jas_matrix_getv(pixels[0],x/x_step[0])));
-          q->green=ScaleShortToQuantum((unsigned short) (scale[1]*
-            jas_matrix_getv(pixels[1],x/x_step[1])));
-          q->blue=ScaleShortToQuantum((unsigned short) (scale[2]*
-            jas_matrix_getv(pixels[2],x/x_step[2])));
+          pixel=(QuantumAny) jas_matrix_getv(pixels[0],x/x_step[0]);
+          q->red=ScaleAnyToQuantum(pixel,image->depth,scale[0]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[1],x/x_step[1]);
+          q->green=ScaleAnyToQuantum(pixel,image->depth,scale[1]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[2],x/x_step[2]);
+          q->blue=ScaleAnyToQuantum(pixel,image->depth,scale[2]);
           q++;
         }
         break;
@@ -576,20 +554,20 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=ScaleShortToQuantum((unsigned short) (scale[0]*
-            jas_matrix_getv(pixels[0],x/x_step[0])));
-          q->green=ScaleShortToQuantum((unsigned short) (scale[1]*
-            jas_matrix_getv(pixels[1],x/x_step[1])));
-          q->blue=ScaleShortToQuantum((unsigned short) (scale[2]*
-            jas_matrix_getv(pixels[2],x/x_step[2])));
-          q->opacity=(Quantum) QuantumRange-ScaleShortToQuantum((unsigned short)
-            (scale[3]*jas_matrix_getv(pixels[3],x/x_step[3])));
+          pixel=(QuantumAny) jas_matrix_getv(pixels[0],x/x_step[0]);
+          q->red=ScaleAnyToQuantum(pixel,image->depth,scale[0]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[1],x/x_step[1]);
+          q->green=ScaleAnyToQuantum(pixel,image->depth,scale[1]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[2],x/x_step[2]);
+          q->blue=ScaleAnyToQuantum(pixel,image->depth,scale[2]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[3],x/x_step[3]);
+          q->opacity=ScaleAnyToQuantum(pixel,image->depth,scale[3]);
           q++;
         }
         break;
       }
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
     status=SetImageProgress(image,LoadImageTag,y,image->rows);
     if (status == MagickFalse)
@@ -896,7 +874,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   }
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
     for (x=0; x < (long) image->columns; x++)
