@@ -10,14 +10,14 @@
 %                        JJJ    P      EEEEE   GGG                            %
 %                                                                             %
 %                                                                             %
-%                       Read/Write JPEG Image Format.                         %
+%                       Read/Write JPEG Image Format                          %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,6 +46,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -919,15 +920,16 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
          {
            if ((value < hash[i]) && (sum < sums[i]))
              continue;
-           image->quality=(unsigned long) i+1;
+           if ((value <= hash[i]) && (sum <= sums[i]))
+             image->quality=(unsigned long) i+1;
            if (image->debug != MagickFalse)
              {
-               if ((value > hash[i]) || (sum > sums[i]))
-                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                   "Quality: %ld (approximate)",image->quality);
-               else
+               if (image->quality != UndefinedCompressionQuality)
                  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                    "Quality: %ld",image->quality);
+               else
+                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                   "Quality: %ld (approximate)",i+1);
              }
            break;
          }
@@ -1054,12 +1056,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      jpeg_destroy_decompress(&jpeg_info);
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   jpeg_pixels=(JSAMPLE *) AcquireQuantumMemory((size_t) image->columns,
     jpeg_info.output_components*sizeof(JSAMPLE));
   if (jpeg_pixels == (JSAMPLE *) NULL)
@@ -1088,6 +1084,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
           image->colormap[i].red=ScaleCharToQuantum(jpeg_info.colormap[0][i]);
           image->colormap[i].green=image->colormap[i].red;
           image->colormap[i].blue=image->colormap[i].red;
+          image->colormap[i].opacity=OpaqueOpacity;
         }
       else
         for (i=0; i < (long) image->colors; i++)
@@ -1095,6 +1092,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
           image->colormap[i].red=ScaleCharToQuantum(jpeg_info.colormap[0][i]);
           image->colormap[i].green=ScaleCharToQuantum(jpeg_info.colormap[1][i]);
           image->colormap[i].blue=ScaleCharToQuantum(jpeg_info.colormap[2][i]);
+          image->colormap[i].opacity=OpaqueOpacity;
         }
     }
   scanline[0]=(JSAMPROW) jpeg_pixels;
@@ -1107,10 +1105,10 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
         continue;
       }
     p=jpeg_pixels;
-    q=SetImagePixels(image,0,y,image->columns,1);
+    q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetAuthenticIndexQueue(image);
     if (jpeg_info.data_precision > 8)
       {
         if (jpeg_info.output_components == 1)
@@ -1138,6 +1136,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
                 (GETJSAMPLE(*p++) << 4));
               q->blue=ScaleShortToQuantum((unsigned char)
                 (GETJSAMPLE(*p++) << 4));
+              q->opacity=OpaqueOpacity;
               q++;
             }
           else
@@ -1149,6 +1148,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
                 (unsigned char) (GETJSAMPLE(*p++) << 4));
               q->blue=(Quantum) QuantumRange-ScaleShortToQuantum((unsigned char)
                 (GETJSAMPLE(*p++) << 4));
+              q->opacity=OpaqueOpacity;
               indexes[x]=(IndexPacket) QuantumRange-ScaleShortToQuantum(
                 (unsigned char) (GETJSAMPLE(*p++) << 4));
               q++;
@@ -1170,6 +1170,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
             q->red=ScaleCharToQuantum((unsigned char) GETJSAMPLE(*p++));
             q->green=ScaleCharToQuantum((unsigned char) GETJSAMPLE(*p++));
             q->blue=ScaleCharToQuantum((unsigned char) GETJSAMPLE(*p++));
+            q->opacity=OpaqueOpacity;
             q++;
           }
         else
@@ -1181,11 +1182,12 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
               GETJSAMPLE(*p++));
             q->blue=(Quantum) QuantumRange-ScaleCharToQuantum((unsigned char)
               GETJSAMPLE(*p++));
+            q->opacity=OpaqueOpacity;
             indexes[x]=(IndexPacket) QuantumRange-ScaleCharToQuantum(
               (unsigned char) GETJSAMPLE(*p++));
             q++;
           }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
     if (SetImageProgress(image,LoadImageTag,y,image->rows) == MagickFalse)
       break;
@@ -1474,7 +1476,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
       }
     if (LocaleCompare(name,"XMP") == 0)
       {
-        StringInfo      
+        StringInfo
           *xmp_profile;
 
         /*
@@ -1565,6 +1567,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   Image *image)
 {
   const char
+    *option,
     *sampling_factor,
     *value;
 
@@ -1583,15 +1586,12 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  MagickSizeType
-    number_pixels;
+  register const IndexPacket
+    *indexes;
 
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register JSAMPLE
     *q;
 
@@ -1700,24 +1700,60 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       if (image->units == PixelsPerCentimeterResolution)
         jpeg_info.density_unit=(UINT8) 2;
     }
-  number_pixels=(MagickSizeType) image->columns*image->rows;
-  if (number_pixels == (MagickSizeType) ((size_t) number_pixels))
+  option=GetImageOption(image_info,"jpeg:dct-method");
+  if (option != (const char *) NULL)
+    switch (*option)
     {
-      /*
-        Perform optimization only if available memory resources permit it.
-      */
-      status=AcquireMagickResource(MemoryResource,8UL*number_pixels);
-      if (status != MagickFalse)
-        {
-          const char
-            *option;
+      case 'D':
+      case 'd':
+      {
+        if (LocaleCompare(option,"default") == 0)
+          jpeg_info.dct_method=JDCT_DEFAULT;
+        break;
+      }
+      case 'F':
+      case 'f':
+      {
+        if (LocaleCompare(option,"fastest") == 0)
+          jpeg_info.dct_method=JDCT_FASTEST;
+        if (LocaleCompare(option,"float") == 0)
+          jpeg_info.dct_method=JDCT_FLOAT;
+        break;
+      }
+      case 'I':
+      case 'i':
+      {
+        if (LocaleCompare(option,"ifast") == 0)
+          jpeg_info.dct_method=JDCT_IFAST;
+        if (LocaleCompare(option,"islow") == 0)
+          jpeg_info.dct_method=JDCT_ISLOW;
+        break;
+      }
+    }
+  option=GetImageOption(image_info,"jpeg:optimize-coding");
+  if (option != (const char *) NULL)
+    {
+      jpeg_info.optimize_coding=MagickFalse;
+      if (IsMagickTrue(option) != MagickFalse)
+        jpeg_info.optimize_coding=MagickTrue;
+    }
+  else
+    {
+      MagickSizeType
+        length;
 
-          option=GetImageOption(image_info,"jpeg:optimize-coding");
-          if ((option == (const char *) NULL) ||
-              (LocaleCompare(option,"true") == 0))
+      length=(MagickSizeType) jpeg_info.input_components*image->columns*
+        image->rows*sizeof(JSAMPLE);
+      if (length == (MagickSizeType) ((size_t) length))
+        {
+          /*
+            Perform optimization only if available memory resources permit it.
+          */
+          status=AcquireMagickResource(MemoryResource,length);
+          if (status != MagickFalse)
             jpeg_info.optimize_coding=MagickTrue;
+          RelinquishMagickResource(MemoryResource,length);
         }
-      RelinquishMagickResource(MemoryResource,8*number_pixels);
     }
 #if (JPEG_LIB_VERSION >= 61) && defined(C_PROGRESSIVE_SUPPORTED)
   if ((LocaleCompare(image_info->magick,"PJPEG") == 0) ||
@@ -1741,7 +1777,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       (image->quality <= 100))
     {
       if (image->quality == UndefinedCompressionQuality)
-        jpeg_set_quality(&jpeg_info,85,MagickTrue);
+        jpeg_set_quality(&jpeg_info,92,MagickTrue);
       else
         jpeg_set_quality(&jpeg_info,(int) image->quality,MagickTrue);
       if (image->debug != MagickFalse)
@@ -1982,7 +2018,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       if (jpeg_info.in_color_space == JCS_GRAYSCALE)
         for (y=0; y < (long) image->rows; y++)
         {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
           q=jpeg_pixels;
@@ -2001,7 +2037,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
             (jpeg_info.in_color_space == JCS_YCbCr))
           for (y=0; y < (long) image->rows; y++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+            p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
             q=jpeg_pixels;
@@ -2020,11 +2056,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
         else
           for (y=0; y < (long) image->rows; y++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+            p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
             q=jpeg_pixels;
-            indexes=GetIndexes(image);
+            indexes=GetVirtualIndexQueue(image);
             for (x=0; x < (long) image->columns; x++)
             {
               /*
@@ -2046,7 +2082,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
     if (jpeg_info.in_color_space == JCS_GRAYSCALE)
       for (y=0; y < (long) image->rows; y++)
       {
-        p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+        p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
         q=jpeg_pixels;
@@ -2064,7 +2100,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           (jpeg_info.in_color_space == JCS_YCbCr))
         for (y=0; y < (long) image->rows; y++)
         {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
           q=jpeg_pixels;
@@ -2082,11 +2118,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       else
         for (y=0; y < (long) image->rows; y++)
         {
-          p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+          p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
           q=jpeg_pixels;
-          indexes=GetIndexes(image);
+          indexes=GetVirtualIndexQueue(image);
           for (x=0; x < (long) image->columns; x++)
           {
             /*
