@@ -10,14 +10,14 @@
 %                            P      DDDD   F                                  %
 %                                                                             %
 %                                                                             %
-%                   Read/Write Portable Document Format.                      %
+%                   Read/Write Portable Document Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -245,6 +246,24 @@ static MagickBooleanType IsPDF(const unsigned char *magick,const size_t offset)
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
+static MagickBooleanType IsPDFRendered(const char *path)
+{
+  MagickBooleanType
+    status;
+
+  struct stat
+    attributes;
+
+  if ((path == (const char *) NULL) || (*path == '\0'))
+    return(MagickFalse);
+  status=GetPathAttributes(path,&attributes);
+  if ((status != MagickFalse) && S_ISREG(attributes.st_mode) &&
+      (attributes.st_size > 0))
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
 static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
 #define CropBox  "CropBox"
@@ -391,7 +410,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       c=' ';
     *p++=(char) c;
     if ((c != (int) '/') && (c != (int) '%') && 
-        ((size_t) (p-command) < MaxTextExtent))
+        ((size_t) (p-command) < (MaxTextExtent-1)))
       continue;
     *(--p)='\0';
     p=command;
@@ -514,7 +533,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImage(image);
       return((Image *) NULL);
     }
-  (void) write(file," ",1);
+  status=write(file," ",1);
   file=close(file)-1;
   /*
     Render Postscript with the Ghostscript delegate.
@@ -580,7 +599,8 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->filename,postscript_filename,input_filename);
   status=InvokePostscriptDelegate(read_info->verbose,command);
   pdf_image=(Image *) NULL;
-  if (status == MagickFalse)
+  if ((status == MagickFalse) &&
+      (IsPDFRendered(read_info->filename) != MagickFalse))
     pdf_image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(postscript_filename);
   (void) RelinquishUniqueFileResource(read_info->filename);
@@ -897,12 +917,12 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     media_info,
     page_info;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register unsigned char
     *q;
 
@@ -1068,7 +1088,9 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   scene=0;
   do
   {
-    compression=image_info->compression;
+    compression=image->compression;
+    if (image_info->compression != UndefinedCompression)
+      compression=image_info->compression;
     switch (compression)
     {
       case FaxCompression:
@@ -1177,7 +1199,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     scale.y=(double) (geometry.height*delta.y)/resolution.y;
     geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info,
+      &image->exception);
     if (image->gravity != UndefinedGravity)
       {
         geometry.x=(-page_info.x);
@@ -1460,7 +1483,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             q=pixels;
             for (y=0; y < (long) image->rows; y++)
             {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
+              p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
@@ -1501,7 +1524,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             Ascii85Initialize(image);
             for (y=0; y < (long) image->rows; y++)
             {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
+              p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
@@ -1561,11 +1584,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             q=pixels;
             for (y=0; y < (long) image->rows; y++)
             {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
+              p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(image);
+              indexes=GetVirtualIndexQueue(image);
               for (x=0; x < (long) image->columns; x++)
               {
                 *q++=ScaleQuantumToChar(p->red);
@@ -1607,11 +1630,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             Ascii85Initialize(image);
             for (y=0; y < (long) image->rows; y++)
             {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
+              p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(image);
+              indexes=GetVirtualIndexQueue(image);
               for (x=0; x < (long) image->columns; x++)
               {
                 Ascii85Encode(image,ScaleQuantumToChar(p->red));
@@ -1657,11 +1680,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               q=pixels;
               for (y=0; y < (long) image->rows; y++)
               {
-                p=AcquireImagePixels(image,0,y,image->columns,1,
+                p=GetVirtualPixels(image,0,y,image->columns,1,
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(image);
+                indexes=GetVirtualIndexQueue(image);
                 for (x=0; x < (long) image->columns; x++)
                   *q++=(unsigned char) indexes[x];
                 if (image->previous == (Image *) NULL)
@@ -1696,11 +1719,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               Ascii85Initialize(image);
               for (y=0; y < (long) image->rows; y++)
               {
-                p=AcquireImagePixels(image,0,y,image->columns,1,
+                p=GetVirtualPixels(image,0,y,image->columns,1,
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(image);
+                indexes=GetVirtualIndexQueue(image);
                 for (x=0; x < (long) image->columns; x++)
                   Ascii85Encode(image,(unsigned char) indexes[x]);
                 if (image->previous == (Image *) NULL)
@@ -1896,7 +1919,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             q=pixels;
             for (y=0; y < (long) tile_image->rows; y++)
             {
-              p=AcquireImagePixels(tile_image,0,y,tile_image->columns,1,
+              p=GetVirtualPixels(tile_image,0,y,tile_image->columns,1,
                 &tile_image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
@@ -1931,7 +1954,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             Ascii85Initialize(image);
             for (y=0; y < (long) tile_image->rows; y++)
             {
-              p=AcquireImagePixels(tile_image,0,y,tile_image->columns,1,
+              p=GetVirtualPixels(tile_image,0,y,tile_image->columns,1,
                 &tile_image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
@@ -1989,11 +2012,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             q=pixels;
             for (y=0; y < (long) tile_image->rows; y++)
             {
-              p=AcquireImagePixels(tile_image,0,y,tile_image->columns,1,
+              p=GetVirtualPixels(tile_image,0,y,tile_image->columns,1,
                 &tile_image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(tile_image);
+              indexes=GetVirtualIndexQueue(tile_image);
               for (x=0; x < (long) tile_image->columns; x++)
               {
                 *q++=ScaleQuantumToChar(p->red);
@@ -2029,11 +2052,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             Ascii85Initialize(image);
             for (y=0; y < (long) tile_image->rows; y++)
             {
-              p=AcquireImagePixels(tile_image,0,y,tile_image->columns,1,
+              p=GetVirtualPixels(tile_image,0,y,tile_image->columns,1,
                 &tile_image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              indexes=GetIndexes(tile_image);
+              indexes=GetVirtualIndexQueue(tile_image);
               for (x=0; x < (long) tile_image->columns; x++)
               {
                 Ascii85Encode(image,ScaleQuantumToChar(p->red));
@@ -2076,11 +2099,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               q=pixels;
               for (y=0; y < (long) tile_image->rows; y++)
               {
-                p=AcquireImagePixels(tile_image,0,y,tile_image->columns,1,
+                p=GetVirtualPixels(tile_image,0,y,tile_image->columns,1,
                   &tile_image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(tile_image);
+                indexes=GetVirtualIndexQueue(tile_image);
                 for (x=0; x < (long) tile_image->columns; x++)
                   *q++=(unsigned char) indexes[x];
               }
@@ -2109,11 +2132,11 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
               Ascii85Initialize(image);
               for (y=0; y < (long) tile_image->rows; y++)
               {
-                p=AcquireImagePixels(tile_image,0,y,tile_image->columns,1,
+                p=GetVirtualPixels(tile_image,0,y,tile_image->columns,1,
                   &tile_image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                indexes=GetIndexes(tile_image);
+                indexes=GetVirtualIndexQueue(tile_image);
                 for (x=0; x < (long) tile_image->columns; x++)
                   Ascii85Encode(image,(unsigned char) indexes[x]);
               }
@@ -2272,7 +2295,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             q=pixels;
             for (y=0; y < (long) image->rows; y++)
             {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
+              p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
@@ -2307,7 +2330,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
             Ascii85Initialize(image);
             for (y=0; y < (long) image->rows; y++)
             {
-              p=AcquireImagePixels(image,0,y,image->columns,1,
+              p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
