@@ -10,7 +10,7 @@
 %                            P      SSSSS  33333                              %
 %                                                                             %
 %                                                                             %
-%                     Write Postscript Level III Format.                      %
+%                     Write Postscript Level III Format                       %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,6 +43,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/compress.h"
@@ -223,10 +224,10 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
   q=(*pixels);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=AcquireIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     if (image->colorspace != CMYKColorspace)
       for (x=0; x < (long) image->columns; x++)
       {
@@ -296,7 +297,7 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
   q=(*pixels);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
     if (pack == 1)
@@ -365,10 +366,10 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
   q=(*pixels);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=AcquireIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     for (x=0; x < (long) image->columns; x++)
       *q++=(unsigned char) indexes[x];
     if (image->previous == (Image *) NULL)
@@ -839,7 +840,9 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(MagickFalse);
-  compression=image_info->compression;
+  compression=image->compression;
+  if (image_info->compression != UndefinedCompression)
+    compression=image_info->compression;
   switch (compression)
   {
     case FaxCompression:
@@ -926,7 +929,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     scale.y=(double) (geometry.height*delta.y)/resolution.y;
     geometry.height=(unsigned long) (scale.y+0.5);
     (void) ParseAbsoluteGeometry(page_geometry,&media_info);
-    (void) ParseGravityGeometry(image,page_geometry,&page_info);
+    (void) ParseGravityGeometry(image,page_geometry,&page_info,
+      &image->exception);
     if (image->gravity != UndefinedGravity)
       {
         geometry.x=(-page_info.x);
