@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/constitute.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
@@ -173,12 +174,12 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
       x=0;
       for (; (x < (long) image->columns) && (j < MAGICKCORE_QUANTUM_DEPTH); x++)
       {
-        pixel=AcquireOnePixel(watermark,k % (long) watermark->columns,
-          k/(long) watermark->columns,exception);
-        q=GetImagePixels(image,x,y,1,1);
+        (void) GetOneVirtualPixel(watermark,k % (long) watermark->columns,
+          k/(long) watermark->columns,&pixel,exception);
+        q=GetAuthenticPixels(image,x,y,1,1,exception);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         switch (c)
         {
           case 0:
@@ -197,7 +198,7 @@ static Image *ReadSTEGANOImage(const ImageInfo *image_info,
             break;
           }
         }
-        if (SyncImagePixels(image) == MagickFalse)
+        if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
         c++;
         if (c == 3)
@@ -248,7 +249,7 @@ ModuleExport unsigned long RegisterSTEGANOImage(void)
 
   entry=SetMagickInfo("STEGANO");
   entry->decoder=(DecodeImageHandler *) ReadSTEGANOImage;
-  entry->format_type=ImplicitFormatType;
+  entry->format_type=ExplicitFormatType;
   entry->description=ConstantString("Steganographic image");
   entry->module=ConstantString("STEGANO");
   (void) RegisterMagickInfo(entry);
