@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -43,6 +43,7 @@
 #include "magick/annotate.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/colorspace.h"
@@ -232,11 +233,6 @@ static Image *ReadTEXTImage(const ImageInfo *image_info,
   image->rows=(unsigned long) (((page.height*image->y_resolution)/delta.y)+0.5);
   image->page.x=0;
   image->page.y=0;
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   texture=(Image *) NULL;
   if (image_info->texture != (char *) NULL)
     {
@@ -466,7 +462,7 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
           &pixel.red,&pixel.green,&pixel.blue);
     if (count < 5)
       continue;
-    q=GetImagePixels(image,x,y,1,1);
+    q=GetAuthenticPixels(image,x,y,1,1,exception);
     if (q == (PixelPacket *) NULL)
       continue;
     q->red=ScaleAnyToQuantum(pixel.red,image->depth,scale);
@@ -474,13 +470,13 @@ static Image *ReadTXTImage(const ImageInfo *image_info,ExceptionInfo *exception)
     q->blue=ScaleAnyToQuantum(pixel.blue,image->depth,scale);
     if (image->colorspace == CMYKColorspace)
       {
-        indexes=GetIndexes(image);
+        indexes=GetAuthenticIndexQueue(image);
         *indexes=ScaleAnyToQuantum(pixel.index,image->depth,scale);
       }
     if (image->matte != MagickFalse)
       q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel.opacity,
         image->depth,scale));
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
   return(GetFirstImageInList(image));
@@ -599,12 +595,12 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
   MagickPixelPacket
     pixel;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -633,10 +629,10 @@ static MagickBooleanType WriteTXTImage(const ImageInfo *image_info,Image *image)
   GetMagickPixelPacket(image,&pixel);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=GetIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     for (x=0; x < (long) image->columns; x++)
     {
       (void) FormatMagickString(buffer,MaxTextExtent,"%ld,%ld: ",x,y);
