@@ -10,14 +10,14 @@
 %               CCCC   OOO   M   M  P      A   A  R  R    EEEEE               %
 %                                                                             %
 %                                                                             %
-%                         Image Comparison Methods                            %
+%                      MagickCore Image Comparison Methods                    %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -96,7 +96,7 @@
 %
 %    o distortion: the computed distortion between the images.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -125,18 +125,18 @@ MagickExport Image *CompareImageChannels(Image *image,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   MagickPixelPacket
     highlight,
     lowlight,
     zero;
 
   ViewInfo
-    **highlight_view,
-    **image_view,
-    **reconstruct_view;
-
-  volatile MagickBooleanType
-    status;
+    *highlight_view,
+    *image_view,
+    *reconstruct_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -192,12 +192,17 @@ MagickExport Image *CompareImageChannels(Image *image,
   */
   status=MagickTrue;
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
-  highlight_view=AcquireCacheViewThreadSet(highlight_image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
+  highlight_view=AcquireCacheView(highlight_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
+    MagickBooleanType
+      sync;
+
     MagickPixelPacket
       pixel,
       reconstruct_pixel;
@@ -214,7 +219,6 @@ MagickExport Image *CompareImageChannels(Image *image,
       *highlight_indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
@@ -222,20 +226,20 @@ MagickExport Image *CompareImageChannels(Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
-    q=AcquireCacheViewPixels(reconstruct_view[id],0,y,
-      reconstruct_image->columns,1,exception);
-    r=SetCacheViewPixels(highlight_view[id],0,y,highlight_image->columns,1);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,
+      1,exception);
+    r=QueueCacheViewAuthenticPixels(highlight_view,0,y,highlight_image->columns,
+      1,exception);
     if ((p == (const PixelPacket *) NULL) ||
         (q == (const PixelPacket *) NULL) || (r == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=AcquireCacheViewIndexes(image_view[id]);
-    reconstruct_indexes=AcquireCacheViewIndexes(reconstruct_view[id]);
-    highlight_indexes=GetCacheViewIndexes(highlight_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
+    highlight_indexes=GetCacheViewAuthenticIndexQueue(highlight_view);
     pixel=zero;
     reconstruct_pixel=zero;
     for (x=0; x < (long) image->columns; x++)
@@ -260,7 +264,8 @@ MagickExport Image *CompareImageChannels(Image *image,
             difference=MagickTrue;
           if (((channel & BlueChannel) != 0) && (p->blue != q->blue))
             difference=MagickTrue;
-          if (((channel & OpacityChannel) != 0) && (p->opacity != q->opacity))
+          if (((channel & OpacityChannel) != 0) &&
+              (image->matte != MagickFalse) && (p->opacity != q->opacity))
             difference=MagickTrue;
           if ((((channel & IndexChannel) != 0) &&
                (image->colorspace == CMYKColorspace) &&
@@ -276,12 +281,13 @@ MagickExport Image *CompareImageChannels(Image *image,
       q++;
       r++;
     }
-    if (SyncCacheView(highlight_view[id]) == MagickFalse)
+    sync=SyncCacheViewAuthenticPixels(highlight_view,exception);
+    if (sync == MagickFalse)
       status=MagickFalse;
   }
-  highlight_view=DestroyCacheViewThreadSet(highlight_view);
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  highlight_view=DestroyCacheView(highlight_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
   (void) CompositeImage(difference_image,image->compose,highlight_image,0,0);
   highlight_image=DestroyImage(highlight_image);
   if (status == MagickFalse)
@@ -321,7 +327,7 @@ MagickExport Image *CompareImageChannels(Image *image,
 %
 %    o distortion: the computed distortion between the images.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -344,24 +350,26 @@ static MagickBooleanType GetAbsoluteError(const Image *image,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   MagickPixelPacket
     zero;
 
   ViewInfo
-    **image_view,
-    **reconstruct_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view,
+    *reconstruct_view;
 
   /*
     Compute the absolute difference in pixels between two images.
   */
   status=MagickTrue;
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     double
@@ -381,22 +389,20 @@ static MagickBooleanType GetAbsoluteError(const Image *image,
 
     register long
       i,
-      id,
       x;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
-    q=AcquireCacheViewPixels(reconstruct_view[id],0,y,
-      reconstruct_image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,
+      1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=AcquireCacheViewIndexes(image_view[id]);
-    reconstruct_indexes=AcquireCacheViewIndexes(reconstruct_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     pixel=zero;
     reconstruct_pixel=pixel;
     (void) ResetMagickMemory(channel_distortion,0,sizeof(channel_distortion));
@@ -413,7 +419,8 @@ static MagickBooleanType GetAbsoluteError(const Image *image,
             channel_distortion[GreenChannel]++;
           if ((channel & BlueChannel) != 0)
             channel_distortion[BlueChannel]++;
-          if ((channel & OpacityChannel) != 0)
+          if (((channel & OpacityChannel) != 0) &&
+              (image->matte != MagickFalse))
             channel_distortion[OpacityChannel]++;
           if (((channel & IndexChannel) != 0) &&
               (image->colorspace == CMYKColorspace))
@@ -423,12 +430,14 @@ static MagickBooleanType GetAbsoluteError(const Image *image,
       p++;
       q++;
     }
-    #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
     for (i=0; i <= (long) AllChannels; i++)
       distortion[i]+=channel_distortion[i];
   }
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -445,7 +454,8 @@ static unsigned long GetNumberChannels(const Image *image,
     channels++;
   if ((channel & BlueChannel) != 0)
     channels++;
-  if ((channel & OpacityChannel) != 0)
+  if (((channel & OpacityChannel) != 0) &&
+       (image->matte != MagickFalse))
     channels++;
   if (((channel & IndexChannel) != 0) &&
       (image->colorspace == CMYKColorspace))
@@ -463,18 +473,20 @@ static MagickBooleanType GetMeanAbsoluteError(const Image *image,
   long
     y;
 
-  ViewInfo
-    **image_view,
-    **reconstruct_view;
-
-  volatile MagickBooleanType
+  MagickBooleanType
     status;
 
+  ViewInfo
+    *image_view,
+    *reconstruct_view;
+
   status=MagickTrue;
   scale=1.0/((double) image->columns*image->rows);
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     double
@@ -490,22 +502,20 @@ static MagickBooleanType GetMeanAbsoluteError(const Image *image,
 
     register long
       i,
-      id,
       x;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
-    q=AcquireCacheViewPixels(reconstruct_view[id],0,y,
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,
       reconstruct_image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=AcquireCacheViewIndexes(image_view[id]);
-    reconstruct_indexes=AcquireCacheViewIndexes(reconstruct_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     (void) ResetMagickMemory(channel_distortion,0,sizeof(channel_distortion));
     for (x=0; x < (long) image->columns; x++)
     {
@@ -530,7 +540,8 @@ static MagickBooleanType GetMeanAbsoluteError(const Image *image,
           channel_distortion[BlueChannel]+=scale*distance;
           channel_distortion[AllChannels]+=scale*distance;
         }
-      if ((channel & OpacityChannel) != 0)
+      if (((channel & OpacityChannel) != 0) &&
+          (image->matte != MagickFalse))
         {
           distance=fabs(p->opacity-(double) q->opacity);
           channel_distortion[OpacityChannel]+=scale*distance;
@@ -546,12 +557,14 @@ static MagickBooleanType GetMeanAbsoluteError(const Image *image,
       p++;
       q++;
     }
-    #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
     for (i=0; i <= (long) AllChannels; i++)
       distortion[i]+=channel_distortion[i];
   }
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
   distortion[AllChannels]/=(double) GetNumberChannels(image,channel);
   return(status);
 }
@@ -598,16 +611,16 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
     register long
       x;
 
-    p=AcquireCacheViewPixels(image_view,0,y,image->columns,1,exception);
-    q=AcquireCacheViewPixels(reconstruct_view,0,y,reconstruct_image->columns,1,
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         break;
       }
-    indexes=AcquireCacheViewIndexes(image_view);
-    reconstruct_indexes=AcquireCacheViewIndexes(reconstruct_view);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     for (x=0; x < (long) image->columns; x++)
     {
       MagickRealType
@@ -650,7 +663,8 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
             maximum_error=distance;
           area++;
         }
-      if ((channel & OpacityChannel) != 0)
+      if (((channel & OpacityChannel) != 0) &&
+          (image->matte != MagickFalse))
         {
           distance=fabs((double) p->opacity-q->opacity);
           distortion[OpacityChannel]+=distance;
@@ -694,18 +708,20 @@ static MagickBooleanType GetMeanSquaredError(const Image *image,
   long
     y;
 
-  ViewInfo
-    **image_view,
-    **reconstruct_view;
-
-  volatile MagickBooleanType
+  MagickBooleanType
     status;
 
+  ViewInfo
+    *image_view,
+    *reconstruct_view;
+
   status=MagickTrue;
   scale=QuantumScale/((double) image->columns*image->rows);
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     double
@@ -721,22 +737,20 @@ static MagickBooleanType GetMeanSquaredError(const Image *image,
 
     register long
       i,
-      id,
       x;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
-    q=AcquireCacheViewPixels(reconstruct_view[id],0,y,
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,
       reconstruct_image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=AcquireCacheViewIndexes(image_view[id]);
-    reconstruct_indexes=AcquireCacheViewIndexes(reconstruct_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     (void) ResetMagickMemory(channel_distortion,0,sizeof(channel_distortion));
     for (x=0; x < (long) image->columns; x++)
     {
@@ -761,7 +775,8 @@ static MagickBooleanType GetMeanSquaredError(const Image *image,
           channel_distortion[BlueChannel]+=scale*distance*distance;
           channel_distortion[AllChannels]+=scale*distance*distance;
         }
-      if ((channel & OpacityChannel) != 0)
+      if (((channel & OpacityChannel) != 0) &&
+          (image->matte != MagickFalse))
         {
           distance=(p->opacity-(MagickRealType) q->opacity);
           channel_distortion[OpacityChannel]+=scale*distance*distance;
@@ -778,12 +793,14 @@ static MagickBooleanType GetMeanSquaredError(const Image *image,
       p++;
       q++;
     }
-    #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
     for (i=0; i <= (long) AllChannels; i++)
       distortion[i]+=channel_distortion[i];
   }
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
   distortion[AllChannels]/=(double) GetNumberChannels(image,channel);
   return(status);
 }
@@ -795,17 +812,19 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
   long
     y;
 
-  ViewInfo
-    **image_view,
-    **reconstruct_view;
-
-  volatile MagickBooleanType
+  MagickBooleanType
     status;
 
+  ViewInfo
+    *image_view,
+    *reconstruct_view;
+
   status=MagickTrue;
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     double
@@ -821,22 +840,20 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
 
     register long
       i,
-      id,
       x;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
-    q=AcquireCacheViewPixels(reconstruct_view[id],0,y,
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,
       reconstruct_image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=AcquireCacheViewIndexes(image_view[id]);
-    reconstruct_indexes=AcquireCacheViewIndexes(reconstruct_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     (void) ResetMagickMemory(channel_distortion,0,sizeof(channel_distortion));
     for (x=0; x < (long) image->columns; x++)
     {
@@ -867,7 +884,8 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
           if (distance > channel_distortion[AllChannels])
             channel_distortion[AllChannels]=distance;
         }
-      if ((channel & OpacityChannel) != 0)
+      if (((channel & OpacityChannel) != 0) &&
+          (image->matte != MagickFalse))
         {
           distance=fabs(p->opacity-(double) q->opacity);
           if (distance > channel_distortion[OpacityChannel])
@@ -888,13 +906,15 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
       p++;
       q++;
     }
-    #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
     for (i=0; i <= (long) AllChannels; i++)
       if (channel_distortion[i] > distortion[i])
         distortion[i]=channel_distortion[i];
   }
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -916,7 +936,8 @@ static MagickBooleanType GetPeakSignalToNoiseRatio(const Image *image,
   if ((channel & BlueChannel) != 0)
     distortion[BlueChannel]=20.0*log10((double) 1.0/sqrt(QuantumScale*
       distortion[BlueChannel]));
-  if ((channel & OpacityChannel) != 0)
+  if (((channel & OpacityChannel) != 0) &&
+      (image->matte != MagickFalse))
     distortion[OpacityChannel]=20.0*log10((double) 1.0/sqrt(QuantumScale*
       distortion[OpacityChannel]));
   if (((channel & IndexChannel) != 0) &&
@@ -943,7 +964,8 @@ static MagickBooleanType GetRootMeanSquaredError(const Image *image,
     distortion[GreenChannel]=sqrt(QuantumRange*distortion[GreenChannel]);
   if ((channel & BlueChannel) != 0)
     distortion[BlueChannel]=sqrt(QuantumRange*distortion[BlueChannel]);
-  if ((channel & OpacityChannel) != 0)
+  if (((channel & OpacityChannel) != 0) &&
+      (image->matte != MagickFalse))
     distortion[OpacityChannel]=sqrt(QuantumRange*distortion[OpacityChannel]);
   if (((channel & IndexChannel) != 0) &&
       (image->colorspace == CMYKColorspace))
@@ -1068,7 +1090,7 @@ MagickExport MagickBooleanType GetImageChannelDistortion(Image *image,
 %
 %    o metric: the metric.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport double *GetImageChannelDistortions(Image *image,
@@ -1208,6 +1230,9 @@ MagickExport double *GetImageChannelDistortions(Image *image,
 MagickExport MagickBooleanType IsImagesEqual(Image *image,
   const Image *reconstruct_image)
 {
+  ExceptionInfo
+    *exception;
+
   long
     y;
 
@@ -1235,6 +1260,7 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
   maximum_error=0.0;
   mean_error_per_pixel=0.0;
   mean_error=0.0;
+  exception=(&image->exception);
   image_view=AcquireCacheView(image);
   reconstruct_view=AcquireCacheView(reconstruct_image);
   for (y=0; y < (long) image->rows; y++)
@@ -1250,13 +1276,13 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
     register long
       x;
 
-    p=AcquireCacheViewPixels(image_view,0,y,image->columns,1,&image->exception);
-    q=AcquireCacheViewPixels(reconstruct_view,0,y,reconstruct_image->columns,1,
-      &image->exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,
+      1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       break;
-    indexes=AcquireCacheViewIndexes(image_view);
-    reconstruct_indexes=AcquireCacheViewIndexes(reconstruct_view);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     for (x=0; x < (long) image->columns; x++)
     {
       MagickRealType
@@ -1280,12 +1306,15 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
       if (distance > maximum_error)
         maximum_error=distance;
       area++;
-      distance=fabs(p->opacity-(double) q->opacity);
-      mean_error_per_pixel+=distance;
-      mean_error+=distance*distance;
-      if (distance > maximum_error)
-        maximum_error=distance;
-      area++;
+      if (image->matte != MagickFalse)
+        {
+          distance=fabs(p->opacity-(double) q->opacity);
+          mean_error_per_pixel+=distance;
+          mean_error+=distance*distance;
+          if (distance > maximum_error)
+            maximum_error=distance;
+          area++;
+        }
       if ((image->colorspace == CMYKColorspace) &&
           (reconstruct_image->colorspace == CMYKColorspace))
         {
