@@ -3,21 +3,21 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%        CCCC   OOO  M   M  PPPP    OOO   SSSSS  IIIII  TTTTT  EEEEE          %
-%       C      O   O MM MM  P   P  O   O  SS       I      T    E              %
-%       C      O   O M M M  PPPP   O   O   SSS     I      T    EEE            %
-%       C      O   O M   M  P      O   O     SS    I      T    E              %
-%        CCCC   OOO  M   M  P       OOO   SSSSS  IIIII    T    EEEEE          %
+%        CCCC   OOO   M   M  PPPP    OOO   SSSSS  IIIII  TTTTT  EEEEE         %
+%       C      O   O  MM MM  P   P  O   O  SS       I      T    E             %
+%       C      O   O  M M M  PPPP   O   O   SSS     I      T    EEE           %
+%       C      O   O  M   M  P      O   O     SS    I      T    E             %
+%        CCCC   OOO   M   M  P       OOO   SSSSS  IIIII    T    EEEEE         %
 %                                                                             %
 %                                                                             %
-%                    ImageMagick Image Composite Methods                      %
+%                     MagickCore Image Composite Methods                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -999,6 +999,9 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
   double
     sans;
 
+  ExceptionInfo
+    *exception;
+
   GeometryInfo
     geometry_info;
 
@@ -1010,7 +1013,8 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
     y;
 
   MagickBooleanType
-    modify_outside_overlay;
+    modify_outside_overlay,
+    status;
 
   MagickPixelPacket
     zero;
@@ -1029,11 +1033,8 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
     flags;
 
   ViewInfo
-    **composite_view,
-    **image_view;
-
-  volatile MagickBooleanType
-    status;
+    *composite_view,
+    *image_view;
 
   /*
     Prepare composite image.
@@ -1118,8 +1119,8 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
         composite_image->rows,MagickTrue,&image->exception);
       if (displace_image == (Image *) NULL)
         return(MagickFalse);
-      horizontal_scale=20.0;
-      vertical_scale=20.0;
+      horizontal_scale=(MagickRealType) composite_image->columns;
+      vertical_scale=(MagickRealType) composite_image->rows;
       if (composite_image->geometry != (char *) NULL)
         {
           /*
@@ -1136,12 +1137,16 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
         Shift image pixels as defined by a displacement map.
       */
       pixel=zero;
+      exception=(&image->exception);
       resample_filter=AcquireResampleFilter(image,&image->exception);
       image_view=AcquireCacheView(image);
       displace_view=AcquireCacheView(displace_image);
       composite_view=AcquireCacheView(composite_image);
       for (y=0; y < (long) composite_image->rows; y++)
       {
+        MagickBooleanType
+          sync;
+
         register const PixelPacket
           *p;
 
@@ -1153,14 +1158,16 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
 
         if (((y+y_offset) < 0) || ((y+y_offset) >= (long) image->rows))
           continue;
-        p=AcquireCacheViewPixels(composite_view,0,y,composite_image->columns,1,
-          &image->exception);
-        q=GetCacheViewPixels(image_view,0,y+y_offset,image->columns,1);
-        r=SetCacheViewPixels(displace_view,0,y,displace_image->columns,1);
+        p=GetCacheViewVirtualPixels(composite_view,0,y,composite_image->columns,
+          1,exception);
+        q=GetCacheViewAuthenticPixels(image_view,0,y+y_offset,image->columns,1,
+          exception);
+        r=QueueCacheViewAuthenticPixels(displace_view,0,y,
+          displace_image->columns,1,&image->exception);
         if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL) ||
             (r == (PixelPacket *) NULL))
           break;
-        displace_indexes=GetCacheViewIndexes(displace_view);
+        displace_indexes=GetCacheViewAuthenticIndexQueue(displace_view);
         q+=x_offset;
         for (x=0; x < (long) composite_image->columns; x++)
         {
@@ -1170,25 +1177,22 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
               q++;
               continue;
             }
-          x_displace=(horizontal_scale*(PixelIntensityToQuantum(p)-
+          x_displace=(horizontal_scale*(p->red -
             (((MagickRealType) QuantumRange+1.0)/2.0)))/
             (((MagickRealType) QuantumRange+1.0)/2.0);
-          if (composite_image->matte != MagickFalse)
-            y_displace=(-vertical_scale*(p->opacity-(((MagickRealType)
-              QuantumRange+1.0)/2.0)))/(((MagickRealType) QuantumRange+1.0)/
-              2.0);
-          else
-            y_displace=(vertical_scale*(PixelIntensityToQuantum(p)-
-              (((MagickRealType) QuantumRange+1.0)/2.0)))/
-              (((MagickRealType) QuantumRange+1.0)/2.0);
-          pixel=ResamplePixelColor(resample_filter,(double)
-            (x_offset+x+x_displace),(double) (y_offset+y+y_displace));
+          y_displace=(vertical_scale*(p->green -
+            (((MagickRealType) QuantumRange+1.0)/2.0)))/
+            (((MagickRealType) QuantumRange+1.0)/2.0);
+          (void) ResamplePixelColor(resample_filter,(double)
+            (x_offset+x+x_displace),(double) (y_offset+y+y_displace),&pixel);
+          /* Add transparency handling here */
           SetPixelPacket(displace_image,&pixel,r,displace_indexes+x);
           p++;
           q++;
           r++;
         }
-        if (SyncCacheView(displace_view) == MagickFalse)
+        sync=SyncCacheViewAuthenticPixels(displace_view,exception);
+        if (sync == MagickFalse)
           break;
       }
       resample_filter=DestroyResampleFilter(resample_filter);
@@ -1285,9 +1289,12 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
   status=MagickTrue;
   progress=0;
   midpoint=((MagickRealType) QuantumRange+1.0)/2;
-  image_view=AcquireCacheViewThreadSet(image);
-  composite_view=AcquireCacheViewThreadSet(composite_image);
-  #pragma omp parallel for schedule(static,64)
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
+  composite_view=AcquireCacheView(composite_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     const PixelPacket
@@ -1313,7 +1320,6 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
       *indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
@@ -1321,7 +1327,6 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
     if (modify_outside_overlay == MagickFalse)
       {
         if (y < y_offset)
@@ -1336,8 +1341,8 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
     p=(PixelPacket *) NULL;
     if ((y >= y_offset) && ((y-y_offset) < (long) composite_image->rows))
       {
-        p=AcquireCacheViewPixels(composite_view[id],0,y-y_offset,
-          composite_image->columns,1,&image->exception);
+        p=GetCacheViewVirtualPixels(composite_view,0,y-y_offset,
+          composite_image->columns,1,exception);
         if (p == (const PixelPacket *) NULL)
           {
             status=MagickFalse;
@@ -1347,14 +1352,15 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
         if (x_offset < 0)
           p-=x_offset;
       }
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
+      exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view[id]);
-    composite_indexes=AcquireCacheViewIndexes(composite_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    composite_indexes=GetCacheViewVirtualIndexQueue(composite_view);
     GetMagickPixelPacket(composite_image,&source);
     GetMagickPixelPacket(image,&destination);
     hue=0.0;
@@ -1823,22 +1829,24 @@ MagickExport MagickBooleanType CompositeImageChannel(Image *image,
         p=pixels;
       q++;
     }
-    if (SyncCacheView(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,CompositeImageTag,progress++,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  composite_view=DestroyCacheViewThreadSet(composite_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  composite_view=DestroyCacheView(composite_view);
+  image_view=DestroyCacheView(image_view);
   if (displace_image != (Image * ) NULL)
     displace_image=DestroyImage(displace_image);
   return(status);
