@@ -10,14 +10,14 @@
 %            DDDD   EEEEE   CCCC   OOO   R  R   A   A    T    EEEEE           %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Decoration Methods                    %
+%                     MagickCore Image Decoration Methods                     %
 %                                                                             %
 %                                Software Design                              %
 %                                  John Cristy                                %
 %                                   July 1992                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -91,7 +91,7 @@
 %
 %    o border_info:  Define the width and height of the border.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *BorderImage(const Image *image,
@@ -154,7 +154,7 @@ MagickExport Image *BorderImage(const Image *image,
 %
 %    o frame_info: Define the width and height of the frame and its bevels.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
@@ -282,115 +282,142 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
       ConvertRGBToCMYK(&shadow);
       ConvertRGBToCMYK(&trough);
     }
+  status=MagickTrue;
+  progress=0;
+  frame_view=AcquireCacheView(frame_image);
   height=(unsigned long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
     frame_info->inner_bevel);
-  frame_view=AcquireCacheView(frame_image);
-  progress=0;
-  q=SetCacheViewPixels(frame_view,0,0,frame_image->columns,height);
-  frame_indexes=GetCacheViewIndexes(frame_view);
-  if (q != (PixelPacket *) NULL)
+  if (height != 0)
     {
       /*
         Draw top of ornamental border.
       */
-      for (y=0; y < (long) frame_info->outer_bevel; y++)
-      {
-        for (x=0; x < (long) (frame_image->columns-y); x++)
+      q=QueueCacheViewAuthenticPixels(frame_view,0,0,frame_image->columns,
+        height,exception);
+      frame_indexes=GetCacheViewAuthenticIndexQueue(frame_view);
+      if (q != (PixelPacket *) NULL)
         {
-          if (x < y)
-            SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          else
-            SetPixelPacket(frame_image,&accentuate,q,frame_indexes);
-          q++;
-          frame_indexes++;
+          /*
+            Draw top of ornamental border.
+          */
+          for (y=0; y < (long) frame_info->outer_bevel; y++)
+          {
+            for (x=0; x < (long) (frame_image->columns-y); x++)
+            {
+              if (x < y)
+                SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              else
+                SetPixelPacket(frame_image,&accentuate,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for ( ; x < (long) frame_image->columns; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          for (y=0; y < (long) (frame_info->y-bevel_width); y++)
+          {
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=frame_image->columns-2*frame_info->outer_bevel;
+            for (x=0; x < (long) width; x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          for (y=0; y < (long) frame_info->inner_bevel; y++)
+          {
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) (frame_info->x-bevel_width); x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=image->columns+((unsigned long) frame_info->inner_bevel << 1)-
+              y;
+            for (x=0; x < (long) width; x++)
+            {
+              if (x < y)
+                SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              else
+                SetPixelPacket(frame_image,&trough,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=frame_info->width-frame_info->x-image->columns-bevel_width;
+            for (x=0; x < (long) width; x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          (void) SyncCacheViewAuthenticPixels(frame_view,exception);
         }
-        for ( ; x < (long) frame_image->columns; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      for (y=0; y < (long) (frame_info->y-bevel_width); y++)
-      {
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=frame_image->columns-2*frame_info->outer_bevel;
-        for (x=0; x < (long) width; x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      for (y=0; y < (long) frame_info->inner_bevel; y++)
-      {
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) (frame_info->x-bevel_width); x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=image->columns+((unsigned long) frame_info->inner_bevel << 1)-y;
-        for (x=0; x < (long) width; x++)
-        {
-          if (x < y)
-            SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          else
-            SetPixelPacket(frame_image,&trough,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=frame_info->width-frame_info->x-image->columns-bevel_width;
-        for (x=0; x < (long) width; x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      (void) SyncCacheView(frame_view);
     }
   /*
     Draw sides of ornamental border.
   */
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *frame_indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *q;
+
     /*
       Initialize scanline with matte color.
     */
-    q=SetCacheViewPixels(frame_view,0,frame_info->y+y,frame_image->columns,1);
+    if (status == MagickFalse)
+      continue;
+    q=QueueCacheViewAuthenticPixels(frame_view,0,frame_info->y+y,
+      frame_image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
-      break;
-    frame_indexes=GetCacheViewIndexes(frame_view);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    frame_indexes=GetCacheViewAuthenticIndexQueue(frame_view);
     for (x=0; x < (long) frame_info->outer_bevel; x++)
     {
       SetPixelPacket(frame_image,&highlight,q,frame_indexes);
@@ -437,13 +464,16 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
       q++;
       frame_indexes++;
     }
-    if (SyncCacheView(frame_view) == MagickFalse)
-      break;
+    if (SyncCacheViewAuthenticPixels(frame_view,exception) == MagickFalse)
+      status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,FrameImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -451,99 +481,105 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   }
   height=(unsigned long) (frame_info->inner_bevel+frame_info->height-
     frame_info->y-image->rows-bevel_width+frame_info->outer_bevel);
-  q=SetCacheViewPixels(frame_view,0,(long) (frame_image->rows-height),
-    frame_image->columns,height);
-  if (q != (PixelPacket *) NULL)
+  if (height != 0)
     {
       /*
         Draw bottom of ornamental border.
       */
-      frame_indexes=GetIndexes(frame_image);
-      for (y=frame_info->inner_bevel-1; y >= 0; y--)
-      {
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
+      q=QueueCacheViewAuthenticPixels(frame_view,0,(long) (frame_image->rows-
+        height),frame_image->columns,height,exception);
+      if (q != (PixelPacket *) NULL)
         {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
+          /*
+            Draw bottom of ornamental border.
+          */
+          frame_indexes=GetCacheViewAuthenticIndexQueue(frame_view);
+          for (y=frame_info->inner_bevel-1; y >= 0; y--)
+          {
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) (frame_info->x-bevel_width); x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < y; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
+            {
+              if (x >= (long) (image->columns+2*frame_info->inner_bevel-y))
+                SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              else
+                SetPixelPacket(frame_image,&accentuate,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=frame_info->width-frame_info->x-image->columns-bevel_width;
+            for (x=0; x < (long) width; x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          height=frame_info->height-frame_info->y-image->rows-bevel_width;
+          for (y=0; y < (long) height; y++)
+          {
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=frame_image->columns-2*frame_info->outer_bevel;
+            for (x=0; x < (long) width; x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          for (y=frame_info->outer_bevel-1; y >= 0; y--)
+          {
+            for (x=0; x < y; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for ( ; x < (long) frame_image->columns; x++)
+            {
+              if (x >= (long) (frame_image->columns-y))
+                SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              else
+                SetPixelPacket(frame_image,&trough,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          (void) SyncCacheViewAuthenticPixels(frame_view,exception);
         }
-        for (x=0; x < (long) (frame_info->x-bevel_width); x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < y; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
-        {
-          if (x >= (long) (image->columns+2*frame_info->inner_bevel-y))
-            SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          else
-            SetPixelPacket(frame_image,&accentuate,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=frame_info->width-frame_info->x-image->columns-bevel_width;
-        for (x=0; x < (long) width; x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      height=frame_info->height-frame_info->y-image->rows-bevel_width;
-      for (y=0; y < (long) height; y++)
-      {
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=frame_image->columns-2*frame_info->outer_bevel;
-        for (x=0; x < (long) width; x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      for (y=frame_info->outer_bevel-1; y >= 0; y--)
-      {
-        for (x=0; x < y; x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for ( ; x < (long) frame_image->columns; x++)
-        {
-          if (x >= (long) (frame_image->columns-y))
-            SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          else
-            SetPixelPacket(frame_image,&trough,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      (void) SyncCacheView(frame_view);
     }
   frame_view=DestroyCacheView(frame_view);
   x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
@@ -594,8 +630,12 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
 #define RaiseImageTag  "Raise/Image"
 #define TroughFactor  ScaleCharToQuantum(135)
 
+  ExceptionInfo
+    *exception;
+
   long
-    progress;
+    progress,
+    y;
 
   MagickBooleanType
     status;
@@ -604,9 +644,6 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     foreground,
     background;
 
-  long
-    y;
-
   ViewInfo
     *image_view;
 
@@ -633,7 +670,11 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
   */
   status=MagickTrue;
   progress=0;
+  exception=(&image->exception);
   image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) raise_info->height; y++)
   {
     register long
@@ -644,7 +685,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
 
     if (status == MagickFalse)
       continue;
-    q=GetCacheViewPixels(image_view,0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -686,7 +727,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
-    if (SyncCacheView(image_view) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -698,7 +739,10 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
           status=MagickFalse;
       }
   }
-  for ( ; y < (long) (image->rows-raise_info->height); y++)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
+  for (y=(long) raise_info->height; y < (long) (image->rows-raise_info->height); y++)
   {
     register long
       x;
@@ -708,7 +752,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
 
     if (status == MagickFalse)
       continue;
-    q=GetCacheViewPixels(image_view,0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -739,7 +783,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
-    if (SyncCacheView(image_view) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -751,7 +795,10 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
           status=MagickFalse;
       }
   }
-  for ( ; y < (long) image->rows; y++)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
+  for (y=(long) (image->rows-raise_info->height); y < (long) image->rows; y++)
   {
     register long
       x;
@@ -761,7 +808,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
 
     if (status == MagickFalse)
       continue;
-    q=GetCacheViewPixels(image_view,0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -800,13 +847,16 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
-    if (SyncCacheView(image_view) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,RaiseImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
