@@ -46,6 +46,7 @@
 #include "magick/annotate.h"
 #include "magick/artifact.h"
 #include "magick/blob.h"
+#include "magick/cache.h"
 #include "magick/client.h"
 #include "magick/coder.h"
 #include "magick/color.h"
@@ -125,6 +126,7 @@
 %      about the image.
 %
 */
+
 MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   const MagickBooleanType verbose)
 {
@@ -137,6 +139,9 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
     format[MaxTextExtent],
     key[MaxTextExtent];
 
+  ColorspaceType
+    colorspace;
+
   const char
     *artifact,
     *name,
@@ -247,7 +252,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
     Display verbose info about the image.
   */
   exception=AcquireExceptionInfo();
-  pixels=AcquireImagePixels(image,0,0,1,1,exception);
+  pixels=GetVirtualPixels(image,0,0,1,1,exception);
   exception=DestroyExceptionInfo(exception);
   ping=pixels == (const PixelPacket *) NULL ? MagickTrue : MagickFalse;
   type=GetImageType(image,&image->exception);
@@ -314,7 +319,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         (void) fprintf(file,"  Depth: %lu/%lu-bit\n",image->depth,depth);
       channel_statistics=GetImageChannelStatistics(image,&image->exception);
       (void) fprintf(file,"  Channel depth:\n");
-      switch (image->colorspace)
+      colorspace=image->colorspace;
+      if (IsGrayImage(image,&image->exception) != MagickFalse)
+        colorspace=GRAYColorspace;
+      switch (colorspace)
       {
         case RGBColorspace:
         default:
@@ -355,10 +363,12 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           break;
         }
       }
-      scale=QuantumRange/((unsigned long) QuantumRange >> ((unsigned long)
-        MAGICKCORE_QUANTUM_DEPTH-channel_statistics[AllChannels].depth));
+      scale=1;
+      if (image->depth <= MAGICKCORE_QUANTUM_DEPTH)
+        scale=QuantumRange/((unsigned long) QuantumRange >> ((unsigned long)
+          MAGICKCORE_QUANTUM_DEPTH-image->depth));
       (void) fprintf(file,"  Channel statistics:\n");
-      switch (image->colorspace)
+      switch (colorspace)
       {
         case RGBColorspace:
         default:
@@ -485,10 +495,10 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           indexes=(IndexPacket *) NULL;
           for (y=0; y < (long) image->rows; y++)
           {
-            p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+            p=GetVirtualPixels(image,0,y,image->columns,1,exception);
             if (p == (const PixelPacket *) NULL)
               break;
-            indexes=AcquireIndexes(image);
+            indexes=GetVirtualIndexQueue(image);
             for (x=0; x < (long) image->columns; x++)
             {
               if (p->opacity == (Quantum) TransparentOpacity)
@@ -782,7 +792,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
               }
           }
 #endif
-        if (LocaleCompare(name,"8bim") == 0)
+        if (LocaleCompare(name,"iptc") == 0)
           {
             char
               *attribute,
@@ -803,9 +813,6 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
               length,
               profile_length;
 
-            /*
-              Identify IPTC data.
-            */
             profile_length=GetStringInfoLength(profile);
             for (i=0; i < (long) profile_length; i+=(long) length)
             {
@@ -871,13 +878,13 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
                 case 219: tag="Custom Field 20"; break;
                 default: tag="unknown"; break;
               }
-              (void) fprintf(file,"    %s[%ld,%ld]: ",tag,dataset,record);
+              (void) fprintf(file,"      %s[%ld,%ld]: ",tag,dataset,record);
               length=(size_t) (GetStringInfoDatum(profile)[i++] << 8);
               length|=GetStringInfoDatum(profile)[i++];
               attribute=(char *) NULL;
               if (~length >= MaxTextExtent)
-                attribute=(char *) AcquireQuantumMemory(length+MaxTextExtent,
-                  sizeof(*attribute));
+                attribute=(char *) AcquireQuantumMemory(length+
+                  MaxTextExtent,sizeof(*attribute));
               if (attribute != (char *) NULL)
                 {
                   (void) CopyMagickString(attribute,(char *)
