@@ -8,7 +8,7 @@
 %                     L      A   A    Y    E      R R                         %
 %                     LLLLL  A   A    Y    EEEEE  R  R                        %
 %                                                                             %
-%                     ImageMagick Image Layering Methods                      %
+%                      MagickCore Image Layering Methods                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
@@ -16,7 +16,7 @@
 %                               January 2006                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,6 +39,7 @@
 */
 #include "magick/studio.h"
 #include "magick/artifact.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/composite.h"
@@ -93,6 +94,9 @@
 */
 static void ClearBounds(Image *image,RectangleInfo *bounds)
 {
+  ExceptionInfo
+    *exception;
+
   long
     y;
 
@@ -105,9 +109,10 @@ static void ClearBounds(Image *image,RectangleInfo *bounds)
   if ( bounds->x < 0 ) return;
   if (image->matte == MagickFalse)
     (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
+  exception=(&image->exception);
   for (y=0; y < (long) bounds->height; y++)
   {
-    q=GetImagePixels(image,bounds->x,bounds->y+y,bounds->width,1);
+    q=GetAuthenticPixels(image,bounds->x,bounds->y+y,bounds->width,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
     for (x=0; x < (long) bounds->width; x++)
@@ -115,7 +120,7 @@ static void ClearBounds(Image *image,RectangleInfo *bounds)
       q->opacity=(Quantum) TransparentOpacity;
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
 }
@@ -147,7 +152,7 @@ static void ClearBounds(Image *image,RectangleInfo *bounds)
 %
 %    o bounds: the area to be clear within the imag image
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 %  WARNING: no bounds checks are performed, except for the null or
 %  missed image, for images that don't change. in all other cases
@@ -171,10 +176,10 @@ static MagickBooleanType IsBoundsCleared(const Image *image1,
 
   for (y=0; y < (long) bounds->height; y++)
   {
-    p=AcquireImagePixels(image1,bounds->x,bounds->y+y,bounds->width,1,
-        exception);
-    q=AcquireImagePixels(image2,bounds->x,bounds->y+y,bounds->width,1,
-        exception);
+    p=GetVirtualPixels(image1,bounds->x,bounds->y+y,bounds->width,1,
+      exception);
+    q=GetVirtualPixels(image2,bounds->x,bounds->y+y,bounds->width,1,
+      exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
     for (x=0; x < (long) bounds->width; x++)
@@ -217,7 +222,7 @@ static MagickBooleanType IsBoundsCleared(const Image *image1,
 %
 %    o image: the image sequence.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
@@ -366,7 +371,7 @@ MagickExport Image *CoalesceImages(const Image *image,ExceptionInfo *exception)
 %
 %    o image: the image sequence.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *DisposeImages(const Image *image,ExceptionInfo *exception)
@@ -577,7 +582,7 @@ static MagickBooleanType ComparePixels(const ImageLayerMethod method,
 %
 %    o image1, image2: the two images to compare.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -587,14 +592,14 @@ static RectangleInfo CompareImageBounds(const Image *image1,const Image *image2,
   RectangleInfo
     bounds;
 
-  register IndexPacket
-    *indexes1,
-    *indexes2;
-
   MagickPixelPacket
     pixel1,
     pixel2;
 
+  register const IndexPacket
+    *indexes1,
+    *indexes2;
+
   register const PixelPacket
     *p,
     *q;
@@ -612,13 +617,13 @@ static RectangleInfo CompareImageBounds(const Image *image1,const Image *image2,
   GetMagickPixelPacket(image2,&pixel2);
   for (x=0; x < (long) image1->columns; x++)
   {
-    p=AcquireImagePixels(image1,x,0,1,image1->rows,exception);
-    q=AcquireImagePixels(image2,x,0,1,image2->rows,exception);
+    p=GetVirtualPixels(image1,x,0,1,image1->rows,exception);
+    q=GetVirtualPixels(image2,x,0,1,image2->rows,exception);
     if ((p == (const PixelPacket *) NULL) ||
         (q == (const PixelPacket *) NULL))
       break;
-    indexes1=GetIndexes(image1);
-    indexes2=GetIndexes(image2);
+    indexes1=GetVirtualIndexQueue(image1);
+    indexes2=GetVirtualIndexQueue(image2);
     for (y=0; y < (long) image1->rows; y++)
     {
       SetMagickPixelPacket(image1,p,indexes1+x,&pixel1);
@@ -645,13 +650,13 @@ static RectangleInfo CompareImageBounds(const Image *image1,const Image *image2,
   bounds.x=x;
   for (x=(long) image1->columns-1; x >= 0; x--)
   {
-    p=AcquireImagePixels(image1,x,0,1,image1->rows,exception);
-    q=AcquireImagePixels(image2,x,0,1,image2->rows,exception);
+    p=GetVirtualPixels(image1,x,0,1,image1->rows,exception);
+    q=GetVirtualPixels(image2,x,0,1,image2->rows,exception);
     if ((p == (const PixelPacket *) NULL) ||
         (q == (const PixelPacket *) NULL))
       break;
-    indexes1=GetIndexes(image1);
-    indexes2=GetIndexes(image2);
+    indexes1=GetVirtualIndexQueue(image1);
+    indexes2=GetVirtualIndexQueue(image2);
     for (y=0; y < (long) image1->rows; y++)
     {
       SetMagickPixelPacket(image1,p,indexes1+x,&pixel1);
@@ -667,13 +672,13 @@ static RectangleInfo CompareImageBounds(const Image *image1,const Image *image2,
   bounds.width=(unsigned long) (x-bounds.x+1);
   for (y=0; y < (long) image1->rows; y++)
   {
-    p=AcquireImagePixels(image1,0,y,image1->columns,1,exception);
-    q=AcquireImagePixels(image2,0,y,image2->columns,1,exception);
+    p=GetVirtualPixels(image1,0,y,image1->columns,1,exception);
+    q=GetVirtualPixels(image2,0,y,image2->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) ||
         (q == (const PixelPacket *) NULL))
       break;
-    indexes1=GetIndexes(image1);
-    indexes2=GetIndexes(image2);
+    indexes1=GetVirtualIndexQueue(image1);
+    indexes2=GetVirtualIndexQueue(image2);
     for (x=0; x < (long) image1->columns; x++)
     {
       SetMagickPixelPacket(image1,p,indexes1+x,&pixel1);
@@ -689,13 +694,13 @@ static RectangleInfo CompareImageBounds(const Image *image1,const Image *image2,
   bounds.y=y;
   for (y=(long) image1->rows-1; y >= 0; y--)
   {
-    p=AcquireImagePixels(image1,0,y,image1->columns,1,exception);
-    q=AcquireImagePixels(image2,0,y,image2->columns,1,exception);
+    p=GetVirtualPixels(image1,0,y,image1->columns,1,exception);
+    q=GetVirtualPixels(image2,0,y,image2->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) ||
         (q == (const PixelPacket *) NULL))
       break;
-    indexes1=GetIndexes(image1);
-    indexes2=GetIndexes(image2);
+    indexes1=GetVirtualIndexQueue(image1);
+    indexes2=GetVirtualIndexQueue(image2);
     for (x=0; x < (long) image1->columns; x++)
     {
       SetMagickPixelPacket(image1,p,indexes1+x,&pixel1);
@@ -746,7 +751,7 @@ static RectangleInfo CompareImageBounds(const Image *image1,const Image *image2,
 %    o method: the layers type to compare images with. Must be one of...
 %              CompareAnyLayer, CompareClearLayer, CompareOverlayLayer.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -883,7 +888,7 @@ MagickExport Image *CompareImageLayers(const Image *image,
 %
 %    o image: the image.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -927,7 +932,7 @@ MagickExport Image *DeconstructImages(const Image *images,
 %    o method: the layers type to optimize with. Must be one of...
 %             OptimizeImageLayer, or  OptimizePlusLayer
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 /*
@@ -1324,7 +1329,7 @@ static Image *OptimizeLayerFrames(const Image *image,
 %
 %    o image: the image.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *OptimizeImageLayers(const Image *image,
@@ -1357,7 +1362,7 @@ MagickExport Image *OptimizeImageLayers(const Image *image,
 %
 %    o image: the image.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *OptimizePlusImageLayers(const Image *image,
@@ -1394,7 +1399,7 @@ MagickExport Image *OptimizePlusImageLayers(const Image *image,
 %
 %    o image: the image sequence
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport void OptimizeImageTransparency(const Image *image,
@@ -1520,7 +1525,7 @@ MagickExport void OptimizeImageTransparency(const Image *image,
 %
 %    o images: the image list
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport void RemoveDuplicateLayers(Image **images,
@@ -1601,7 +1606,7 @@ MagickExport void RemoveDuplicateLayers(Image **images,
 %
 %    o images: the image list
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport void RemoveZeroDelayLayers(Image **images,
@@ -1686,7 +1691,7 @@ MagickExport void RemoveZeroDelayLayers(Image **images,
 %
 %    o compose, x_offset, y_offset:  arguments passed on to CompositeImages()
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static inline void CompositeCanvas(Image *destination,
@@ -1816,7 +1821,7 @@ MagickExport void CompositeLayers(Image *destination,
 %           WARNING: a new image is not returned, the original image
 %           sequence page data is modified instead.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *MergeImageLayers(Image *image,
