@@ -10,14 +10,14 @@
 %                  LLLLL   OOO    CCCC  A   A  LLLLL  EEEEE                   %
 %                                                                             %
 %                                                                             %
-%                      ImageMagick Image Locale Methods                       %
+%                      MagickCore Image Locale Methods                        %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -180,7 +180,7 @@ MagickExport LinkedListInfo *DestroyLocaleOptions(LinkedListInfo *messages)
 %
 %    o tag: the locale tag.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
@@ -228,7 +228,7 @@ MagickExport const LocaleInfo *GetLocaleInfo_(const char *tag,
 %    o number_messages:  This integer returns the number of locale messages in
 %    the list.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -325,7 +325,7 @@ MagickExport const LocaleInfo **GetLocaleInfoList(const char *pattern,
 %    o number_messages:  This integer returns the number of messages in the
 %      list.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -460,7 +460,7 @@ MagickExport const char *GetLocaleMessage(const char *tag)
 %
 %    o filename: the locale file tag.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport LinkedListInfo *GetLocaleOptions(const char *filename,
@@ -571,7 +571,7 @@ MagickExport const char *GetLocaleValue(const LocaleInfo *locale_info)
 %
 %  A description of each parameter follows.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
@@ -633,7 +633,7 @@ static MagickBooleanType InitializeLocaleList(ExceptionInfo *exception)
 %
 %    o file:  An pointer to a FILE.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
@@ -710,10 +710,33 @@ MagickExport MagickBooleanType ListLocaleInfo(FILE *file,
 %
 %    o depth: depth of <include /> statements.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
+static void ChopLocaleComponents(char *path,const unsigned long components)
+{
+  long
+    count;
+
+  register char
+    *p;
+
+  if (*path == '\0')
+    return;
+  p=path+strlen(path)-1;
+  if (*p == '/')
+    *p='\0';
+  for (count=0; (count < (long) components) && (p > path); p--)
+    if (*p == '/')
+      {
+        *p='\0';
+        count++;
+      }
+  if (count < (long) components)
+    *path='\0';
+}
+
 static void *DestroyLocaleNode(void *locale_info)
 {
   register LocaleInfo
@@ -729,107 +752,32 @@ static void *DestroyLocaleNode(void *locale_info)
   return(RelinquishMagickMemory(p));
 }
 
-static MagickBooleanType TraverseLocaleMap(const char *filename,
-  XMLTreeInfo **components,XMLTreeInfo *node,const unsigned long depth,
-  ExceptionInfo *exception)
-{
-  MagickBooleanType
-    status;
-
-  status=MagickTrue;
-  if (depth >= MaxRecursionDepth)
-    (void) ThrowMagickException(exception,GetMagickModule(),ConfigureError,
-      "IncludeElementNestedTooDeeply","`%s'",filename);
-  else
-    if (node != (XMLTreeInfo *) NULL)
-      {
-        XMLTreeInfo
-          *child,
-          *sibling;
-
-        components[depth]=node;
-        if (strcmp(GetXMLTreeTag(node),"message") == 0)
-          {
-            char
-              *message,
-              *tag;
-
-            const char
-              *attribute,
-              *content;
-
-            LocaleInfo
-              *locale_info;
-
-            register long
-              i;
-
-            locale_info=(LocaleInfo *) AcquireMagickMemory(
-              sizeof(*locale_info));
-            if (locale_info == (LocaleInfo *) NULL)
-              ThrowFatalException(ResourceLimitFatalError,
-                "MemoryAllocationFailed");
-            (void) ResetMagickMemory(locale_info,0,sizeof(*locale_info));
-            locale_info->path=ConstantString(filename);
-            locale_info->signature=MagickSignature;
-            tag=AcquireString((char *) NULL);
-            for (i=1; i < (long) depth; i++)
-            {
-              (void) ConcatenateString(&tag,GetXMLTreeTag(components[i]));
-              (void) ConcatenateString(&tag,"/");
-            }
-            attribute=GetXMLTreeAttribute(node,"name");
-            if (attribute != (const char *) NULL)
-              {
-                (void) ConcatenateString(&tag,attribute);
-                (void) ConcatenateString(&tag,"/");
-              }
-            locale_info->tag=ConstantString(tag);
-            tag=DestroyString(tag);
-            message=AcquireString((char *) NULL);
-            content=GetXMLTreeContent(node);
-            if (content != (const char *) NULL)
-              {
-                (void) ConcatenateString(&message,content);
-                StripString(message);
-              }
-            locale_info->message=ConstantString(message);
-            message=DestroyString(message);
-            status=AddValueToSplayTree(locale_list,locale_info->tag,
-              locale_info);
-            if (status == MagickFalse)
-              (void) ThrowMagickException(exception,GetMagickModule(),
-                ResourceLimitError,"MemoryAllocationFailed","`%s'",filename);
-          }
-        child=GetXMLTreeChild(node,(const char *) NULL);
-        if (child != (XMLTreeInfo *) NULL)
-          status=TraverseLocaleMap(filename,components,child,depth+1,exception);
-        sibling=GetXMLTreeOrdered(node);
-        if (sibling != (XMLTreeInfo *) NULL)
-          status=TraverseLocaleMap(filename,components,sibling,depth,exception);
-      }
-  return(status);
-}
-
 static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
   const char *locale,const unsigned long depth,ExceptionInfo *exception)
 {
+  char
+    keyword[MaxTextExtent],
+    message[MaxTextExtent],
+    tag[MaxTextExtent],
+    *token;
+
   const char
-    *attribute;
+    *q;
+
+  LocaleInfo
+    *locale_info;
 
   MagickBooleanType
     status;
 
-  XMLTreeInfo
-    *include,
-    **components,
-    *locale_map;
+  register char
+    *p;
 
   /*
-    Load the locale map file.
+    Read the locale configure file.
   */
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
-    "Loading locale map \"%s\" ...",filename);
+    "Loading locale configure file \"%s\" ...",filename);
   if (xml == (const char *) NULL)
     return(MagickFalse);
   if (locale_list == (SplayTreeInfo *) NULL)
@@ -843,61 +791,195 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
           return(MagickFalse);
         }
     }
-  locale_map=NewXMLTree(xml,exception);
-  if (locale_map == (XMLTreeInfo *) NULL)
-    return(MagickFalse);
-  include=GetXMLTreeChild(locale_map,"include");
-  while (include != (XMLTreeInfo *) NULL)
+  status=MagickTrue;
+  locale_info=(LocaleInfo *) NULL;
+  *tag='\0';
+  *message='\0';
+  *keyword='\0';
+  token=AcquireString((char *) xml);
+  for (q=(char *) xml; *q != '\0'; )
   {
     /*
-      Process include element.
+      Interpret XML.
     */
-    attribute=GetXMLTreeAttribute(include,"locale");
-    if ((attribute != (const char *) NULL) &&
-        (LocaleCompare(locale,attribute) != 0))
+    GetMagickToken(q,&q,token);
+    if (*token == '\0')
+      break;
+    (void) CopyMagickString(keyword,token,MaxTextExtent);
+    if (LocaleNCompare(keyword,"<!DOCTYPE",9) == 0)
       {
-        include=GetNextXMLTreeTag(include);
+        /*
+          Doctype element.
+        */
+        while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
+        {
+          GetMagickToken(q,&q,token);
+          while (isspace((int) ((unsigned char) *q)) != 0)
+            q++;
+        }
         continue;
       }
-    attribute=GetXMLTreeAttribute(include,"file");
-    if (attribute != (const char *) NULL)
+    if (LocaleNCompare(keyword,"<!--",4) == 0)
       {
-        if (depth > MaxRecursionDepth)
-          (void) ThrowMagickException(exception,GetMagickModule(),
-            ConfigureError,"IncludeElementNestedTooDeeply","`%s'",filename);
-        else
-          {
-            char
-              path[MaxTextExtent],
-              *xml;
-
-            GetPathComponent(filename,HeadPath,path);
-            if (*path != '\0')
-              (void) ConcatenateMagickString(path,DirectorySeparator,
-                MaxTextExtent);
-            (void) ConcatenateMagickString(path,attribute,MaxTextExtent);
-            xml=FileToString(path,~0,exception);
-            if (xml != (char *) NULL)
-              {
-                status=LoadLocaleList(xml,path,locale,depth+1,exception);
-                xml=DestroyString(xml);
-              }
-          }
+        /*
+          Comment element.
+        */
+        while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
+        {
+          GetMagickToken(q,&q,token);
+          while (isspace((int) ((unsigned char) *q)) != 0)
+            q++;
+        }
+        continue;
       }
-    include=GetNextXMLTreeTag(include);
+    if (LocaleCompare(keyword,"<include") == 0)
+      {
+        /*
+          Include element.
+        */
+        while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
+        {
+          (void) CopyMagickString(keyword,token,MaxTextExtent);
+          GetMagickToken(q,&q,token);
+          if (*token != '=')
+            continue;
+          GetMagickToken(q,&q,token);
+          if (LocaleCompare(keyword,"locale") == 0)
+            {
+              if (LocaleCompare(locale,token) != 0)
+                break;
+              continue;
+            }
+          if (LocaleCompare(keyword,"file") == 0)
+            {
+              if (depth > 200)
+                (void) ThrowMagickException(exception,GetMagickModule(),
+                  ConfigureError,"IncludeElementNestedTooDeeply","`%s'",token);
+              else
+                {
+                  char
+                    path[MaxTextExtent],
+                    *xml;
+
+                  *path='\0';
+                  GetPathComponent(filename,HeadPath,path);
+                  if (*path != '\0')
+                    (void) ConcatenateMagickString(path,DirectorySeparator,
+                      MaxTextExtent);
+                  if (*token == *DirectorySeparator)
+                    (void) CopyMagickString(path,token,MaxTextExtent);
+                  else
+                    (void) ConcatenateMagickString(path,token,MaxTextExtent);
+                  xml=FileToString(path,~0,exception);
+                  if (xml != (char *) NULL)
+                    {
+                      status=LoadLocaleList(xml,path,locale,depth+1,exception);
+                      xml=(char *) RelinquishMagickMemory(xml);
+                    }
+                }
+            }
+        }
+        continue;
+      }
+    if (LocaleCompare(keyword,"<locale") == 0)
+      {
+        /*
+          Locale element.
+        */
+        while ((*token != '>') && (*q != '\0'))
+        {
+          (void) CopyMagickString(keyword,token,MaxTextExtent);
+          GetMagickToken(q,&q,token);
+          if (*token != '=')
+            continue;
+          GetMagickToken(q,&q,token);
+        }
+        continue;
+      }
+    if (LocaleCompare(keyword,"</locale>") == 0)
+      {
+        ChopLocaleComponents(tag,1);
+        (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
+        continue;
+      }
+    if (LocaleCompare(keyword,"<localemap>") == 0)
+      continue;
+    if (LocaleCompare(keyword,"</localemap>") == 0)
+      continue;
+    if (LocaleCompare(keyword,"<message") == 0)
+      {
+        /*
+          Message element.
+        */
+        while ((*token != '>') && (*q != '\0'))
+        {
+          (void) CopyMagickString(keyword,token,MaxTextExtent);
+          GetMagickToken(q,&q,token);
+          if (*token != '=')
+            continue;
+          GetMagickToken(q,&q,token);
+          if (LocaleCompare(keyword,"name") == 0)
+            {
+              (void) ConcatenateMagickString(tag,token,MaxTextExtent);
+              (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
+            }
+        }
+        for (p=(char *) q; (*q != '<') && (*q != '\0'); q++);
+        while (isspace((int) ((unsigned char) *p)) != 0)
+          p++;
+        q--;
+        while ((isspace((int) ((unsigned char) *q)) != 0) && (q > p))
+          q--;
+        (void) CopyMagickString(message,p,(size_t) (q-p+2));
+        locale_info=(LocaleInfo *) AcquireMagickMemory(sizeof(*locale_info));
+        if (locale_info == (LocaleInfo *) NULL)
+          ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+        (void) ResetMagickMemory(locale_info,0,sizeof(*locale_info));
+        locale_info->path=ConstantString(filename);
+        locale_info->tag=ConstantString(tag);
+        locale_info->message=ConstantString(message);
+        locale_info->signature=MagickSignature;
+        status=AddValueToSplayTree(locale_list,locale_info->tag,locale_info);
+        if (status == MagickFalse)
+          (void) ThrowMagickException(exception,GetMagickModule(),
+            ResourceLimitError,"MemoryAllocationFailed","`%s'",
+            locale_info->tag);
+        (void) ConcatenateMagickString(tag,message,MaxTextExtent);
+        (void) ConcatenateMagickString(tag,"\n",MaxTextExtent);
+        q++;
+        continue;
+      }
+    if (LocaleCompare(keyword,"</message>") == 0)
+      {
+        ChopLocaleComponents(tag,2);
+        (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
+        continue;
+      }
+    if (*keyword == '<')
+      {
+        /*
+          Subpath element.
+        */
+        if (*(keyword+1) == '?')
+          continue;
+        if (*(keyword+1) == '/')
+          {
+            ChopLocaleComponents(tag,1);
+            if (*tag != '\0')
+              (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
+            continue;
+          }
+        token[strlen(token)-1]='\0';
+        (void) CopyMagickString(token,token+1,MaxTextExtent);
+        (void) ConcatenateMagickString(tag,token,MaxTextExtent);
+        (void) ConcatenateMagickString(tag,"/",MaxTextExtent);
+        continue;
+      }
+    GetMagickToken(q,(const char **) NULL,token);
+    if (*token != '=')
+      continue;
   }
-  components=(XMLTreeInfo **) AcquireQuantumMemory(MaxRecursionDepth,
-    sizeof(*components));
-  if (components == (XMLTreeInfo **) NULL)
-    {
-      locale_map=DestroyXMLTree(locale_map);
-      ThrowFileException(exception,ResourceLimitError,
-        "MemoryAllocationFailed",filename);
-      return(MagickFalse);
-    }
-  status=TraverseLocaleMap(filename,components,locale_map,0,exception);
-  components=(XMLTreeInfo **) RelinquishMagickMemory(components);
-  locale_map=DestroyXMLTree(locale_map);
+  token=(char *) RelinquishMagickMemory(token);
   return(status);
 }
 
@@ -926,7 +1008,7 @@ static MagickBooleanType LoadLocaleList(const char *xml,const char *filename,
 %
 %    o locale: the actual locale.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static MagickBooleanType LoadLocaleLists(const char *filename,
