@@ -10,14 +10,14 @@
 %               M   M   OOO   N   N    T    A   A   GGG   EEEEE               %
 %                                                                             %
 %                                                                             %
-%                ImageMagick Methods to Create Image Thumbnails               %
+%                MagickCore Methods to Create Image Thumbnails                %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -265,7 +265,7 @@ MagickExport void GetMontageInfo(const ImageInfo *image_info,
 %
 %    o images: Specifies a pointer to an array of Image structures.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -442,7 +442,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
     (void) ParseAbsoluteGeometry("0x0+0+0",&image->page);
     progress_monitor=SetImageProgressMonitor(image,(MagickProgressMonitor) NULL,
       image->client_data);
-    flags=ParseSizeGeometry(image,montage_info->geometry,&geometry);
+    flags=ParseSizeGeometry(image,montage_info->geometry,&geometry,exception);
     thumbnail=ThumbnailImage(image,geometry.width,geometry.height,exception);
     if (thumbnail == (Image *) NULL)
       break;
@@ -762,7 +762,7 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
         tile_image->gravity=image->gravity;
       (void) FormatMagickString(tile_geometry,MaxTextExtent,"%lux%lu+0+0",
         image->columns,image->rows);
-      flags=ParseGravityGeometry(tile_image,tile_geometry,&geometry);
+      flags=ParseGravityGeometry(tile_image,tile_geometry,&geometry,exception);
       x=(long) (geometry.x+border_width);
       y=(long) (geometry.y+border_width);
       if ((montage_info->frame != (char *) NULL) && (bevel_width != 0))
@@ -857,8 +857,8 @@ MagickExport Image *MontageImageList(const ImageInfo *image_info,
           if (status == MagickFalse)
             break;
         }
-      image=DestroyImage(image);
       image_list[tile]=DestroyImage(image_list[tile]);
+      image=DestroyImage(image);
       tiles++;
     }
     if ((i+1) < (long) images_per_page)
