@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -53,6 +53,7 @@ typedef enum
   MagickFontsOptions,
   MagickFormatOptions,
   MagickGravityOptions,
+  MagickImageListOptions,
   MagickIntentOptions,
   MagickInterlaceOptions,
   MagickInterpolateOptions,
@@ -69,7 +70,6 @@ typedef enum
   MagickMimeOptions,
   MagickModeOptions,
   MagickModuleOptions,
-  MagickMogrifyOptions,
   MagickNoiseOptions,
   MagickOrientationOptions,
   MagickPreviewOptions,
@@ -117,8 +117,7 @@ extern MagickExport MagickBooleanType
   DeleteImageOption(ImageInfo *,const char *),
   IsMagickOption(const char *),
   ListMagickOptions(FILE *,const MagickOption,ExceptionInfo *),
-  SetImageOption(ImageInfo *,const char *,const char *),
-  SyncImageOptions(const ImageInfo *,Image *);
+  SetImageOption(ImageInfo *,const char *,const char *);
 
 extern MagickExport void
   DestroyImageOptions(ImageInfo *),
