@@ -16,7 +16,7 @@
 %                                 July 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -55,6 +55,10 @@
 #include "magick/pixel-private.h"
 #include "magick/string_.h"
 
+static inline double MagickMax(const double x,const double y)
+{
+    return( x > y ? x : y);
+}
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -123,6 +127,9 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     } \
 }
 
+  ExceptionInfo
+    *exception;
+
   Image
     *floodplane_image;
 
@@ -143,12 +150,12 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   PixelPacket
     fill_color;
 
+  register const IndexPacket
+    *indexes;
+
   register const PixelPacket
     *p;
 
-  register IndexPacket
-    *indexes;
-
   register long
     x;
 
@@ -181,8 +188,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   /*
     Set floodfill state.
   */
-  floodplane_image=CloneImage(image,image->columns,image->rows,MagickTrue,
-    &image->exception);
+  floodplane_image=CloneImage(image,0,0,MagickTrue,&image->exception);
   if (floodplane_image == (Image *) NULL)
     return(MagickFalse);
   (void) SetImageAlphaChannel(floodplane_image,OpaqueAlphaChannel);
@@ -197,6 +203,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   /*
     Push initial segment on stack.
   */
+  exception=(&image->exception);
   x=x_offset;
   y=y_offset;
   start=0;
@@ -218,11 +225,12 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
     /*
       Recolor neighboring pixels.
     */
-    p=AcquireImagePixels(image,0,y,(unsigned long) (x1+1),1,&image->exception);
-    q=GetImagePixels(floodplane_image,0,y,(unsigned long) (x1+1),1);
+    p=GetVirtualPixels(image,0,y,(unsigned long) (x1+1),1,exception);
+    q=GetAuthenticPixels(floodplane_image,0,y,(unsigned long) (x1+1),1,
+      exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    indexes=GetIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     p+=x1;
     q+=x1;
     for (x=x1; x >= 0; x--)
@@ -236,7 +244,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       p--;
       q--;
     }
-    if (SyncImagePixels(floodplane_image) == MagickFalse)
+    if (SyncAuthenticPixels(floodplane_image,exception) == MagickFalse)
       break;
     skip=x >= x1 ? MagickTrue : MagickFalse;
     if (skip == MagickFalse)
@@ -252,13 +260,13 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
         {
           if (x < (long) image->columns)
             {
-              p=AcquireImagePixels(image,x,y,image->columns-x,1,
-                &image->exception);
-              q=GetImagePixels(floodplane_image,x,y,image->columns-x,1);
+              p=GetVirtualPixels(image,x,y,image->columns-x,1,exception);
+              q=GetAuthenticPixels(floodplane_image,x,y,image->columns-x,1,
+                exception);
               if ((p == (const PixelPacket *) NULL) ||
                   (q == (PixelPacket *) NULL))
                 break;
-              indexes=GetIndexes(image);
+              indexes=GetVirtualIndexQueue(image);
               for ( ; x < (long) image->columns; x++)
               {
                 if (q->opacity == (Quantum) TransparentOpacity)
@@ -270,7 +278,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
                 p++;
                 q++;
               }
-              if (SyncImagePixels(floodplane_image) == MagickFalse)
+              if (SyncAuthenticPixels(floodplane_image,exception) == MagickFalse)
                 break;
             }
           PushSegmentStack(y,start,x-1,offset);
@@ -281,12 +289,12 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       x++;
       if (x <= x2)
         {
-          p=AcquireImagePixels(image,x,y,(unsigned long) (x2-x+1),1,
-            &image->exception);
-          q=GetImagePixels(floodplane_image,x,y,(unsigned long) (x2-x+1),1);
+          p=GetVirtualPixels(image,x,y,(unsigned long) (x2-x+1),1,exception);
+          q=GetAuthenticPixels(floodplane_image,x,y,(unsigned long) (x2-x+1),1,
+            exception);
           if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
             break;
-          indexes=GetIndexes(image);
+          indexes=GetVirtualIndexQueue(image);
           for ( ; x <= x2; x++)
           {
             if (q->opacity == (Quantum) TransparentOpacity)
@@ -303,20 +311,22 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
   }
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *indexes;
+
     /*
       Tile fill color onto floodplane.
     */
-    p=AcquireImagePixels(floodplane_image,0,y,image->columns,1,
-      &image->exception);
-    q=GetImagePixels(image,0,y,image->columns,1);
+    p=GetVirtualPixels(floodplane_image,0,y,image->columns,1,exception);
+    q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    indexes=GetIndexes(image);
+    indexes=GetAuthenticIndexQueue(image);
     for (x=0; x < (long) image->columns; x++)
     {
       if (p->opacity != OpaqueOpacity)
         {
-          fill_color=GetFillColor(draw_info,x,y);
+          (void) GetFillColor(draw_info,x,y,&fill_color);
           SetMagickPixelPacket(image,&fill_color,(IndexPacket *) NULL,&fill);
           if (image->colorspace == CMYKColorspace)
             ConvertRGBToCMYK(&fill);
@@ -335,7 +345,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
       p++;
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
+    if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
   }
   segment_stack=(SegmentInfo *) RelinquishMagickMemory(segment_stack);
@@ -348,6 +358,115 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
++     G r a d i e n t I m a g e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GradientImage() applies a continuously smooth color transitions along a
+%  vector from one color to another.
+%
+%  Note, the interface of this method will change in the future to support
+%  more than one transistion.
+%
+%  The format of the GradientImage method is:
+%
+%      MagickBooleanType GradientImage(Image *image,const GradientType type,
+%        const SpreadMethod method,const PixelPacket *start_color,
+%        const PixelPacket *stop_color)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o type: the gradient type: linear or radial.
+%
+%    o spread: the gradient spread meathod: pad, reflect, or repeat.
+%
+%    o start_color: the start color.
+%
+%    o stop_color: the stop color.
+%
+% This provides a good example of making use of the DrawGradientImage
+% function and the gradient structure in draw_info.
+*/
+MagickExport MagickBooleanType GradientImage(Image *image,
+  const GradientType type,const SpreadMethod method,
+  const PixelPacket *start_color,const PixelPacket *stop_color)
+{
+  DrawInfo
+    *draw_info;
+
+  GradientInfo
+    *gradient;
+
+  MagickBooleanType
+    status;
+
+  register long
+    i;
+
+  /*
+    Set gradient start-stop end points.
+  */
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(start_color != (const PixelPacket *) NULL);
+  assert(stop_color != (const PixelPacket *) NULL);
+  draw_info=AcquireDrawInfo();
+  gradient=(&draw_info->gradient);
+  gradient->type=type;
+  gradient->bounding_box.width=image->columns;
+  gradient->bounding_box.height=image->rows;
+  gradient->gradient_vector.x2=(double) image->columns-1.0;
+  gradient->gradient_vector.y2=(double) image->rows-1.0;
+  if ((type == LinearGradient) && (gradient->gradient_vector.y2 != 0.0))
+    gradient->gradient_vector.x2=0.0;
+  gradient->center.x=(double) gradient->gradient_vector.x2/2.0;
+  gradient->center.y=(double) gradient->gradient_vector.y2/2.0;
+  gradient->radius=MagickMax(gradient->center.x,gradient->center.y);
+  gradient->spread=method;
+  /*
+    Define the gradient to fill between the stops.
+  */
+  gradient->number_stops=2;
+  gradient->stops=(StopInfo *) AcquireQuantumMemory(gradient->number_stops,
+    sizeof(*gradient->stops));
+  if (gradient->stops == (StopInfo *) NULL)
+    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      image->filename);
+  (void) ResetMagickMemory(gradient->stops,0,gradient->number_stops*
+    sizeof(*gradient->stops));
+  for (i=0; i < (long) gradient->number_stops; i++)
+    GetMagickPixelPacket(image,&gradient->stops[i].color);
+  SetMagickPixelPacket(image,start_color,(IndexPacket *) NULL,
+    &gradient->stops[0].color);
+  gradient->stops[0].offset=0.0;
+  SetMagickPixelPacket(image,stop_color,(IndexPacket *) NULL,
+    &gradient->stops[1].color);
+  gradient->stops[1].offset=1.0;
+  /*
+    Draw a gradient on the image.
+  */
+  status=DrawGradientImage(image,draw_info);
+  draw_info=DestroyDrawInfo(draw_info);
+  if ((start_color->opacity == OpaqueOpacity) &&
+      (stop_color->opacity == OpaqueOpacity))
+    image->matte=MagickFalse;
+  if ((IsGrayPixel(start_color) != MagickFalse) &&
+      (IsGrayPixel(stop_color) != MagickFalse))
+    image->type=GrayscaleType;
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %     O i l P a i n t I m a g e                                               %
 %                                                                             %
 %                                                                             %
@@ -369,7 +488,7 @@ MagickExport MagickBooleanType FloodfillPaintImage(Image *image,
 %
 %    o radius: the radius of the circular neighborhood.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -379,7 +498,7 @@ static unsigned long **DestroyHistogramThreadSet(unsigned long **histogram)
     i;
 
   assert(histogram != (unsigned long **) NULL);
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+  for (i=0; i < (long) GetPixelCacheMaximumThreads(); i++)
     if (histogram[i] != (unsigned long *) NULL)
       histogram[i]=(unsigned long *) RelinquishMagickMemory(histogram[i]);
   return((unsigned long **) RelinquishMagickMemory(histogram));
@@ -391,15 +510,16 @@ static unsigned long **AcquireHistogramThreadSet(const size_t count)
     i;
 
   unsigned long
-    **histogram;
+    **histogram,
+    number_threads;
 
-  histogram=(unsigned long **) AcquireQuantumMemory(
-    GetCacheViewMaximumThreads(),sizeof(*histogram));
+  number_threads=GetPixelCacheMaximumThreads();
+  histogram=(unsigned long **) AcquireQuantumMemory(number_threads,
+    sizeof(*histogram));
   if (histogram == (unsigned long **) NULL)
     return((unsigned long **) NULL);
-  (void) ResetMagickMemory(histogram,0,GetCacheViewMaximumThreads()*
-    sizeof(*histogram));
-  for (i=0; i < (long) GetCacheViewMaximumThreads(); i++)
+  (void) ResetMagickMemory(histogram,0,number_threads*sizeof(*histogram));
+  for (i=0; i < (long) number_threads; i++)
   {
     histogram[i]=(unsigned long *) AcquireQuantumMemory(count,
       sizeof(**histogram));
@@ -422,16 +542,16 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
     progress,
     y;
 
+  MagickBooleanType
+    status;
+
   unsigned long
-    **histogram,
+    **histograms,
     width;
 
   ViewInfo
-    **image_view,
-    **paint_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view,
+    *paint_view;
 
   /*
     Initialize painted image attributes.
@@ -454,8 +574,8 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       paint_image=DestroyImage(paint_image);
       return((Image *) NULL);
     }
-  histogram=AcquireHistogramThreadSet(NumberPaintBins);
-  if (histogram == (unsigned long **) NULL)
+  histograms=AcquireHistogramThreadSet(NumberPaintBins);
+  if (histograms == (unsigned long **) NULL)
     {
       paint_image=DestroyImage(paint_image);
       ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
@@ -465,9 +585,11 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  paint_view=AcquireCacheViewThreadSet(paint_image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+  paint_view=AcquireCacheView(paint_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     const IndexPacket
@@ -480,25 +602,28 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       *p;
 
     register long
-      id,
       x;
 
     register PixelPacket
       *q;
 
+    register unsigned long
+      *histogram;
+
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],-((long) width/2L),y-(long)
-      (width/2L),image->columns+width,width,exception);
-    q=SetCacheViewPixels(paint_view[id],0,y,paint_image->columns,1);
+    p=GetCacheViewVirtualPixels(image_view,-((long) width/2L),y-(long) (width/
+      2L),image->columns+width,width,exception);
+    q=QueueCacheViewAuthenticPixels(paint_view,0,y,paint_image->columns,1,
+      exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=AcquireCacheViewIndexes(image_view[id]);
-    paint_indexes=GetCacheViewIndexes(paint_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    paint_indexes=GetCacheViewAuthenticIndexQueue(paint_view);
+    histogram=histograms[GetPixelCacheThreadId()];
     for (x=0; x < (long) image->columns; x++)
     {
       long
@@ -519,18 +644,17 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       i=0;
       j=0;
       count=0;
-      (void) ResetMagickMemory(histogram[id],0,NumberPaintBins*
-        sizeof(**histogram));
+      (void) ResetMagickMemory(histogram,0,NumberPaintBins*sizeof(*histogram));
       for (v=0; v < (long) width; v++)
       {
         for (u=0; u < (long) width; u++)
         {
           k=(long) ScaleQuantumToChar(PixelIntensityToQuantum(p+u+i));
-          histogram[id][k]++;
-          if (histogram[id][k] > count)
+          histogram[k]++;
+          if (histogram[k] > count)
             {
               j=i+u;
-              count=histogram[id][k];
+              count=histogram[k];
             }
         }
         i+=image->columns+width;
@@ -541,22 +665,24 @@ MagickExport Image *OilPaintImage(const Image *image,const double radius,
       p++;
       q++;
     }
-    if (SyncCacheView(paint_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(paint_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,OilPaintImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  paint_view=DestroyCacheViewThreadSet(paint_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
-  histogram=DestroyHistogramThreadSet(histogram);
+  paint_view=DestroyCacheView(paint_view);
+  image_view=DestroyCacheView(image_view);
+  histograms=DestroyHistogramThreadSet(histograms);
   if (status == MagickFalse)
     paint_image=DestroyImage(paint_image);
   return(paint_image);
@@ -618,18 +744,21 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
 {
 #define OpaquePaintImageTag  "Opaque/Image"
 
+  ExceptionInfo
+    *exception;
+
   long
     progress,
     y;
 
+  MagickBooleanType
+    status;
+
   MagickPixelPacket
     zero;
 
   ViewInfo
-    **image_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -644,9 +773,12 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
   */
   status=MagickTrue;
   progress=0;
+  exception=(&image->exception);
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     MagickPixelPacket
@@ -656,7 +788,6 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
       *indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
@@ -664,14 +795,13 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     pixel=zero;
     for (x=0; x < (long) image->columns; x++)
     {
@@ -692,21 +822,23 @@ MagickExport MagickBooleanType OpaquePaintImageChannel(Image *image,
         }
       q++;
     }
-    if (SyncCacheView(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,OpaquePaintImageTag,progress++,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -753,18 +885,21 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
 {
 #define TransparentPaintImageTag  "Transparent/Image"
 
+  ExceptionInfo
+    *exception;
+
   long
     progress,
     y;
 
+  MagickBooleanType
+    status;
+
   MagickPixelPacket
     zero;
 
   ViewInfo
-    **image_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -780,9 +915,12 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
   */
   status=MagickTrue;
   progress=0;
+  exception=(&image->exception);
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     MagickPixelPacket
@@ -792,7 +930,6 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
       *indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
@@ -800,14 +937,13 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     pixel=zero;
     for (x=0; x < (long) image->columns; x++)
     {
@@ -816,20 +952,159 @@ MagickExport MagickBooleanType TransparentPaintImage(Image *image,
         q->opacity=opacity;
       q++;
     }
-    if (SyncCacheView(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,TransparentPaintImageTag,progress++,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
+  return(status);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%     T r a n s p a r e n t P a i n t I m a g e C h r o m a                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TransparentPaintImageChroma() changes the opacity value associated with any
+%  pixel that matches color to the value defined by opacity.
+%
+%  As there is one fuzz value for the all the channels, the
+%  TransparentPaintImage() API is not suitable for the operations like chroma,
+%  where the tolerance for similarity of two color component (RGB) can be
+%  different, Thus we define this method take two target pixels (one
+%  low and one hight) and all the pixels of an image which are lying between
+%  these two pixels are made transparent.
+%
+%  The format of the TransparentPaintImage method is:
+%
+%      MagickBooleanType TransparentPaintImage(Image *image,
+%        const MagickPixelPacket *low,const MagickPixelPacket *hight,
+%        const Quantum opacity,const MagickBooleanType invert)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o low: the low target color.
+%
+%    o high: the high target color.
+%
+%    o opacity: the replacement opacity value.
+%
+%    o invert: paint any pixel that does not match the target color.
+%
+*/
+MagickExport MagickBooleanType TransparentPaintImageChroma(Image *image,
+  const MagickPixelPacket *low,const MagickPixelPacket *high,
+  const Quantum opacity,const MagickBooleanType invert)
+{
+#define TransparentPaintImageTag  "Transparent/Image"
+
+  ExceptionInfo
+    *exception;
+
+  long
+    progress,
+    y;
+
+  MagickBooleanType
+    status;
+
+  ViewInfo
+    *image_view;
+
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  assert(high != (MagickPixelPacket *) NULL);
+  assert(low != (MagickPixelPacket *) NULL);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+    return(MagickFalse);
+  if (image->matte == MagickFalse)
+    (void) SetImageAlphaChannel(image,ResetAlphaChannel);
+  /*
+    Make image color transparent.
+  */
+  status=MagickTrue;
+  progress=0;
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
+  for (y=0; y < (long) image->rows; y++)
+  {
+    MagickBooleanType
+      match;
+
+    MagickPixelPacket
+      pixel;
+
+    register IndexPacket
+      *indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
+    if (q == (PixelPacket *) NULL)
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    GetMagickPixelPacket(image,&pixel);
+    for (x=0; x < (long) image->columns; x++)
+    {
+      SetMagickPixelPacket(image,q,indexes+x,&pixel);
+      match=((pixel.red >= low->red) && (pixel.red <= high->red) &&
+        (pixel.green >= low->green) && (pixel.green <= high->green) &&
+        (pixel.blue  >= low->blue) && (pixel.blue <= high->blue)) ?
+        MagickTrue : MagickFalse;
+      if (match != invert)
+        q->opacity=opacity;
+      q++;
+    }
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,TransparentPaintImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
+  }
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
