@@ -10,14 +10,14 @@
 %               P      R  R    OOO   F      IIIII  LLLLL  EEEEE               %
 %                                                                             %
 %                                                                             %
-%                      ImageMagick Image Profile Methods                      %
+%                       MagickCore Image Profile Methods                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/configure.h"
 #include "magick/exception.h"
@@ -55,6 +56,7 @@
 #include "magick/quantum-private.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
+#include "magick/token.h"
 #include "magick/utility.h"
 #if defined(MAGICKCORE_LCMS_DELEGATE)
 #if defined(MAGICKCORE_HAVE_LCMS_LCMS_H)
@@ -726,12 +728,46 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
   assert(name != (const char *) NULL);
   if ((datum == (const void *) NULL) || (length == 0))
     {
-      if (*name == '*')
+      char
+        **arguments,
+        *names;
+
+      int
+        number_arguments;
+
+      register long
+        i;
+
+      /*
+        Delete image profile(s).
+      */
+      names=ConstantString(name);
+      (void) SubstituteString(&names,","," ");
+      arguments=StringToArgv(names,&number_arguments);
+      names=DestroyString(names);
+      if (arguments == (char **) NULL)
+        return(MagickTrue);
+      ResetImageProfileIterator(image);
+      for (name=GetNextImageProfile(image); name != (const char *) NULL; )
+      {
+        for (i=1; i < number_arguments; i++)
         {
-          DestroyImageProfiles(image);
-          return(MagickTrue);
+          if ((*arguments[i] == '!') &&
+              (LocaleCompare(name,arguments[i]+1) == 0))
+            break;
+          if (GlobExpression(name,arguments[i],MagickTrue) != MagickFalse)
+            {
+              (void) DeleteImageProfile(image,name);
+              ResetImageProfileIterator(image);
+              break;
+            }
         }
-      return(DeleteImageProfile(image,name));
+        name=GetNextImageProfile(image);
+      }
+      for (i=0; i < number_arguments; i++)
+        arguments[i]=DestroyString(arguments[i]);
+      arguments=(char **) RelinquishMagickMemory(arguments);
+      return(MagickTrue);
     }
   /*
     Add a ICC, IPTC, or generic profile to the image.
@@ -790,6 +826,9 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             source_type,
             target_type;
 
+          ExceptionInfo
+            *exception;
+
           IndexPacket
             *indexes;
 
@@ -937,10 +976,11 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               break;
             }
           }
+          exception=(&image->exception);
           if (((source_colorspace == UndefinedColorspace) ||
                (target_colorspace == UndefinedColorspace)) ||
               ((source_colorspace == GRAYColorspace) &&
-               (!IsGrayImage(image,&image->exception))) ||
+               (!IsGrayImage(image,exception))) ||
               ((source_colorspace == CMYKColorspace) &&
                (image->colorspace != CMYKColorspace)) ||
               ((source_colorspace != GRAYColorspace) &&
@@ -990,11 +1030,11 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             image->colorspace=target_colorspace;
           for (y=0; y < (long) image->rows; y++)
           {
-            q=GetImagePixels(image,0,y,image->columns,1);
+            q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
             if (q == (PixelPacket *) NULL)
               break;
             p=source_pixels;
-            indexes=GetIndexes(image);
+            indexes=GetAuthenticIndexQueue(image);
             for (x=0; x < (long) image->columns; x++)
             {
               *p++=ScaleQuantumToShort(q->red);
@@ -1031,7 +1071,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                 }
               q++;
             }
-            if (SyncImagePixels(image) == MagickFalse)
+            if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
           }
           image->colorspace=target_colorspace;
@@ -1261,7 +1301,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
 
   datum=GetStringInfoDatum(resource_block);
   length=GetStringInfoLength(resource_block);
-  for (p=datum; p < (datum+length-7); )
+  for (p=datum; p < (datum+length-16); )
   {
     if (LocaleNCompare((char *) p,"8BIM",4) != 0)
       break;
@@ -1292,6 +1332,18 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         image->y_resolution=(double) resolution;
         break;
       }
+      case 0x0404:
+      {
+        /*
+          IPTC Profile
+        */
+        profile=AcquireStringInfo(count);
+        SetStringInfoDatum(profile,p);
+        (void) SetImageProfile(image,"iptc",profile);
+        profile=DestroyStringInfo(profile);
+        p+=count;
+        break;
+      }
       case 0x040c:
       {
         /*
@@ -1308,6 +1360,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
         (void) SetImageProfile(image,"icc",profile);
+        profile=DestroyStringInfo(profile);
         p+=count;
         break;
       }
@@ -1319,6 +1372,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
         (void) SetImageProfile(image,"exif",profile);
+        profile=DestroyStringInfo(profile);
         p+=count;
         break;
       }
@@ -1330,6 +1384,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
         (void) SetImageProfile(image,"xmp",profile);
+        profile=DestroyStringInfo(profile);
         p+=count;
         break;
       }
@@ -1487,14 +1542,14 @@ static inline void WriteProfileLong(const EndianType endian,
       buffer[1]=(unsigned char) (value >> 16);
       buffer[2]=(unsigned char) (value >> 8);
       buffer[3]=(unsigned char) value;
-      (void) memcpy(p,buffer,4);
+      (void) CopyMagickMemory(p,buffer,4);
       return;
     }
   buffer[0]=(unsigned char) value;
   buffer[1]=(unsigned char) (value >> 8);
   buffer[2]=(unsigned char) (value >> 16);
   buffer[3]=(unsigned char) (value >> 24);
-  (void) memcpy(p,buffer,4);
+  (void) CopyMagickMemory(p,buffer,4);
 }
 
 static void WriteProfileShort(const EndianType endian,
@@ -1507,12 +1562,12 @@ static void WriteProfileShort(const EndianType endian,
     {
       buffer[0]=(unsigned char) (value >> 8);
       buffer[1]=(unsigned char) value;
-      (void) memcpy(p,buffer,2);
+      (void) CopyMagickMemory(p,buffer,2);
       return;
     }
   buffer[0]=(unsigned char) value;
   buffer[1]=(unsigned char) (value >> 8);
-  (void) memcpy(p,buffer,2);
+  (void) CopyMagickMemory(p,buffer,2);
 }
 
 MagickExport MagickBooleanType SyncImageProfiles(Image *image)
@@ -1668,12 +1723,14 @@ MagickExport MagickBooleanType SyncImageProfiles(Image *image)
         {
           (void) WriteProfileLong(endian,(unsigned long)
             (image->y_resolution+0.5),p);
+          (void) WriteProfileLong(endian,1UL,p+4);
           break;
         }
         case 0x0112:
         {
           (void) WriteProfileShort(endian,(unsigned short)
             image->orientation,p);
+          (void) WriteProfileLong(endian,1UL,p+4);
           break;
         }
         case 0x0128:
