@@ -15,7 +15,7 @@
 %                   E       X X   P      O   O  R R      T                    %
 %                   EEEEE  X   X  P       OOO   R  R     T                    %
 %                                                                             %
-%                      Methods to Export Quantum Pixels                       %
+%                 MagickCore Methods to Export Quantum Pixels                 %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
@@ -89,13 +89,16 @@
 %
 %  The format of the ExportQuantumPixels method is:
 %
-%      size_t ExportQuantumPixels(Image *image,const QuantumInfo *quantum_info,
-%        const QuantumType quantum_type,unsigned char *pixels)
+%      size_t ExportQuantumPixels(const Image *image,const ViewInfo *cache_view,
+%        const QuantumInfo *quantum_info,const QuantumType quantum_type,
+%        unsigned char *pixels,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image: the image.
 %
+%    o cache_view: the image cache view.
+%
 %    o quantum_info: the quantum info.
 %
 %    o quantum_type: Declare which pixel components to transfer (RGB, RGBA,
@@ -103,6 +106,8 @@
 %
 %    o pixels:  The components are transferred to this buffer.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 
 static inline unsigned char *PopDoublePixel(const QuantumState *quantum_state,
@@ -220,9 +225,9 @@ static inline unsigned char *PopQuantumLongPixel(QuantumState *quantum_state,
   return(pixels);
 }
 
-MagickExport size_t ExportQuantumPixels(Image *image,
-  const QuantumInfo *quantum_info,const QuantumType quantum_type,
-  unsigned char *pixels)
+MagickExport size_t ExportQuantumPixels(const Image *image,
+  const ViewInfo *cache_view,const QuantumInfo *quantum_info,
+  const QuantumType quantum_type,unsigned char *pixels,ExceptionInfo *exception)
 {
   EndianType
     endian;
@@ -264,10 +269,16 @@ MagickExport size_t ExportQuantumPixels(Image *image,
   assert(quantum_info != (QuantumInfo *) NULL);
   assert(quantum_info->signature == MagickSignature);
   if (pixels == (unsigned char *) NULL)
-    pixels=quantum_info->pixels;
-  number_pixels=GetPixelCacheExtent(image);
-  x=0;
-  p=GetPixels(image);
+    pixels=GetQuantumPixels(quantum_info);
+  number_pixels=GetImageExtent(image);
+  p=GetVirtualPixelQueue(image);
+  indexes=GetVirtualIndexQueue(image);
+  if (cache_view != (ViewInfo *) NULL)
+    {
+      number_pixels=GetCacheViewExtent(cache_view);
+      p=GetCacheViewVirtualPixelQueue(cache_view);
+      indexes=GetCacheViewVirtualIndexQueue(cache_view);
+    }
   if (quantum_info->alpha_type == AssociatedQuantumAlpha)
     {
       register PixelPacket
@@ -276,7 +287,9 @@ MagickExport size_t ExportQuantumPixels(Image *image,
       /*
         Associate alpha.
       */
-      q=GetPixels(image);
+      q=GetAuthenticPixelQueue(image);
+      if (cache_view != (ViewInfo *) NULL)
+        q=GetAuthenticPixelQueue(image);
       for (x=0; x < (long) image->columns; x++)
       {
         alpha=QuantumScale*((double) QuantumRange-q->opacity);
@@ -286,7 +299,7 @@ MagickExport size_t ExportQuantumPixels(Image *image,
         q++;
       }
     }
-  indexes=GetIndexes(image);
+  x=0;
   q=pixels;
   InitializeQuantumState(quantum_info,image->endian,&quantum_state);
   extent=GetQuantumExtent(image,quantum_info,quantum_type);
@@ -297,8 +310,8 @@ MagickExport size_t ExportQuantumPixels(Image *image,
     {
       if (image->storage_class != PseudoClass)
         {
-          (void) ThrowMagickException(&image->exception,GetMagickModule(),
-            ImageError,"ColormappedImageRequired","`%s'",image->filename);
+          (void) ThrowMagickException(exception,GetMagickModule(),ImageError,
+            "ColormappedImageRequired","`%s'",image->filename);
           return(extent);
         }
       switch (quantum_info->depth)
@@ -428,8 +441,8 @@ MagickExport size_t ExportQuantumPixels(Image *image,
     {
       if (image->storage_class != PseudoClass)
         {
-          (void) ThrowMagickException(&image->exception,GetMagickModule(),
-            ImageError,"ColormappedImageRequired","`%s'",image->filename);
+          (void) ThrowMagickException(exception,GetMagickModule(),ImageError,
+            "ColormappedImageRequired","`%s'",image->filename);
           return(extent);
         }
       switch (quantum_info->depth)
@@ -1328,12 +1341,96 @@ MagickExport size_t ExportQuantumPixels(Image *image,
       }
       break;
     }
+    case OpacityQuantum:
+    {
+      switch (quantum_info->depth)
+      {
+        case 8:
+        {
+          register unsigned char
+            pixel;
+
+          for (x=0; x < (long) number_pixels; x++)
+          {
+            pixel=ScaleQuantumToChar(p->opacity);
+            q=PopCharPixel(pixel,q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+        case 16:
+        {
+          register unsigned short
+            pixel;
+
+          for (x=0; x < (long) number_pixels; x++)
+          {
+            pixel=ScaleQuantumToShort(p->opacity);
+            q=PopShortPixel(endian,pixel,q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+        case 32:
+        {
+          register unsigned long
+            pixel;
+
+          if (quantum_info->format == FloatingPointQuantumFormat)
+            {
+              for (x=0; x < (long) number_pixels; x++)
+              {
+                q=PopFloatPixel(&quantum_state,(float) p->opacity,q);
+                p++;
+                q+=quantum_info->pad;
+              }
+              break;
+            }
+          for (x=0; x < (long) number_pixels; x++)
+          {
+            pixel=ScaleQuantumToLong(p->opacity);
+            q=PopLongPixel(endian,pixel,q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+        case 64:
+        {
+          if (quantum_info->format == FloatingPointQuantumFormat)
+            {
+              for (x=0; x < (long) number_pixels; x++)
+              {
+                q=PopDoublePixel(&quantum_state,(double) p->opacity,q);
+                p++;
+                q+=quantum_info->pad;
+              }
+              break;
+            }
+        }
+        default:
+        {
+          scale=GetQuantumScale(image->depth);
+          for (x=0; x < (long) number_pixels; x++)
+          {
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              p->opacity,image->depth,scale),q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+      }
+      break;
+    }
     case BlackQuantum:
     {
       if (image->colorspace != CMYKColorspace)
         {
-          (void) ThrowMagickException(&image->exception,GetMagickModule(),
-            ImageError,"ColorSeparatedImageRequired","`%s'",image->filename);
+          (void) ThrowMagickException(exception,GetMagickModule(),ImageError,
+            "ColorSeparatedImageRequired","`%s'",image->filename);
           return(extent);
         }
       switch (quantum_info->depth)
@@ -1914,8 +2011,8 @@ MagickExport size_t ExportQuantumPixels(Image *image,
     {
       if (image->colorspace != CMYKColorspace)
         {
-          (void) ThrowMagickException(&image->exception,GetMagickModule(),
-            ImageError,"ColorSeparatedImageRequired","`%s'",image->filename);
+          (void) ThrowMagickException(exception,GetMagickModule(),ImageError,
+            "ColorSeparatedImageRequired","`%s'",image->filename);
           return(extent);
         }
       switch (quantum_info->depth)
@@ -2034,8 +2131,8 @@ MagickExport size_t ExportQuantumPixels(Image *image,
     {
       if (image->colorspace != CMYKColorspace)
         {
-          (void) ThrowMagickException(&image->exception,GetMagickModule(),
-            ImageError,"ColorSeparatedImageRequired","`%s'",image->filename);
+          (void) ThrowMagickException(exception,GetMagickModule(),ImageError,
+            "ColorSeparatedImageRequired","`%s'",image->filename);
           return(extent);
         }
       switch (quantum_info->depth)
