@@ -10,14 +10,14 @@
 %               SSSSS  EEEEE   GGGG  M   M  EEEEE  N   N    T                 %
 %                                                                             %
 %                                                                             %
-%        Methods to Segment an Image with Thresholding Fuzzy c-Means          %
+%    MagickCore Methods to Segment an Image with Thresholding Fuzzy c-Means   %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                April 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -83,6 +83,7 @@
 */
 
 #include "magick/studio.h"
+#include "magick/cache.h"
 #include "magick/color.h"
 #include "magick/colorspace.h"
 #include "magick/exception.h"
@@ -246,50 +247,37 @@ static MagickBooleanType Classify(Image *image,short **extrema,
     *last_cluster,
     *next_cluster;
 
+  ExceptionInfo
+    *exception;
+
   ExtentPacket
     blue,
     green,
     red;
 
-  IndexPacket
-    index;
-
   long
     count,
-    j,
-    k,
+    progress,
     y;
 
   MagickBooleanType
-    proceed;
+    status;
 
   MagickRealType
-    distance_squared,
-    *free_squares,
-    local_minima,
-    numerator,
-    ratio,
-    sum;
-
-  register const PixelPacket
-    *p;
-
-  register IndexPacket
-    *indexes;
+    *free_squares;
 
   register long
-    i,
-    x;
+    i;
 
   register MagickRealType
     *squares;
 
-  register PixelPacket
-    *q;
-
   unsigned long
     number_clusters;
 
+  ViewInfo
+    *image_view;
+
   /*
     Form clusters.
   */
@@ -356,10 +344,20 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   /*
     Count the pixels for each cluster.
   */
+  status=MagickTrue;
   count=0;
+  progress=0;
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    register const PixelPacket
+      *p;
+
+    register long
+      x;
+
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     for (x=0; x < (long) image->columns; x++)
@@ -391,10 +389,21 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           }
       p++;
     }
-    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
-    if (proceed == MagickFalse)
-      break;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,SegmentImageTag,progress++,
+          2*image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
   }
+  image_view=DestroyCacheView(image_view);
   /*
     Remove clusters that do not meet minimum cluster threshold.
   */
@@ -435,6 +444,8 @@ static MagickBooleanType Classify(Image *image,short **extrema,
       */
       (void) fprintf(stdout,"Fuzzy C-means Statistics\n");
       (void) fprintf(stdout,"===================\n\n");
+      (void) fprintf(stdout,"\tCluster Threshold = %g\n",cluster_threshold);
+      (void) fprintf(stdout,"\tWeighting Exponent = %g\n",weighting_exponent);
       (void) fprintf(stdout,"\tTotal Number of Clusters = %lu\n\n",
         number_clusters);
       /*
@@ -490,31 +501,55 @@ static MagickBooleanType Classify(Image *image,short **extrema,
   if (AcquireImageColormap(image,number_clusters) == MagickFalse)
     ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
       image->filename);
-  cluster=head;
-  for (i=0; i < (long) number_clusters; i++)
+  i=0;
+  for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
   {
     image->colormap[i].red=ScaleCharToQuantum((unsigned char)
       (cluster->red.center+0.5));
     image->colormap[i].green=ScaleCharToQuantum((unsigned char)
       (cluster->green.center+0.5));
     image->colormap[i].blue=ScaleCharToQuantum((unsigned char)
-       (cluster->blue.center+0.5));
-    cluster=cluster->next;
+      (cluster->blue.center+0.5));
+    i++;
   }
   /*
     Do course grain classes.
   */
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
-    q=GetImagePixels(image,0,y,image->columns,1);
+    Cluster
+      *cluster;
+
+    register const PixelPacket
+      *p;
+
+    register IndexPacket
+      *indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *q;
+
+    if (status == MagickFalse)
+      continue;
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
-      break;
-    indexes=GetIndexes(image);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (long) image->columns; x++)
     {
-      index=(IndexPacket) 0;
-      cluster=head;
-      for (i=0; i < (long) number_clusters; i++)
+      indexes[x]=(IndexPacket) 0;
+      for (cluster=head; cluster != (Cluster *) NULL; cluster=cluster->next)
       {
         if (((long) ScaleQuantumToChar(q->red) >=
              (cluster->red.left-SafeMargin)) &&
@@ -532,13 +567,23 @@ static MagickBooleanType Classify(Image *image,short **extrema,
             /*
               Classify this pixel.
             */
-            index=(IndexPacket) cluster->id;
+            indexes[x]=(IndexPacket) cluster->id;
             break;
           }
-        cluster=cluster->next;
       }
       if (cluster == (Cluster *) NULL)
         {
+          MagickRealType
+            distance_squared,
+            local_minima,
+            numerator,
+            ratio,
+            sum;
+
+          register long
+            j,
+            k;
+
           /*
             Compute fuzzy membership.
           */
@@ -572,20 +617,30 @@ static MagickBooleanType Classify(Image *image,short **extrema,
                   Classify this pixel.
                 */
                 local_minima=1.0/sum;
-                index=(IndexPacket) j;
+                indexes[x]=(IndexPacket) j;
               }
           }
         }
-      indexes[x]=index;
-      *q=image->colormap[(long) index];
       q++;
     }
-    if (SyncImagePixels(image) == MagickFalse)
-      break;
-    proceed=SetImageProgress(image,SegmentImageTag,y,2*image->rows);
-    if (proceed == MagickFalse)
-      break;
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+        #pragma omp critical
+#endif
+        proceed=SetImageProgress(image,SegmentImageTag,progress++,
+          2*image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
   }
+  image_view=DestroyCacheView(image_view);
+  status&=SyncImage(image);
   /*
     Relinquish resources.
   */
@@ -858,9 +913,9 @@ static void DerivativeHistogram(const MagickRealType *histogram,
 %
 %  The format of the GetImageDynamicThreshold method is:
 %
-%      MagickPixelPacket GetImageDynamicThreshold(const Image *image,
+%      MagickBooleanType GetImageDynamicThreshold(const Image *image,
 %        const double cluster_threshold,const double smooth_threshold,
-%        ExceptionInfo *)
+%        MagickPixelPacket *pixel,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -874,12 +929,14 @@ static void DerivativeHistogram(const MagickRealType *histogram,
 %      derivative of the histogram.  As the value is increased, you can expect a
 %      smoother second derivative.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o pixel: return the dynamic threshold here.
+%
+%    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
+MagickExport MagickBooleanType GetImageDynamicThreshold(const Image *image,
   const double cluster_threshold,const double smooth_threshold,
-  ExceptionInfo *exception)
+  MagickPixelPacket *pixel,ExceptionInfo *exception)
 {
   Cluster
     *background,
@@ -902,9 +959,6 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   MagickBooleanType
     proceed;
 
-  MagickPixelPacket
-    pixel;
-
   MagickRealType
     threshold;
 
@@ -925,7 +979,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
+  GetMagickPixelPacket(image,pixel);
   for (i=0; i < MaxDimension; i++)
   {
     histogram[i]=(long *) AcquireQuantumMemory(256UL,sizeof(**histogram));
@@ -939,7 +993,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
         }
         (void) ThrowMagickException(exception,GetMagickModule(),
           ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
-        return(pixel);
+        return(MagickFalse);
       }
   }
   /*
@@ -987,7 +1041,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
             (void) ThrowMagickException(exception,GetMagickModule(),
               ResourceLimitError,"MemoryAllocationFailed","`%s'",
               image->filename);
-            return(pixel);
+            return(MagickFalse);
           }
         /*
           Initialize a new class.
@@ -1010,7 +1064,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
         {
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
-          return(pixel);
+          return(MagickFalse);
         }
       /*
         Initialize a new class.
@@ -1028,7 +1082,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
   count=0;
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     for (x=0; x < (long) image->columns; x++)
@@ -1118,13 +1172,13 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
       }
     }
   threshold=(background->red.center+object->red.center)/2.0;
-  pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+  pixel->red=(MagickRealType) ScaleCharToQuantum((unsigned char)
     (threshold+0.5));
   threshold=(background->green.center+object->green.center)/2.0;
-  pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+  pixel->green=(MagickRealType) ScaleCharToQuantum((unsigned char)
     (threshold+0.5));
   threshold=(background->blue.center+object->blue.center)/2.0;
-  pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+  pixel->blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
     (threshold+0.5));
   /*
     Relinquish resources.
@@ -1139,7 +1193,7 @@ MagickExport MagickPixelPacket GetImageDynamicThreshold(const Image *image,
     extrema[i]=(short *) RelinquishMagickMemory(extrema[i]);
     histogram[i]=(long *) RelinquishMagickMemory(histogram[i]);
   }
-  return(pixel);
+  return(MagickTrue);
 }
 
 /*
@@ -1192,7 +1246,7 @@ static void InitializeHistogram(const Image *image,long **histogram,
   }
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
     for (x=0; x < (long) image->columns; x++)
