@@ -10,14 +10,14 @@
 %                      SSSSS  H   H  EEEEE  A   A   R  R                      %
 %                                                                             %
 %                                                                             %
-%            Methods to Shear or Rotate an Image by an Arbitrary Angle        %
+%    MagickCore Methods to Shear or Rotate an Image by an Arbitrary Angle     %
 %                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
 %                                  July 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -100,7 +100,7 @@
 %
 %    o affine_matrix: the affine matrix.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *AffineTransformImage(const Image *image,
@@ -159,7 +159,7 @@ MagickExport Image *AffineTransformImage(const Image *image,
 %
 %    o x_shear, y_shear, width, height: Defines a region of the image to crop.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 static inline void CropToFitImage(Image **image,const MagickRealType x_shear,
@@ -258,7 +258,7 @@ static inline void CropToFitImage(Image **image,const MagickRealType x_shear,
 %
 %    o threshold: separate background from foreground.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -382,13 +382,15 @@ static RadonInfo *AcquireRadonInfo(const Image *image,const unsigned long width,
       status=AcquireMagickResource(MemoryResource,radon_info->length);
       if (status != MagickFalse)
         {
-          radon_info->cells=(unsigned short *) MapBlob(-1,IOMode,0,(size_t)
+          radon_info->mapped=MagickFalse;
+          radon_info->cells=(unsigned short *) AcquireMagickMemory((size_t)
             radon_info->length);
-          if (radon_info->cells != (unsigned short *) NULL)
-            radon_info->mapped=MagickTrue;
-          else
-            radon_info->cells=(unsigned short *) AcquireMagickMemory((size_t)
-              radon_info->length);
+          if (radon_info->cells == (unsigned short *) NULL)
+            {
+              radon_info->mapped=MagickTrue;
+              radon_info->cells=(unsigned short *) MapBlob(-1,IOMode,0,(size_t)
+                radon_info->length);
+            }
           if (radon_info->cells == (unsigned short *) NULL)
             RelinquishMagickResource(MemoryResource,radon_info->length);
         }
@@ -443,7 +445,9 @@ static inline ssize_t ReadRadonCell(const RadonInfo *radon_info,
     count;
 
 #if !defined(MAGICKCORE_HAVE_PPREAD)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp critical
+#endif
   {
     i=(-1);
     if (MagickSeek(radon_info->file,offset,SEEK_SET) >= 0)
@@ -484,8 +488,10 @@ static inline ssize_t WriteRadonCell(const RadonInfo *radon_info,
   ssize_t
     count;
 
-#if !defined(MAGICKCORE_HAVE_PPWRITE)
+#if !defined(MAGICKCORE_HAVE_PWRITE)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp critical
+#endif
   {
     if (MagickSeek(radon_info->file,offset,SEEK_SET) >= 0)
       {
@@ -493,7 +499,7 @@ static inline ssize_t WriteRadonCell(const RadonInfo *radon_info,
         count=0;
         for (i=0; i < (ssize_t) length; i+=count)
         {
-#if !defined(MAGICKCORE_HAVE_PPWRITE)
+#if !defined(MAGICKCORE_HAVE_PWRITE)
           count=write(radon_info->file,buffer+i,MagickMin(length-i,(size_t)
             SSIZE_MAX));
 #else
@@ -509,7 +515,7 @@ static inline ssize_t WriteRadonCell(const RadonInfo *radon_info,
               break;
             }
         }
-#if !defined(MAGICKCORE_HAVE_PPWRITE)
+#if !defined(MAGICKCORE_HAVE_PWRITE)
       }
   }
 #endif
@@ -619,7 +625,9 @@ static void RadonProjection(RadonInfo *source_cells,
     p=q;
     q=swap;
   }
-  #pragma omp parallel for schedule(static,64)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for
+#endif
   for (x=0; x < (long) p->width; x++)
   {
     register long
@@ -647,6 +655,9 @@ static MagickBooleanType RadonTransform(const Image *image,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   RadonInfo
     *destination_cells,
     *source_cells;
@@ -665,10 +676,7 @@ static MagickBooleanType RadonTransform(const Image *image,
     bits[256];
 
   ViewInfo
-    **image_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view;
 
   for (width=1; width < ((image->columns+7)/8); width<<=1) ;
   source_cells=AcquireRadonInfo(image,width,image->rows,exception);
@@ -696,8 +704,10 @@ static MagickBooleanType RadonTransform(const Image *image,
     bits[i]=(unsigned short) count;
   }
   status=MagickTrue;
-  image_view=AcquireCacheViewThreadSet(image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     register const PixelPacket
@@ -705,7 +715,6 @@ static MagickBooleanType RadonTransform(const Image *image,
 
     register long
       i,
-      id,
       x;
 
     unsigned long
@@ -714,8 +723,7 @@ static MagickBooleanType RadonTransform(const Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -748,7 +756,9 @@ static MagickBooleanType RadonTransform(const Image *image,
   }
   RadonProjection(source_cells,destination_cells,-1,projection);
   (void) ResetRadonCells(source_cells);
-  #pragma omp parallel for schedule(static,64)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     register const PixelPacket
@@ -756,7 +766,6 @@ static MagickBooleanType RadonTransform(const Image *image,
 
     register long
       i,
-      id,
       x;
 
     unsigned long
@@ -765,8 +774,7 @@ static MagickBooleanType RadonTransform(const Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -798,7 +806,7 @@ static MagickBooleanType RadonTransform(const Image *image,
       }
   }
   RadonProjection(source_cells,destination_cells,1,projection);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   destination_cells=DestroyRadonInfo(destination_cells);
   source_cells=DestroyRadonInfo(source_cells);
   return(MagickTrue);
@@ -837,7 +845,7 @@ static void GetImageBackgroundColor(Image *image,const long offset,
 
     if ((y >= offset) && (y < ((long) image->rows-offset)))
       continue;
-    p=AcquireCacheViewPixels(image_view,0,y,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       continue;
     for (x=0; x < (long) image->columns; x++)
@@ -1001,15 +1009,15 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
     progress,
     y;
 
+  MagickBooleanType
+    status;
+
   RectangleInfo
     page;
 
   ViewInfo
-    **image_view,
-    **rotate_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view,
+    *rotate_view;
 
   /*
     Initialize rotated image attributes.
@@ -1030,8 +1038,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  rotate_view=AcquireCacheViewThreadSet(rotate_image);
+  image_view=AcquireCacheView(image);
+  rotate_view=AcquireCacheView(rotate_image);
   switch (rotations)
   {
     case 0:
@@ -1039,9 +1047,14 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       /*
         Rotate 0 degrees.
       */
-      #pragma omp parallel for schedule(static,64)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
       for (y=0; y < (long) image->rows; y++)
       {
+        MagickBooleanType
+          sync;
+
         register const IndexPacket
           *indexes;
 
@@ -1051,38 +1064,39 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
         register IndexPacket
           *rotate_indexes;
 
-        register long
-          id;
-
         register PixelPacket
           *q;
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
-        p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
-        q=SetCacheViewPixels(rotate_view[id],0,y,rotate_image->columns,1);
+        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,
+          exception);
+        q=QueueCacheViewAuthenticPixels(rotate_view,0,y,
+          rotate_image->columns,1,exception);
         if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
           {
             status=MagickFalse;
             continue;
           }
-        indexes=AcquireCacheViewIndexes(image_view[id]);
+        indexes=GetCacheViewVirtualIndexQueue(image_view);
         (void) CopyMagickMemory(q,p,image->columns*sizeof(*p));
         if (indexes != (IndexPacket *) NULL)
           {
-            rotate_indexes=GetCacheViewIndexes(rotate_view[id]);
+            rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
             (void) CopyMagickMemory(rotate_indexes,indexes,image->columns*
               sizeof(*indexes));
           }
-        if (SyncCacheView(rotate_view[id]) == MagickFalse)
+        sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
+        if (sync == MagickFalse)
           status=MagickFalse;
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
           {
             MagickBooleanType
               proceed;
 
-            #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
             proceed=SetImageProgress(image,RotateImageTag,progress++,
               image->rows);
             if (proceed == MagickFalse)
@@ -1099,18 +1113,21 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       /*
         Rotate 90 degrees.
       */
-      #pragma omp parallel for
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
       for (tile_y=0; tile_y < (long) image->rows; tile_y+=TileHeight)
       {
         register long
-          id,
           tile_x;
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
         for (tile_x=0; tile_x < (long) image->columns; tile_x+=TileWidth)
         {
+          MagickBooleanType
+            sync;
+
           register const IndexPacket
             *indexes;
 
@@ -1120,6 +1137,9 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           register IndexPacket
             *rotate_indexes;
 
+          register long
+            y;
+
           register PixelPacket
             *q;
 
@@ -1135,17 +1155,14 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           if ((tile_y+TileHeight) > (long) image->rows)
             tile_height=(unsigned long) (TileHeight-(tile_y+TileHeight-
               image->rows));
-          p=AcquireCacheViewPixels(image_view[id],tile_x,tile_y,tile_width,
+          p=GetCacheViewVirtualPixels(image_view,tile_x,tile_y,tile_width,
             tile_height,exception);
-          q=SetCacheViewPixels(rotate_view[id],(long) rotate_image->columns-
-            (tile_y+tile_height),tile_x,tile_height,tile_width);
-          if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+          if (p == (const PixelPacket *) NULL)
             {
               status=MagickFalse;
-              continue;
+              break;
             }
-          indexes=AcquireCacheViewIndexes(image_view[id]);
-          rotate_indexes=GetCacheViewIndexes(rotate_view[id]);
+          indexes=GetCacheViewVirtualIndexQueue(image_view);
           for (y=0; y < (long) tile_width; y++)
           {
             register const PixelPacket
@@ -1154,13 +1171,23 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
             register long
               x;
 
+            q=QueueCacheViewAuthenticPixels(rotate_view,
+              rotate_image->columns-(tile_y+tile_height),y+tile_x,tile_height,
+              1,exception);
+            if (q == (PixelPacket *) NULL)
+              {
+                status=MagickFalse;
+                break;
+              }
             tile_pixels=p+(tile_height-1)*tile_width+y;
             for (x=0; x < (long) tile_height; x++)
             {
               *q++=(*tile_pixels);
               tile_pixels-=tile_width;
             }
-            if (indexes != (IndexPacket *) NULL)
+            rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
+            if ((indexes != (IndexPacket *) NULL) &&
+                (rotate_indexes != (IndexPacket *) NULL))
               {
                 register const IndexPacket
                   *tile_indexes;
@@ -1172,16 +1199,19 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
                   tile_indexes-=tile_width;
                 }
               }
+            sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
+            if (sync == MagickFalse)
+              status=MagickFalse;
           }
-          if (SyncCacheView(rotate_view[id]) == MagickFalse)
-            status=MagickFalse;
         }
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
           {
             MagickBooleanType
               proceed;
 
-            #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
             proceed=SetImageProgress(image,RotateImageTag,progress+=TileHeight,
               image->rows);
             if (proceed == MagickFalse)
@@ -1199,9 +1229,14 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       /*
         Rotate 180 degrees.
       */
-      #pragma omp parallel for schedule(static,64)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
       for (y=0; y < (long) image->rows; y++)
       {
+        MagickBooleanType
+          sync;
+
         register const IndexPacket
           *indexes;
 
@@ -1212,7 +1247,6 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           *rotate_indexes;
 
         register long
-          id,
           x;
 
         register PixelPacket
@@ -1220,33 +1254,36 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
-        p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
-        q=SetCacheViewPixels(rotate_view[id],0,(long) (image->rows-y-1),
-          image->columns,1);
+        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,
+          exception);
+        q=QueueCacheViewAuthenticPixels(rotate_view,0,(long) (image->rows-
+          y-1),image->columns,1,exception);
         if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
           {
             status=MagickFalse;
             continue;
           }
-        indexes=AcquireCacheViewIndexes(image_view[id]);
+        indexes=GetCacheViewVirtualIndexQueue(image_view);
         q+=image->columns;
         for (x=0; x < (long) image->columns; x++)
           *--q=(*p++);
         if (indexes != (IndexPacket *) NULL)
           {
-            rotate_indexes=GetCacheViewIndexes(rotate_view[id]);
+            rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
             for (x=0; x < (long) image->columns; x++)
               rotate_indexes[image->columns-x-1]=indexes[x];
           }
-        if (SyncCacheView(rotate_view[id]) == MagickFalse)
+        sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
+        if (sync == MagickFalse)
           status=MagickFalse;
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
           {
             MagickBooleanType
               proceed;
 
-            #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
             proceed=SetImageProgress(image,RotateImageTag,progress++,
               image->rows);
             if (proceed == MagickFalse)
@@ -1267,18 +1304,21 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       /*
         Rotate 270 degrees.
       */
-      #pragma omp parallel for
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
       for (tile_y=0; tile_y < (long) image->rows; tile_y+=TileHeight)
       {
         register long
-          id,
           tile_x;
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
         for (tile_x=0; tile_x < (long) image->columns; tile_x+=TileWidth)
         {
+          MagickBooleanType
+            sync;
+
           register const IndexPacket
             *indexes;
 
@@ -1288,6 +1328,9 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           register IndexPacket
             *rotate_indexes;
 
+          register long
+            y;
+
           register PixelPacket
             *q;
 
@@ -1303,17 +1346,14 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           if ((tile_y+TileHeight) > (long) image->rows)
             tile_height=(unsigned long) (TileHeight-(tile_y+TileHeight-
               image->rows));
-          p=AcquireCacheViewPixels(image_view[id],tile_x,tile_y,tile_width,
+          p=GetCacheViewVirtualPixels(image_view,tile_x,tile_y,tile_width,
             tile_height,exception);
-          q=SetCacheViewPixels(rotate_view[id],tile_y,(long) rotate_image->rows-
-            (tile_x+tile_width),tile_height,tile_width);
-          if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+          if (p == (const PixelPacket *) NULL)
             {
               status=MagickFalse;
               break;
             }
-          indexes=AcquireCacheViewIndexes(image_view[id]);
-          rotate_indexes=GetCacheViewIndexes(rotate_view[id]);
+          indexes=GetCacheViewVirtualIndexQueue(image_view);
           for (y=0; y < (long) tile_width; y++)
           {
             register const PixelPacket
@@ -1322,13 +1362,22 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
             register long
               x;
 
+            q=QueueCacheViewAuthenticPixels(rotate_view,tile_y,(long)
+              y+rotate_image->rows-(tile_x+tile_width),tile_height,1,exception);
+            if (q == (PixelPacket *) NULL)
+              {
+                status=MagickFalse;
+                break;
+              }
             tile_pixels=p+(tile_width-1)-y;
             for (x=0; x < (long) tile_height; x++)
             {
               *q++=(*tile_pixels);
               tile_pixels+=tile_width;
             }
-            if (indexes != (IndexPacket *) NULL)
+            rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
+            if ((indexes != (IndexPacket *) NULL) &&
+                (rotate_indexes != (IndexPacket *) NULL))
               {
                 register const IndexPacket
                   *tile_indexes;
@@ -1340,16 +1389,19 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
                   tile_indexes+=tile_width;
                 }
               }
+            sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
+            if (sync == MagickFalse)
+              status=MagickFalse;
           }
-          if (SyncCacheView(rotate_view[id]) == MagickFalse)
-            status=MagickFalse;
         }
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
           {
             MagickBooleanType
               proceed;
 
-            #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
             proceed=SetImageProgress(image,RotateImageTag,progress+=TileHeight,
               image->rows);
             if (proceed == MagickFalse)
@@ -1363,8 +1415,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       break;
     }
   }
-  rotate_view=DestroyCacheViewThreadSet(rotate_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  rotate_view=DestroyCacheView(rotate_view);
+  image_view=DestroyCacheView(image_view);
   rotate_image->type=image->type;
   rotate_image->page=page;
   if (status == MagickFalse)
@@ -1456,8 +1508,14 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
 {
 #define XShearImageTag  "XShear/Image"
 
-  enum {LEFT, RIGHT}
-    direction;
+  typedef enum
+  {
+    LEFT,
+    RIGHT
+  } ShearDirection;
+
+  ExceptionInfo
+    *exception;
 
   long
     progress,
@@ -1486,7 +1544,11 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
   */
   status=MagickTrue;
   progress=0;
+  exception=(&image->exception);
   image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
+#endif
   for (y=0; y < (long) height; y++)
   {
     IndexPacket
@@ -1512,13 +1574,19 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
       *p,
       *q;
 
-    p=GetCacheViewPixels(image_view,0,y_offset+y,image->columns,1);
+    ShearDirection
+      direction;
+
+    if (status == MagickFalse)
+      continue;
+    p=GetCacheViewAuthenticPixels(image_view,0,y_offset+y,image->columns,1,
+      exception);
     if (p == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     p+=x_offset;
     indexes+=x_offset;
     displacement=degrees*(MagickRealType) (y-height/2.0);
@@ -1601,19 +1669,20 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
         break;
       }
     }
-    if (SyncCacheView(image_view) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,XShearImageTag,progress++,height);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
-    if (status == MagickFalse)
-      break;
   }
   image_view=DestroyCacheView(image_view);
   return(status);
@@ -1659,8 +1728,14 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
 {
 #define YShearImageTag  "YShear/Image"
 
-  enum {UP, DOWN}
-    direction;
+  typedef enum
+  {
+    UP,
+    DOWN
+  } ShearDirection;
+
+  ExceptionInfo
+    *exception;
 
   long
     progress,
@@ -1689,7 +1764,11 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
   */
   status=MagickTrue;
   progress=0;
+  exception=(&image->exception);
   image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress, status)
+#endif
   for (x=0; x < (long) width; x++)
   {
     IndexPacket
@@ -1715,13 +1794,19 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
       *p,
       *q;
 
-    p=GetCacheViewPixels(image_view,x_offset+x,0,1,image->rows);
+    ShearDirection
+      direction;
+
+    if (status == MagickFalse)
+      continue;
+    p=GetCacheViewAuthenticPixels(image_view,x_offset+x,0,1,image->rows,
+      exception);
     if (p == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     p+=y_offset;
     indexes+=y_offset;
     displacement=degrees*(MagickRealType) (x-width/2.0);
@@ -1804,19 +1889,20 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
         break;
       }
     }
-    if (SyncCacheView(image_view) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,YShearImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
-    if (status == MagickFalse)
-      break;
   }
   image_view=DestroyCacheView(image_view);
   return(status);
@@ -1858,7 +1944,7 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
 %
 %    o degrees: Specifies the number of degrees to rotate the image.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *RotateImage(const Image *image,const double degrees,
@@ -1931,8 +2017,8 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
       height=image->columns;
     }
   y_width=width+(long) (fabs(shear.x)*height+0.5);
-  x_offset=(long) (width+((fabs(shear.y)*height+0.5)-width)/2.0+0.5);
-  y_offset=(long) (height+((fabs(shear.y)*y_width+0.5)-height)/2.0+0.5);
+  x_offset=(long) (width+((fabs(shear.y)*height)-width)/2.0+0.5);
+  y_offset=(long) (height+((fabs(shear.y)*y_width)-height)/2.0+0.5);
   /*
     Surround image with a border.
   */
@@ -1997,7 +2083,7 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
 %
 %    o x_shear, y_shear: Specifies the number of degrees to shear the image.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *ShearImage(const Image *image,const double x_shear,
@@ -2037,7 +2123,7 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   if (integral_image == (Image *) NULL)
     ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
   shear.x=(-tan(DegreesToRadians(x_shear)));
-  shear.y=tan(DegreesToRadians(y_shear));
+  shear.y=(-tan(DegreesToRadians(y_shear)));
   if ((shear.x == 0.0) && (shear.y == 0.0))
     return(integral_image);
   if (SetImageStorageClass(integral_image,DirectClass) == MagickFalse)
@@ -2052,10 +2138,9 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
     Compute image size.
   */
   y_width=image->columns+(long) (fabs(shear.x)*image->rows+0.5);
-  x_offset=(long) (image->columns+((fabs(shear.x)*image->rows)-
-    image->columns)/2.0+0.5);
-  y_offset=(long) (image->rows+((fabs(shear.y)*y_width+0.5)-image->rows)/2.0+
-    0.5);
+  x_offset=(long) (image->columns+((fabs(shear.x)*image->rows)-image->columns)/
+    2.0+0.5);
+  y_offset=(long) (image->rows+((fabs(shear.y)*y_width)-image->rows)/2.0+0.5);
   /*
     Surround image with border.
   */
