@@ -9,14 +9,14 @@
 %        SSSSS  IIIII   GGG   N   N  A   A    T     UUU   R  R   EEEEE        %
 %                                                                             %
 %                                                                             %
-%             Methods to Compute a Message Digest for an Image                %
+%         MagickCore Methods to Compute a Message Digest for an Image         %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/cache.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/property.h"
@@ -469,6 +470,9 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
   char
     *hex_signature;
 
+  ExceptionInfo
+    *exception;
+
   long
     y;
 
@@ -493,6 +497,9 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
   unsigned char
     *pixels;
 
+  ViewInfo
+    *image_view;
+
   /*
     Compute image digital signature.
   */
@@ -516,16 +523,20 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
   signature_info=AcquireSignatureInfo();
   signature=AcquireStringInfo(quantum_info->extent);
   pixels=GetQuantumPixels(quantum_info);
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    length=ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
+    length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,&image->exception);
     SetStringInfoLength(signature,length);
     SetStringInfoDatum(signature,pixels);
     UpdateSignature(signature_info,signature);
   }
+  image_view=DestroyCacheView(image_view);
   FinalizeSignature(signature_info);
   hex_signature=StringInfoToHexString(GetSignatureDigest(signature_info));
   (void) DeleteImageProperty(image,"signature");
@@ -781,7 +792,7 @@ MagickExport void UpdateSignature(SignatureInfo *signature_info,
       i=GetStringInfoLength(signature_info->message)-signature_info->offset;
       if (i > n)
         i=n;
-      (void) memcpy(GetStringInfoDatum(signature_info->message)+
+      (void) CopyMagickMemory(GetStringInfoDatum(signature_info->message)+
         signature_info->offset,p,i);
       n-=i;
       p+=i;
@@ -798,7 +809,7 @@ MagickExport void UpdateSignature(SignatureInfo *signature_info,
     n-=GetStringInfoLength(signature_info->message);
     TransformSignature(signature_info);
   }
-  (void) memcpy(GetStringInfoDatum(signature_info->message),p,n);
+  (void) CopyMagickMemory(GetStringInfoDatum(signature_info->message),p,n);
   signature_info->offset=n;
   /*
     Reset working registers.
