@@ -10,14 +10,14 @@
 %        SSSSS    T    A   A    T    IIIII  SSSSS    T    IIIII   CCCC        %
 %                                                                             %
 %                                                                             %
-%                          ImageMagick Image Methods                          %
+%                          MagickCore Image Methods                           %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -112,7 +112,7 @@
 %
 %    o image: the image.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
@@ -147,39 +147,52 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   bounds.y=(long) image->rows;
   GetMagickPixelPacket(image,&target[0]);
   image_view=AcquireCacheView(image);
-  p=AcquireCacheViewPixels(image_view,0,0,1,1,exception);
+  p=GetCacheViewVirtualPixels(image_view,0,0,1,1,exception);
   if (p == (const PixelPacket *) NULL)
     {
       image_view=DestroyCacheView(image_view);
       return(bounds);
     }
-  SetMagickPixelPacket(image,p,GetCacheViewIndexes(image_view),&target[0]);
+  SetMagickPixelPacket(image,p,GetCacheViewAuthenticIndexQueue(image_view),
+    &target[0]);
   GetMagickPixelPacket(image,&target[1]);
-  p=AcquireCacheViewPixels(image_view,(long) image->columns-1,0,1,1,exception);
-  SetMagickPixelPacket(image,p,GetCacheViewIndexes(image_view),&target[1]);
+  p=GetCacheViewVirtualPixels(image_view,(long) image->columns-1,0,1,1,
+    exception);
+  SetMagickPixelPacket(image,p,GetCacheViewAuthenticIndexQueue(image_view),
+    &target[1]);
   GetMagickPixelPacket(image,&target[2]);
-  p=AcquireCacheViewPixels(image_view,0,(long) image->rows-1,1,1,exception);
-  SetMagickPixelPacket(image,p,GetCacheViewIndexes(image_view),&target[2]);
+  p=GetCacheViewVirtualPixels(image_view,0,(long) image->rows-1,1,1,
+    exception);
+  SetMagickPixelPacket(image,p,GetCacheViewAuthenticIndexQueue(image_view),
+    &target[2]);
   status=MagickTrue;
   GetMagickPixelPacket(image,&zero);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     MagickPixelPacket
       pixel;
 
+    register const IndexPacket
+      *indexes;
+
     register const PixelPacket
       *p;
 
-    register IndexPacket
-      *indexes;
-
     register long
       x;
 
-    p=AcquireCacheViewPixels(image_view,0,y,image->columns,1,exception);
+    if (status == MagickFalse)
+      continue;
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
-      break;
-    indexes=GetCacheViewIndexes(image_view);
+      {
+        status=MagickFalse;
+        continue;
+      }
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
     pixel=zero;
     for (x=0; x < (long) image->columns; x++)
     {
@@ -236,7 +249,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
 %
 %    o channel: the channel.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -252,19 +265,20 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     id;
 
   unsigned long
     *current_depth,
-    depth;
+    depth,
+    number_threads;
     
   ViewInfo
-    **image_view;
+    *image_view;
   
-  volatile MagickBooleanType
-    status;
-
   /*
     Compute image depth.
   */
@@ -272,12 +286,13 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  current_depth=(unsigned long *) AcquireQuantumMemory(
-    GetCacheViewMaximumThreads(),sizeof(*current_depth));
+  number_threads=GetPixelCacheMaximumThreads();
+  current_depth=(unsigned long *) AcquireQuantumMemory(number_threads,
+    sizeof(*current_depth));
   if (current_depth == (unsigned long *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
-  for (id=0; id < (long) GetCacheViewMaximumThreads(); id++)
+  for (id=0; id < (long) number_threads; id++)
     current_depth[id]=1;
   if ((image->storage_class == PseudoClass) && (image->matte == MagickFalse))
     {
@@ -288,12 +303,14 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
         i;
 
       p=image->colormap;
-      #pragma omp parallel for schedule(static,64)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
       for (i=0; i < (long) image->colors; i++)
       {  
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
+        id=GetPixelCacheThreadId();
         while (current_depth[id] < MAGICKCORE_QUANTUM_DEPTH)
         {
           MagickStatusType
@@ -320,14 +337,16 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
         p++;
       }
       depth=current_depth[0];
-      for (id=1; id < (long) GetCacheViewMaximumThreads(); id++)
+      for (id=1; id < (long) number_threads; id++)
         if (depth < current_depth[id])
           depth=current_depth[id];
       current_depth=(unsigned long *) RelinquishMagickMemory(current_depth);
       return(depth);
     }
-  image_view=AcquireCacheViewThreadSet(image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     register const IndexPacket
@@ -342,11 +361,11 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],0,y,image->columns,1,exception);
+    id=GetPixelCacheThreadId();
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       continue;
-    indexes=AcquireCacheViewIndexes(image_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
     for (x=0; x < (long) image->columns; x++)
     {  
       while (current_depth[id] < MAGICKCORE_QUANTUM_DEPTH)
@@ -384,9 +403,9 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
     if (current_depth[id] == MAGICKCORE_QUANTUM_DEPTH)
       status=MagickFalse;
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   depth=current_depth[0];
-  for (id=1; id < (long) GetCacheViewMaximumThreads(); id++)
+  for (id=1; id < (long) number_threads; id++)
     if (depth < current_depth[id])
       depth=current_depth[id];
   current_depth=(unsigned long *) RelinquishMagickMemory(current_depth);
@@ -422,7 +441,7 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
 %
 %    o maxima: the maximum value in the channel.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -483,7 +502,7 @@ MagickExport MagickBooleanType GetImageChannelExtrema(const Image *image,
 %
 %    o standard_deviation: the standard deviation of the channel.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -528,10 +547,10 @@ MagickExport MagickBooleanType GetImageChannelMean(const Image *image,
   area=0.0;
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=AcquireIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     for (x=0; x < (long) image->columns; x++)
     {
       if ((channel & RedChannel) != 0)
@@ -608,7 +627,7 @@ MagickExport MagickBooleanType GetImageChannelMean(const Image *image,
 %
 %    o maxima: the maximum value in the channel.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -646,10 +665,10 @@ MagickExport MagickBooleanType GetImageChannelRange(const Image *image,
   GetMagickPixelPacket(image,&pixel);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=AcquireIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     for (x=0; x < (long) image->columns; x++)
     {
       SetMagickPixelPacket(image,p,indexes+x,&pixel);
@@ -725,7 +744,7 @@ MagickExport MagickBooleanType GetImageChannelRange(const Image *image,
 %
 %    o image: the image.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -800,10 +819,10 @@ MagickExport ChannelStatistics *GetImageChannelStatistics(const Image *image,
   y=(long) image->rows;
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,exception);
+    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    indexes=AcquireIndexes(image);
+    indexes=GetVirtualIndexQueue(image);
     for (x=0; x < (long) image->columns; )
     {
       if (channel_statistics[RedChannel].depth != MAGICKCORE_QUANTUM_DEPTH)
@@ -981,6 +1000,12 @@ MagickExport unsigned long GetImageQuantumDepth(const Image *image,
   else
     if (depth <= 16)
       depth=16;
+    else
+      if (depth <= 32)
+        depth=32;
+      else
+        if (depth <= 64)
+          depth=64;
   if (constrain != MagickFalse)
     depth=(unsigned long) MagickMin((double) depth,(double)
       MAGICKCORE_QUANTUM_DEPTH);
@@ -1025,18 +1050,21 @@ MagickExport MagickBooleanType SetImageDepth(Image *image,
 MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
   const ChannelType channel,const unsigned long depth)
 {
+  ExceptionInfo
+    *exception;
+
   long
     y;
 
+  MagickBooleanType
+    status;
+
   QuantumAny
     scale;
 
   ViewInfo
-    **image_view;
+    *image_view;
   
-  volatile MagickBooleanType
-    status;
-
   assert(image != (Image *) NULL);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
@@ -1052,15 +1080,17 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
   */
   status=MagickTrue;
   scale=GetQuantumScale(depth);
-  image_view=AcquireCacheViewThreadSet(image);
-  #pragma omp parallel for schedule(static,64)
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
       *indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
@@ -1068,14 +1098,14 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
+      exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (long) image->columns; x++)
     {
       if ((channel & RedChannel) != 0)
@@ -1096,13 +1126,13 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
           depth,scale);
       q++;
     }
-    if (SyncCacheView(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       {
         status=MagickFalse;
         continue;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   if (image->storage_class == PseudoClass)
     {
       QuantumAny
@@ -1116,7 +1146,9 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
 
       p=image->colormap;
       scale=GetQuantumScale(depth);
-      #pragma omp parallel for schedule(static,64)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
+#endif
       for (i=0; i < (long) image->colors; i++)
       {
         if ((channel & RedChannel) != 0)
