@@ -10,14 +10,14 @@
 %         T    H   H  R  R   EEEEE  SSSSS  H   H   OOO   LLLLL  DDDD          %
 %                                                                             %
 %                                                                             %
-%                     ImageMagick Image Threshold Methods                     %
+%                      MagickCore Image Threshold Methods                     %
 %                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
 %                                 October 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -135,7 +135,7 @@ struct _ThresholdMap
 %
 %    o offset: the mean offset.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport Image *AdaptiveThresholdImage(const Image *image,
@@ -151,6 +151,9 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
     progress,
     y;
 
+  MagickBooleanType
+    status;
+
   MagickPixelPacket
     zero;
 
@@ -158,11 +161,8 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
     number_pixels;
 
   ViewInfo
-    **image_view,
-    **threshold_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view,
+    *threshold_view;
 
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -172,8 +172,7 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
   assert(exception->signature == MagickSignature);
   if ((image->columns < width) || (image->rows < height))
     ThrowImageException(OptionError,"ImageSmallerThanRadius");
-  threshold_image=CloneImage(image,image->columns,image->rows,MagickTrue,
-    exception);
+  threshold_image=CloneImage(image,0,0,MagickTrue,exception);
   if (threshold_image == (Image *) NULL)
     return((Image *) NULL);
   if (SetImageStorageClass(threshold_image,DirectClass) == MagickFalse)
@@ -189,11 +188,16 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
   progress=0;
   GetMagickPixelPacket(image,&zero);
   number_pixels=(MagickRealType) width*height;
-  image_view=AcquireCacheViewThreadSet(image);
-  threshold_view=AcquireCacheViewThreadSet(threshold_image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+  threshold_view=AcquireCacheView(threshold_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
+    MagickBooleanType
+      sync;
+
     register const IndexPacket
       *indexes;
 
@@ -204,7 +208,6 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       *threshold_indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
@@ -212,17 +215,17 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=AcquireCacheViewPixels(image_view[id],-((long) width/2L),y-height/2L,
+    p=GetCacheViewVirtualPixels(image_view,-((long) width/2L),y-height/2L,
       image->columns+width,height,exception);
-    q=SetCacheViewPixels(threshold_view[id],0,y,threshold_image->columns,1);
+    q=GetCacheViewAuthenticPixels(threshold_view,0,y,threshold_image->columns,1,
+      exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=AcquireCacheViewIndexes(image_view[id]);
-    threshold_indexes=GetCacheViewIndexes(threshold_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    threshold_indexes=GetCacheViewAuthenticIndexQueue(threshold_view);
     for (x=0; x < (long) image->columns; x++)
     {
       long
@@ -260,36 +263,39 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       mean.opacity=(MagickRealType) (pixel.opacity/number_pixels+offset);
       if (image->colorspace == CMYKColorspace)
         mean.index=(MagickRealType) (pixel.index/number_pixels+offset);
-      q->red=(Quantum) (((MagickRealType) p->red <= mean.red) ?
+      q->red=(Quantum) (((MagickRealType) q->red <= mean.red) ?
         0 : QuantumRange);
-      q->green=(Quantum) (((MagickRealType) p->green <= mean.green) ?
+      q->green=(Quantum) (((MagickRealType) q->green <= mean.green) ?
         0 : QuantumRange);
-      q->blue=(Quantum) (((MagickRealType) p->blue <= mean.blue) ?
+      q->blue=(Quantum) (((MagickRealType) q->blue <= mean.blue) ?
         0 : QuantumRange);
-      q->opacity=(Quantum) (((MagickRealType) p->opacity <= mean.opacity) ?
+      q->opacity=(Quantum) (((MagickRealType) q->opacity <= mean.opacity) ?
         0 : QuantumRange);
       if (image->colorspace == CMYKColorspace)
-        threshold_indexes[x]=(IndexPacket) (((MagickRealType) indexes[x] <=
-          mean.index) ? 0 : QuantumRange);
+        threshold_indexes[x]=(IndexPacket) (((MagickRealType)
+          threshold_indexes[x] <= mean.index) ? 0 : QuantumRange);
       p++;
       q++;
     }
-    if (SyncCacheView(threshold_view[id]) == MagickFalse)
+    sync=SyncCacheViewAuthenticPixels(threshold_view,exception);
+    if (sync == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  threshold_view=DestroyCacheViewThreadSet(threshold_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  threshold_view=DestroyCacheView(threshold_view);
+  image_view=DestroyCacheView(image_view);
   if (status == MagickFalse)
     threshold_image=DestroyImage(threshold_image);
   return(threshold_image);
@@ -352,16 +358,19 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
 {
 #define ThresholdImageTag  "Threshold/Image"
 
+  ExceptionInfo
+    *exception;
+
   long
     progress,
     y;
 
-  ViewInfo
-    **image_view;
-
-  volatile MagickBooleanType
+  MagickBooleanType
     status;
 
+  ViewInfo
+    *image_view;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -373,15 +382,17 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  #pragma omp parallel for schedule(static,64)
+  exception=(&image->exception);
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
       *indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
@@ -389,14 +400,13 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     if (channel == DefaultChannels)
       {
         for (x=0; x < (long) image->columns; x++)
@@ -435,21 +445,23 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
             0 : QuantumRange);
         q++;
       }
-    if (SyncCacheView(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -471,16 +483,47 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
 %  The format of the BlackThresholdImage method is:
 %
 %      MagickBooleanType BlackThresholdImage(Image *image,const char *threshold)
+%      MagickBooleanType BlackThresholdImageChannel(Image *image,
+%        const ChannelType channel,const char *threshold,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image: the image.
 %
+%    o channel: the channel or channels to be thresholded.
+%
 %    o threshold: Define the threshold value.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 MagickExport MagickBooleanType BlackThresholdImage(Image *image,
   const char *threshold)
+{
+  MagickBooleanType
+    status;
+
+  status=BlackThresholdImageChannel(image,DefaultChannels,threshold,
+    &image->exception);
+  return(status);
+}
+
+static inline MagickBooleanType IsLongGrayPixel(const LongPixelPacket *pixel)
+{
+#if !defined(MAGICKCORE_HDRI_SUPPORT)
+  if ((pixel->red == pixel->green) && (pixel->green == pixel->blue))
+    return(MagickTrue);
+#else
+  if ((fabs(pixel->red-pixel->green) <= MagickEpsilon) &&
+      (fabs(pixel->green-pixel->blue) <= MagickEpsilon))
+    return(MagickTrue);
+#endif
+  return(MagickFalse);
+}
+
+MagickExport MagickBooleanType BlackThresholdImageChannel(Image *image,
+  const ChannelType channel,const char *threshold,ExceptionInfo *exception)
 {
 #define ThresholdImageTag  "Threshold/Image"
 
@@ -491,17 +534,17 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
     progress,
     y;
 
-  MagickPixelPacket
+  LongPixelPacket
     pixel;
 
+  MagickBooleanType
+    status;
+
   MagickStatusType
     flags;
 
   ViewInfo
-    **image_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -511,36 +554,43 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
     return(MagickTrue);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
-  GetMagickPixelPacket(image,&pixel);
   flags=ParseGeometry(threshold,&geometry_info);
-  pixel.red=geometry_info.rho;
-  pixel.green=geometry_info.sigma;
+  pixel.red=(long) (geometry_info.rho+0.5);
+  pixel.green=(long) (geometry_info.sigma+0.5);
   if ((flags & SigmaValue) == 0)
     pixel.green=pixel.red;
-  pixel.blue=geometry_info.xi;
+  pixel.blue=(long) (geometry_info.xi+0.5);
   if ((flags & XiValue) == 0)
     pixel.blue=pixel.red;
-  pixel.opacity=geometry_info.psi;
-  if ((flags & PsiValue) == 0)
-    pixel.opacity=(MagickRealType) OpaqueOpacity;
+  pixel.opacity=(long) (geometry_info.psi+0.5);
+  if ((flags & XiValue) == 0)
+    pixel.opacity=pixel.red;
+  pixel.index=(long) (geometry_info.chi+0.5);
+  if ((flags & ChiValue) == 0)
+    pixel.index=pixel.red;
   if ((flags & PercentValue) != 0)
     {
-      pixel.red*=(MagickRealType) (QuantumRange/100.0);
-      pixel.green*=(MagickRealType) (QuantumRange/100.0);
-      pixel.blue*=(MagickRealType) (QuantumRange/100.0);
-      pixel.opacity*=(MagickRealType) (QuantumRange/100.0);
+      pixel.red*=(QuantumRange/100.0);
+      pixel.green*=(QuantumRange/100.0);
+      pixel.blue*=(QuantumRange/100.0);
+      pixel.opacity*=(QuantumRange/100.0);
+      pixel.index*=(QuantumRange/100.0);
     }
   /*
     Black threshold image.
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *indexes;
+
     register long
-      id,
       x;
 
     register PixelPacket
@@ -548,52 +598,65 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    if (IsMagickGray(&pixel) != MagickFalse)
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    if (IsLongGrayPixel(&pixel) != MagickFalse)
       for (x=0; x < (long) image->columns; x++)
       {
         if ((MagickRealType) PixelIntensityToQuantum(q) < pixel.red)
           {
-            q->red=(Quantum) 0;
-            q->green=(Quantum) 0;
-            q->blue=(Quantum) 0;
+            if ((channel & RedChannel) != 0)
+              q->red=(Quantum) 0;
+            if ((channel & GreenChannel) != 0)
+              q->green=(Quantum) 0;
+            if ((channel & BlueChannel) != 0)
+              q->blue=(Quantum) 0;
           }
         q++;
       }
     else
       for (x=0; x < (long) image->columns; x++)
       {
-        if ((MagickRealType) q->red < pixel.red)
+        if (((channel & RedChannel) != 0) &&
+            (q->red < pixel.red))
           q->red=(Quantum) 0;
-        if ((MagickRealType) q->green < pixel.green)
+        if (((channel & GreenChannel) != 0) &&
+            (q->green < pixel.green))
           q->green=(Quantum) 0;
-        if ((MagickRealType) q->blue < pixel.blue)
+        if (((channel & BlueChannel) != 0) &&
+            (q->blue < pixel.blue))
           q->blue=(Quantum) 0;
-        if ((MagickRealType) q->opacity < pixel.opacity)
+        if (((channel & OpacityChannel) != 0) &&
+            (q->opacity < pixel.opacity))
           q->opacity=(Quantum) 0;
+        if (((channel & IndexChannel) != 0) &&
+            (image->colorspace == CMYKColorspace) &&
+            (indexes[x] < pixel.index))
+          indexes[x]=(Quantum) 0;
         q++;
       }
-    if (SyncCacheView(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -659,15 +722,27 @@ MagickExport ThresholdMap *DestroyThresholdMap(ThresholdMap *map)
 %
 %    o map_id:  ID of the map to look for in XML list.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
   const char *filename,const char *map_id,ExceptionInfo *exception)
 {
-  const char *attr, *content;
-  XMLTreeInfo *thresholds,*threshold,*description,*levels;
-  ThresholdMap *map;
+  const char
+    *attr,
+    *content;
+
+  double
+    value;
+
+ ThresholdMap
+     *map;
+
+  XMLTreeInfo
+     *description,
+     *levels,
+     *threshold,
+     *thresholds;
 
   map = (ThresholdMap *)NULL;
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
@@ -807,14 +882,15 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
       }
       content = p;
     }
-    (void) strtol(content, &p, 10);
-    if ( p != content ) {
-       (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
-         "XmlInvalidContent", "<level> too many values, map \"%s\"", map_id);
-      thresholds = DestroyXMLTree(thresholds);
-      map = DestroyThresholdMap(map);
-      return(map);
-    }
+    value=strtol(content,&p,10);
+    if (p != content)
+      {
+        (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
+          "XmlInvalidContent", "<level> too many values, map \"%s\"", map_id);
+       thresholds=DestroyXMLTree(thresholds);
+       map=DestroyThresholdMap(map);
+       return(map);
+     }
   }
 
   thresholds = DestroyXMLTree(thresholds);
@@ -844,7 +920,7 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
 %
 %    o map_id:  ID of the map to look for.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport ThresholdMap *GetThresholdMap(const char *map_id,
@@ -896,7 +972,7 @@ MagickExport ThresholdMap *GetThresholdMap(const char *map_id,
 %
 %    o filename:  The threshold map XML filename.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickBooleanType ListThresholdMapFile(FILE *file,const char *xml,
@@ -972,7 +1048,7 @@ MagickBooleanType ListThresholdMapFile(FILE *file,const char *xml,
 %
 %    o file:  An pointer to the output FILE.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType ListThresholdMaps(FILE *file,
@@ -1035,7 +1111,7 @@ MagickExport MagickBooleanType ListThresholdMaps(FILE *file,
 %
 %    o channel: the channel or channels to be thresholded.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -1110,7 +1186,7 @@ MagickExport MagickBooleanType OrderedDitherImageChannel(Image *image,
 %      color level, to basically double the number of color levels with
 %      a bare minimim of dithering.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType OrderedPosterizeImage(Image *image,
@@ -1136,14 +1212,14 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
   LongPixelPacket
     levels;
 
+  MagickBooleanType
+    status;
+
   ThresholdMap
     *map;
 
   ViewInfo
-    **image_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1251,12 +1327,13 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
       }
     status=MagickTrue;
     progress=0;
-    image_view=AcquireCacheViewThreadSet(image);
-    #pragma omp parallel for schedule(static,64)
+    image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
     for (y=0; y < (long) image->rows; y++)
     {
       register long
-        id,
         x;
 
       register IndexPacket
@@ -1267,14 +1344,13 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
 
       if (status == MagickFalse)
         continue;
-      id=GetCacheViewThreadId();
-      q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+      q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         {
           status=MagickFalse;
           continue;
         }
-      indexes=GetCacheViewIndexes(image_view[id]);
+      indexes=GetCacheViewAuthenticIndexQueue(image_view);
       for (x=0; x < (long) image->columns; x++)
       {
         register int
@@ -1328,20 +1404,22 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
         }
         q++;
       }
-      if (SyncCacheView(image_view[id]) == MagickFalse)
+      if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
         status=MagickFalse;
       if (image->progress_monitor != (MagickProgressMonitor) NULL)
         {
           MagickBooleanType
             proceed;
 
-          #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
           proceed=SetImageProgress(image,DitherImageTag,progress++,image->rows);
           if (proceed == MagickFalse)
             status=MagickFalse;
         }
     }
-    image_view=DestroyCacheViewThreadSet(image_view);
+    image_view=DestroyCacheView(image_view);
   }
   map=DestroyThresholdMap(map);
   return(MagickTrue);
@@ -1380,7 +1458,7 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
 %      string contains 2x2, 3x3, or 4x4, an ordered dither of order 2, 3, or 4
 %      is performed instead.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -1410,6 +1488,9 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
     progress,
     y;
 
+  MagickBooleanType
+    status;
+
   MagickPixelPacket
     threshold;
 
@@ -1418,10 +1499,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
     max_threshold;
 
   ViewInfo
-    **image_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1464,15 +1542,19 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
       if (AcquireImageColormap(image,2) == MagickFalse)
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
-      image_view=AcquireCacheViewThreadSet(image);
-      #pragma omp parallel for schedule(static,64)
+      image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
       for (y=0; y < (long) image->rows; y++)
       {
+        MagickBooleanType
+          sync;
+
         register IndexPacket
           *indexes;
 
         register long
-          id,
           x;
 
         register PixelPacket
@@ -1480,14 +1562,14 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
-        q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+        q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
+          exception);
         if (q == (PixelPacket *) NULL)
           {
             status=MagickFalse;
             continue;
           }
-        indexes=GetCacheViewIndexes(image_view[id]);
+        indexes=GetCacheViewAuthenticIndexQueue(image_view);
         for (x=0; x < (long) image->columns; x++)
         {
           IndexPacket
@@ -1508,21 +1590,24 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           indexes[x]=index;
           *q++=image->colormap[(long) index];
         }
-        if (SyncCacheView(image_view[id]) == MagickFalse)
+        sync=SyncCacheViewAuthenticPixels(image_view,exception);
+        if (sync == MagickFalse)
           status=MagickFalse;
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
           {
             MagickBooleanType
               proceed;
 
-            #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
             proceed=SetImageProgress(image,ThresholdImageTag,progress++,
               image->rows);
             if (proceed == MagickFalse)
               status=MagickFalse;
           }
       }
-      image_view=DestroyCacheViewThreadSet(image_view);
+      image_view=DestroyCacheView(image_view);
       return(status);
     }
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
@@ -1530,15 +1615,16 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
       InheritException(exception,&image->exception);
       return(MagickFalse);
     }
-  image_view=AcquireCacheViewThreadSet(image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
       *indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
@@ -1546,14 +1632,13 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     for (x=0; x < (long) image->columns; x++)
     {
       if ((channel & RedChannel) != 0)
@@ -1630,21 +1715,23 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           threshold.index ? 0 : QuantumRange);
       q++;
     }
-    if (SyncCacheView(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -1666,16 +1753,34 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
 %  The format of the WhiteThresholdImage method is:
 %
 %      MagickBooleanType WhiteThresholdImage(Image *image,const char *threshold)
+%      MagickBooleanType WhiteThresholdImageChannel(Image *image,
+%        const ChannelType channel,const char *threshold,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image: the image.
 %
-%    o threshold: Define the threshold value
+%    o channel: the channel or channels to be thresholded.
+%
+%    o threshold: Define the threshold value.
+%
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
   const char *threshold)
+{
+  MagickBooleanType
+    status;
+
+  status=WhiteThresholdImageChannel(image,DefaultChannels,threshold,
+    &image->exception);
+  return(status);
+}
+
+MagickExport MagickBooleanType WhiteThresholdImageChannel(Image *image,
+  const ChannelType channel,const char *threshold,ExceptionInfo *exception)
 {
 #define ThresholdImageTag  "Threshold/Image"
 
@@ -1686,17 +1791,17 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
     progress,
     y;
 
-  MagickPixelPacket
+  LongPixelPacket
     pixel;
 
+  MagickBooleanType
+    status;
+
   MagickStatusType
     flags;
 
   ViewInfo
-    **image_view;
-
-  volatile MagickBooleanType
-    status;
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1706,36 +1811,43 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
     return(MagickTrue);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
-  GetMagickPixelPacket(image,&pixel);
   flags=ParseGeometry(threshold,&geometry_info);
-  pixel.red=geometry_info.rho;
-  pixel.green=geometry_info.sigma;
+  pixel.red=(long) (geometry_info.rho+0.5);
+  pixel.green=(long) (geometry_info.sigma+0.5);
   if ((flags & SigmaValue) == 0)
     pixel.green=pixel.red;
   pixel.blue=geometry_info.xi;
   if ((flags & XiValue) == 0)
     pixel.blue=pixel.red;
-  pixel.opacity=geometry_info.psi;
-  if ((flags & PsiValue) == 0)
-    pixel.opacity=(MagickRealType) OpaqueOpacity;
+  pixel.opacity=(long) (geometry_info.psi+0.5);
+  if ((flags & XiValue) == 0)
+    pixel.opacity=pixel.red;
+  pixel.index=geometry_info.chi;
+  if ((flags & ChiValue) == 0)
+    pixel.index=pixel.red;
   if ((flags & PercentValue) != 0)
     {
-      pixel.red*=(MagickRealType) (QuantumRange/100.0);
-      pixel.green*=(MagickRealType) (QuantumRange/100.0);
-      pixel.blue*=(MagickRealType) (QuantumRange/100.0);
-      pixel.opacity*=(MagickRealType) (QuantumRange/100.0);
+      pixel.red*=(QuantumRange/100.0);
+      pixel.green*=(QuantumRange/100.0);
+      pixel.blue*=(QuantumRange/100.0);
+      pixel.opacity*=(QuantumRange/100.0);
+      pixel.index*=(QuantumRange/100.0);
     }
   /*
     White threshold image.
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  #pragma omp parallel for schedule(static,64)
+  image_view=AcquireCacheView(image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *indexes;
+
     register long
-      id,
       x;
 
     register PixelPacket
@@ -1743,51 +1855,64 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewPixels(image_view[id],0,y,image->columns,1);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    if (IsMagickGray(&pixel) != MagickFalse)
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    if (IsLongGrayPixel(&pixel) != MagickFalse)
       for (x=0; x < (long) image->columns; x++)
       {
         if ((MagickRealType) PixelIntensityToQuantum(q) > pixel.red)
           {
-            q->red=(Quantum) QuantumRange;
-            q->green=(Quantum) QuantumRange;
-            q->blue=(Quantum) QuantumRange;
+            if ((channel & RedChannel) != 0)
+              q->red=(Quantum) QuantumRange;
+            if ((channel & GreenChannel) != 0)
+              q->green=(Quantum) QuantumRange;
+            if ((channel & BlueChannel) != 0)
+              q->blue=(Quantum) QuantumRange;
           }
         q++;
       }
     else
       for (x=0; x < (long) image->columns; x++)
       {
-        if ((MagickRealType) q->red > pixel.red)
+        if (((channel & RedChannel) != 0) &&
+            (q->red > pixel.red))
           q->red=(Quantum) QuantumRange;
-        if ((MagickRealType) q->green > pixel.green)
+        if (((channel & GreenChannel) != 0) &&
+            (q->green > pixel.green))
           q->green=(Quantum) QuantumRange;
-        if ((MagickRealType) q->blue > pixel.blue)
+        if (((channel & BlueChannel) != 0) &&
+            (q->blue > pixel.blue))
           q->blue=(Quantum) QuantumRange;
-        if ((MagickRealType) q->opacity > pixel.opacity)
+        if (((channel & OpacityChannel) != 0) &&
+            (q->opacity > pixel.opacity))
           q->opacity=(Quantum) QuantumRange;
+        if (((channel & IndexChannel) != 0) &&
+            (image->colorspace == CMYKColorspace) &&
+            (indexes[x] > pixel.index))
+          indexes[x]=(Quantum) QuantumRange;
         q++;
       }
-    if (SyncCacheView(image_view[id]) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
           proceed;
 
-        #pragma omp critical
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical
+#endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
