@@ -10,14 +10,14 @@
 %              UUU     T    IIIII  LLLLL  IIIII    T      Y                   %
 %                                                                             %
 %                                                                             %
-%                       ImageMagick Utility Methods                           %
+%                       MagickCore Utility Methods                            %
 %                                                                             %
 %                             Software Design                                 %
 %                               John Cristy                                   %
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -199,9 +199,9 @@ MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
       (void) RelinquishUniqueFileResource(destination);
       return(MagickFalse);
     }
-  quantum=(size_t) MagickMaxBufferSize;
+  quantum=(size_t) MagickMaxBufferExtent;
   if ((fstat(source_file,&attributes) == 0) && (attributes.st_size != 0))
-    quantum=MagickMin((size_t) attributes.st_size,MagickMaxBufferSize);
+    quantum=MagickMin((size_t) attributes.st_size,MagickMaxBufferExtent);
   buffer=(unsigned char *) AcquireQuantumMemory(quantum,sizeof(*buffer));
   if (buffer == (unsigned char *) NULL)
     {
@@ -657,27 +657,22 @@ MagickExport void ExpandFilename(char *path)
 %
 %  The format of the ExpandFilenames function is:
 %
-%      status=ExpandFilenames(int *argc,char ***argv)
+%      status=ExpandFilenames(int *number_arguments,char ***arguments)
 %
 %  A description of each parameter follows:
 %
-%    o argc: Specifies a pointer to an integer describing the number of
-%      elements in the argument vector.
+%    o number_arguments: Specifies a pointer to an integer describing the
+%      number of elements in the argument vector.
 %
-%    o argv: Specifies a pointer to a text array containing the command line
-%      arguments.
+%    o arguments: Specifies a pointer to a text array containing the command
+%      line arguments.
 %
 */
-MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
+MagickExport MagickBooleanType ExpandFilenames(int *number_arguments,
+  char ***arguments)
 {
   char
-    **filelist,
-    filename[MaxTextExtent],
     home_directory[MaxTextExtent],
-    magick[MaxTextExtent],
-    *option,
-    path[MaxTextExtent],
-    subimage[MaxTextExtent],
     **vector;
 
   long
@@ -695,9 +690,9 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
     Allocate argument vector.
   */
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(argc != (int *) NULL);
-  assert(argv != (char ***) NULL);
-  vector=(char **) AcquireQuantumMemory(*argc+1UL,sizeof(*vector));
+  assert(number_arguments != (int *) NULL);
+  assert(arguments != (char ***) NULL);
+  vector=(char **) AcquireQuantumMemory(*number_arguments+1UL,sizeof(*vector));
   if (vector == (char **) NULL)
     return(MagickFalse);
   /*
@@ -706,10 +701,22 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
   if (getcwd(home_directory,MaxTextExtent) == (char *) NULL)
     return(MagickFalse);
   count=0;
-  for (i=0; i < (long) *argc; i++)
+  for (i=0; i < (long) *number_arguments; i++)
   {
-    option=(*argv)[i];
+    char
+      **filelist,
+      filename[MaxTextExtent],
+      magick[MaxTextExtent],
+      *option,
+      path[MaxTextExtent],
+      subimage[MaxTextExtent];
+
+    MagickBooleanType
+      destroy;
+
+    option=(*arguments)[i];
     vector[count++]=ConstantString(option);
+    destroy=MagickTrue;
     parameters=ParseMagickOption(MagickCommandOptions,MagickFalse,option);
     if (parameters > 0)
       {
@@ -719,9 +726,9 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
         for (j=0; j < parameters; j++)
         {
           i++;
-          if (i == (long) *argc)
+          if (i == (long) *number_arguments)
             break;
-          option=(*argv)[i];
+          option=(*arguments)[i];
           vector[count++]=ConstantString(option);
         }
         continue;
@@ -756,11 +763,10 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
     /*
       Transfer file list to argument vector.
     */
-    vector=(char **) ResizeQuantumMemory(vector,(size_t) *argc+count+
-      number_files+1,sizeof(*vector));
+    vector=(char **) ResizeQuantumMemory(vector,(size_t) *number_arguments+
+      count+number_files+1,sizeof(*vector));
     if (vector == (char **) NULL)
       return(MagickFalse);
-    count--;
     for (j=0; j < (long) number_files; j++)
     {
       (void) CopyMagickString(filename,path,MaxTextExtent);
@@ -769,6 +775,8 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
           MaxTextExtent);
       (void) ConcatenateMagickString(filename,filelist[j],MaxTextExtent);
       filelist[j]=DestroyString(filelist[j]);
+      if (strlen(filename) >= MaxTextExtent)
+        ThrowFatalException(OptionFatalError,"FilenameTruncated");
       if (IsPathAccessible(filename) != MagickFalse)
         {
           char
@@ -787,6 +795,14 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
               (void) ConcatenateMagickString(path,subimage,MaxTextExtent);
               (void) ConcatenateMagickString(path,"]",MaxTextExtent);
             }
+          if (strlen(path) >= MaxTextExtent)
+            ThrowFatalException(OptionFatalError,"FilenameTruncated");
+          if (destroy != MagickFalse)
+            {
+              count--;
+              vector[count]=DestroyString(vector[count]);
+              destroy=MagickFalse;
+            }
           vector[count++]=ConstantString(path);
         }
     }
@@ -809,8 +825,8 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
         "Command line: %s",command_line);
       command_line=DestroyString(command_line);
     }
-  *argc=(int) count;
-  *argv=vector;
+  *number_arguments=(int) count;
+  *arguments=vector;
   return(MagickTrue);
 }
 
@@ -841,8 +857,11 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
 */
 MagickExport MagickBooleanType GetExecutionPath(char *path,const size_t extent)
 {
+  char
+    *cwd;
+
   *path='\0';
-  (void) getcwd(path,(unsigned long) extent);
+  cwd=getcwd(path,(unsigned long) extent);
 #if defined(MAGICKCORE_HAVE_GETPID) && defined(MAGICKCORE_HAVE_READLINK)
   {
     char
@@ -1738,13 +1757,13 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
   else
     {
       char
-        **argv;
+        **arguments;
 
       int
-        argc;
+        number_arguments;
 
-      argv=StringToArgv(command,&argc);
-      if (argv == (char **) NULL)
+      arguments=StringToArgv(command,&number_arguments);
+      if (arguments == (char **) NULL)
         status=system(command);
       else
         {
@@ -1763,7 +1782,7 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
           else
             if (child_pid == 0)
               {
-                status=execvp(argv[1],argv+1);
+                status=execvp(arguments[1],arguments+1);
                 _exit(1);
               }
             else
@@ -1787,9 +1806,9 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
                         status=(-1);
                   }
               }
-          for (i=1; i < argc; i++)
-            argv[i]=DestroyString(argv[i]);
-          argv=(char **) RelinquishMagickMemory(argv);
+          for (i=0; i < number_arguments; i++)
+            arguments[i]=DestroyString(arguments[i]);
+          arguments=(char **) RelinquishMagickMemory(arguments);
         }
     }
 #endif
