@@ -17,7 +17,7 @@
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -74,11 +74,11 @@
 %
 %    o metadata: any metadata is returned here.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
-static void CompareUsage(void)
+static MagickBooleanType CompareUsage(void)
 {
   const char
     **p;
@@ -160,7 +160,7 @@ static void CompareUsage(void)
   (void) printf(
     "image type as the filename suffix (i.e. image.ps).  Specify 'file' as\n");
   (void) printf("'-' for standard input or output.\n");
-  exit(0);
+  return(MagickTrue);
 }
 
 WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
@@ -184,7 +184,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
 #define ThrowCompareInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyCompare(); \
   return(MagickFalse); \
 }
@@ -248,7 +248,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         }
     }
   if (argc < 3)
-    CompareUsage();
+    return(CompareUsage());
   channels=AllChannels;
   difference_image=NewImageList();
   distortion=0.0;
@@ -302,7 +302,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           filename=argv[++i];
         (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
-        images=ReadImage(image_info,exception);
+        images=ReadImages(image_info,exception);
         status&=(images != (Image *) NULL) &&
           (exception->severity < ErrorException);
         if (images == (Image *) NULL)
@@ -546,7 +546,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
       {
         if ((LocaleCompare("help",option+1) == 0) ||
             (LocaleCompare("-help",option+1) == 0))
-          CompareUsage();
+          return(CompareUsage());
         if (LocaleCompare("highlight-color",option+1) == 0)
           {
             if (*option == '+')
@@ -588,6 +588,9 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -604,7 +607,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowCompareException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowCompareInvalidArgumentException(option,argv[i]);
             break;
