@@ -3,21 +3,21 @@
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%        CCCC   OOO  M   M  PPPP    OOO   SSSSS  IIIII  TTTTT  EEEEE          %
-%       C      O   O MM MM  P   P  O   O  SS       I      T    E              %
-%       C      O   O M M M  PPPP   O   O   SSS     I      T    EEE            %
-%       C      O   O M   M  P      O   O     SS    I      T    E              %
-%        CCCC   OOO  M   M  P       OOO   SSSSS  IIIII    T    EEEEE          %
+%        CCCC    OOO   M   M  PPPP    OOO   SSSSS  IIIII  TTTTT  EEEEE        %
+%       C       O   O  MM MM  P   P  O   O  SS       I      T    E            %
+%       C       O   O  M M M  PPPP   O   O   SSS     I      T    EEE          %
+%       C       O   O  M   M  P      O   O     SS    I      T    E            %
+%        CCCC    OOO   M   M  P       OOO   SSSSS  IIIII    T    EEEEE        %
 %                                                                             %
 %                                                                             %
-%                    ImageMagick Image Composite Methods                      %
+%                      MagickWand Image Composite Methods                     %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -69,6 +69,9 @@ typedef struct _CompositeOptions
   long
     stegano;
 
+  RectangleInfo
+    offset;
+
   MagickBooleanType
     stereo,
     tile;
@@ -103,7 +106,7 @@ typedef struct _CompositeOptions
 %
 %    o metadata: any metadata is returned here.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
@@ -162,7 +165,9 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
             Image
               *stereo_image;
 
-            stereo_image=StereoImage(*image,composite_image,exception);
+            stereo_image=StereoAnaglyphImage(*image,composite_image,
+              composite_options->offset.x,composite_options->offset.y,
+              exception);
             if (stereo_image != (Image *) NULL)
               {
                 *image=DestroyImageList(*image);
@@ -210,7 +215,8 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
                 "%lux%lu%+ld%+ld",composite_image->columns,
                 composite_image->rows,geometry.x,geometry.y);
               (*image)->gravity=(GravityType) composite_options->gravity;
-              (void) ParseGravityGeometry(*image,composite_geometry,&geometry);
+              (void) ParseGravityGeometry(*image,composite_geometry,&geometry,
+                exception);
               status&=CompositeImageChannel(*image,composite_options->channel,
                 composite_options->compose,composite_image,geometry.x,
                 geometry.y);
@@ -220,7 +226,7 @@ static MagickBooleanType CompositeImageList(ImageInfo *image_info,Image **image,
   return(status != 0 ? MagickTrue : MagickFalse);
 }
 
-static void CompositeUsage(void)
+static MagickBooleanType CompositeUsage(void)
 {
   const char
     **p;
@@ -258,7 +264,7 @@ static void CompositeUsage(void)
       "-sharpen geometry    sharpen the image",
       "-shave geometry      shave pixels from the image edges",
       "-stegano offset      hide watermark within an image",
-      "-stereo              combine two image to create a stereo anaglyph",
+      "-stereo geometry     combine two image to create a stereo anaglyph",
       "-strip               strip image of all profiles and comments",
       "-thumbnail geometry  create a thumbnail of the image",
       "-transform           affine transform image",
@@ -355,7 +361,7 @@ static void CompositeUsage(void)
   (void) printf(
     "image type as the filename suffix (i.e. image.ps).  Specify 'file' as\n");
   (void) printf("'-' for standard input or output.\n");
-  exit(0);
+  return(MagickTrue);
 }
 
 static void GetCompositeOptions(CompositeOptions *composite_options)
@@ -409,7 +415,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
 #define ThrowCompositeInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyComposite(); \
   return(MagickFalse); \
 }
@@ -468,7 +474,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
         }
     }
   if (argc < 4)
-    CompositeUsage();
+    return(CompositeUsage());
   GetCompositeOptions(&composite_options);
   filename=(char *) NULL;
   format="%w,%h,%m";
@@ -522,7 +528,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
         if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           filename=argv[++i];
         (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
-        images=ReadImage(image_info,exception);
+        images=ReadImages(image_info,exception);
         status&=(images != (Image *) NULL) &&
           (exception->severity < ErrorException);
         if (images == (Image *) NULL)
@@ -1019,7 +1025,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
       {
         if ((LocaleCompare("help",option+1) == 0) ||
             (LocaleCompare("-help",option+1) == 0))
-          CompositeUsage();
+          return(CompositeUsage());
         ThrowCompositeException(OptionError,"UnrecognizedOption",option)
       }
       case 'i':
@@ -1078,6 +1084,9 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -1094,7 +1103,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowCompositeException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowCompositeInvalidArgumentException(option,argv[i]);
             break;
@@ -1338,8 +1347,21 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("stereo",option+1) == 0)
           {
-            composite_options.stereo=(*option == '-') ? MagickTrue :
-              MagickFalse;
+            int
+              flags;
+
+            composite_options.stereo=MagickFalse;
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowCompositeException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowCompositeInvalidArgumentException(option,argv[i]);
+            flags=ParseAbsoluteGeometry(argv[i],&composite_options.offset);
+            if ((flags & YValue) == 0)
+              composite_options.offset.y=composite_options.offset.x;
+            composite_options.stereo=MagickTrue;
             break;
           }
         if (LocaleCompare("strip",option+1) == 0)
@@ -1553,8 +1575,23 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
   RemoveImageStack(mask_image);
   if (mask_image != (Image *) NULL)
     {
-      images->mask=mask_image;
-      (void) NegateImage(images->mask,MagickFalse);
+      if (composite_options.compose == DisplaceCompositeOp)
+        {
+          /*
+            Merge Y displacement into X displacement image.
+          */
+          (void) CompositeImage(composite_image,CopyGreenCompositeOp,
+            mask_image,0,0);
+          mask_image=DestroyImage(mask_image);
+        }
+      else
+        {
+          /*
+            Set a blending mask for the composition.
+          */
+          images->mask=mask_image;
+          (void) NegateImage(images->mask,MagickFalse);
+        }
     }
   status&=CompositeImageList(image_info,&images,composite_image,
     &composite_options,exception);
