@@ -17,7 +17,7 @@
 %                                July 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -84,11 +84,11 @@
 %
 %    o metadata: any metadata is returned here.
 %
-%    o exception: Return any errors or warnings in this structure.
+%    o exception: return any errors or warnings in this structure.
 %
 */
 
-static void DisplayUsage(void)
+static MagickBooleanType DisplayUsage(void)
 {
   const char
     **p;
@@ -132,6 +132,7 @@ static void DisplayUsage(void)
       "-gamma value         level of gamma correction",
       "-monochrome          transform image to black and white",
       "-negate              replace every pixel with its complementary color",
+      "-normalize           transform image to span the full range of colors",
       "-raise value         lighten/darken image edges to create a 3-D effect",
       "-resample geometry   change the resolution of an image",
       "-resize geometry     resize the image",
@@ -189,7 +190,6 @@ static void DisplayUsage(void)
       "-respect-parenthesis settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
-      "-scenes range        image scene range",
       "-seed value          seed a new sequence of pseudo-random numbers",
       "-set property value  set an image property",
       "-size geometry       width and height of image",
@@ -249,7 +249,7 @@ static void DisplayUsage(void)
   (void) printf("\nButtons: \n");
   for (p=buttons; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
-  exit(0);
+  return(MagickTrue);
 }
 
 WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
@@ -283,7 +283,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
 #define ThrowDisplayInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyDisplay(); \
   return(MagickFalse); \
 }
@@ -305,14 +305,11 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
     image_stack[MaxImageStackDepth+1];
 
   long
-    first_scene,
     image_number,
     iteration,
     j,
     k,
-    l,
-    last_scene,
-    scene;
+    l;
 
   MagickBooleanType
     fire;
@@ -361,13 +358,11 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
     }
   SetNotifyHandlers;
   display=(Display *) NULL;
-  first_scene=0;
   j=1;
   k=0;
   image_marker=(unsigned long *) NULL;
   image_number=0;
   last_image=0;
-  last_scene=0;
   NewImageStack();
   option=(char *) NULL;
   pend=MagickFalse;
@@ -411,7 +406,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       }
     if ((LocaleCompare("help",option+1) == 0) ||
         (LocaleCompare("-help",option+1) == 0))
-      DisplayUsage();
+      return(DisplayUsage());
   }
   /*
     Get user defaults from X resource database.
@@ -492,170 +487,141 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       }
     if (IsMagickOption(option) == MagickFalse)
       {
+        const char
+          *filename;
+
+        Image
+          *images;
+
         /*
           Option is a file name.
         */
         FireImageStack(MagickFalse,MagickFalse,pend);
-        for (scene=first_scene; scene <= last_scene ; scene++)
-        {
-          const char
-            *filename;
-
-          Image
-            *images;
-
-          /*
-            Read image.
-          */
-          filename=option;
-          if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
-            {
-              option=argv[++i];
-              filename=option;
-            }
-          (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
-          if (first_scene != last_scene)
-            {
-              char
-                filename[MaxTextExtent];
-
-              /*
-                Form filename for multi-part images.
-              */
-              (void) InterpretImageFilename(filename,MaxTextExtent,
-                image_info->filename,(int) scene);
-              if (LocaleCompare(filename,image_info->filename) == 0)
-                (void) FormatMagickString(filename,MaxTextExtent,"%s.%lu",
-                  image_info->filename,scene);
-              (void) CopyMagickString(image_info->filename,filename,
-                MaxTextExtent);
-            }
-          images=ReadImage(image_info,exception);
-          CatchException(exception);
-          status&=(images != (Image *) NULL) &&
-            (exception->severity < ErrorException);
-          if (images == (Image *) NULL)
-            continue;
-          AppendImageStack(images);
-          FireImageStack(MagickFalse,MagickFalse,MagickTrue);
-          do
+        filename=option;
+        if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
           {
-            /*
-              Transmogrify image as defined by the image processing options.
-            */
-            resource_info.quantum=1;
-            if (first_scene != last_scene)
-              image->scene=(unsigned long) scene;
-            if (resource_info.window_id != (char *) NULL)
-              {
-                /*
-                  Display image to a specified X window.
-                */
-                status=XDisplayBackgroundImage(display,&resource_info,
-                  image);
-                if (status != MagickFalse)
-                  state|=RetainColorsState;
-              }
-            else
-              do
-              {
-                Image
-                  *nexus;
-
-                /*
-                  Display image to X server.
-                */
-                nexus=XDisplayImage(display,&resource_info,argv,argc,
-                  &image,&state);
-                if (nexus == (Image *) NULL)
-                  break;
-                while ((nexus != (Image *) NULL) && ((state & ExitState) == 0))
-                {
-                  if (nexus->montage != (char *) NULL)
-                    {
-                      /*
-                        User selected a visual directory image (montage).
-                      */
-                      RemoveAllImageStack()
-                      image=nexus;
-                      break;
-                    }
-                  if (first_scene != last_scene)
-                    image->scene=(unsigned long) scene;
-                  image=XDisplayImage(display,&resource_info,argv,argc,&nexus,
-                    &state);
-                  if ((image == (Image *) NULL) &&
-                      (GetNextImageInList(nexus) != (Image *) NULL))
-                    {
-                      RemoveAllImageStack()
-                      image=GetNextImageInList(nexus);
-                      nexus=NewImageList();
-                    }
-                  else
-                    {
-                      if (nexus != image)
-                        nexus=DestroyImageList(nexus);
-                      nexus=image;
-                    }
-                }
-              } while ((state & ExitState) == 0);
-            if (resource_info.write_filename != (char *) NULL)
-              {
-                /*
-                  Write image.
-                */
-                (void) CopyMagickString(image->filename,
-                  resource_info.write_filename,MaxTextExtent);
-                (void) SetImageInfo(image_info,MagickTrue,
-                  &image->exception);
-                status&=WriteImage(image_info,image);
-                GetImageException(image,exception);
-              }
-            /*
-              Proceed to next/previous image.
-            */
-            image=image;
-            if ((state & FormerImageState) != 0)
-              for (l=0; l < resource_info.quantum; l++)
-              {
-                image=GetPreviousImageInList(image);
-                if (image == (Image *) NULL)
-                  break;
-              }
-            else
-              for (l=0; l < resource_info.quantum; l++)
-              {
-                image=GetNextImageInList(image);
-                if (image == (Image *) NULL)
-                  break;
-              }
-            if (image != (Image *) NULL)
-              image=image;
-          } while ((image != (Image *) NULL) && ((state & ExitState) == 0));
+            option=argv[++i];
+            filename=option;
+          }
+        (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
+        images=ReadImage(image_info,exception);
+        CatchException(exception);
+        status&=(images != (Image *) NULL) &&
+          (exception->severity < ErrorException);
+        if (images == (Image *) NULL)
+          continue;
+        AppendImageStack(images);
+        FireImageStack(MagickFalse,MagickFalse,MagickTrue);
+        do
+        {
           /*
-            Free image resources.
+            Transmogrify image as defined by the image processing options.
           */
-          RemoveAllImageStack()
-          if ((state & FormerImageState) == 0)
+          resource_info.quantum=1;
+          if (resource_info.window_id != (char *) NULL)
             {
-              last_image=(unsigned long) image_number;
-              image_marker[i]=(unsigned long) image_number++;
+              /*
+                Display image to a specified X window.
+              */
+              status=XDisplayBackgroundImage(display,&resource_info,image);
+              if (status != MagickFalse)
+                state|=RetainColorsState;
             }
           else
+            do
+            {
+              Image
+                *nexus;
+
+              /*
+                Display image to X server.
+              */
+              nexus=XDisplayImage(display,&resource_info,argv,argc,&image,
+                &state);
+              if (nexus == (Image *) NULL)
+                break;
+              while ((nexus != (Image *) NULL) && ((state & ExitState) == 0))
+              {
+                if (nexus->montage != (char *) NULL)
+                  {
+                    /*
+                      User selected a visual directory image (montage).
+                    */
+                    RemoveAllImageStack()
+                    image=nexus;
+                    break;
+                  }
+                image=XDisplayImage(display,&resource_info,argv,argc,&nexus,
+                  &state);
+                if ((image == (Image *) NULL) &&
+                    (GetNextImageInList(nexus) != (Image *) NULL))
+                  {
+                    RemoveAllImageStack()
+                    image=GetNextImageInList(nexus);
+                    nexus=NewImageList();
+                  }
+                else
+                  {
+                    if (nexus != image)
+                      nexus=DestroyImageList(nexus);
+                    nexus=image;
+                  }
+              }
+            } while ((state & ExitState) == 0);
+          if (resource_info.write_filename != (char *) NULL)
             {
               /*
-                Proceed to previous image.
+                Write image.
               */
-              for (i--; i > 0; i--)
-                if (image_marker[i] == (unsigned long) (image_number-2))
-                  break;
-              image_number--;
+              (void) CopyMagickString(image->filename,
+                resource_info.write_filename,MaxTextExtent);
+              (void) SetImageInfo(image_info,MagickTrue,
+                &image->exception);
+              status&=WriteImage(image_info,image);
+              GetImageException(image,exception);
             }
-          if ((i == (long) (argc-1)) && ((state & ExitState) == 0))
-            i=0;
-          if ((state & ExitState) != 0)
-            break;
-        }
+          /*
+            Proceed to next/previous image.
+          */
+          if ((state & FormerImageState) != 0)
+            for (l=0; l < resource_info.quantum; l++)
+            {
+              image=GetPreviousImageInList(image);
+              if (image == (Image *) NULL)
+                break;
+            }
+          else
+            for (l=0; l < resource_info.quantum; l++)
+            {
+              image=GetNextImageInList(image);
+              if (image == (Image *) NULL)
+                break;
+            }
+        } while ((image != (Image *) NULL) && ((state & ExitState) == 0));
+        /*
+          Free image resources.
+        */
+        SetImageStack(image);
+        RemoveAllImageStack();
+        if ((state & FormerImageState) == 0)
+          {
+            last_image=(unsigned long) image_number;
+            image_marker[i]=(unsigned long) image_number++;
+          }
+        else
+          {
+            /*
+              Proceed to previous image.
+            */
+            for (i--; i > 0; i--)
+              if (image_marker[i] == (unsigned long) (image_number-2))
+                break;
+            image_number--;
+          }
+        if ((i == (long) (argc-1)) && ((state & ExitState) == 0))
+          i=0;
+        if ((state & ExitState) != 0)
+          break;
         /*
           Determine if we should proceed to the first image.
         */
@@ -1270,6 +1236,9 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -1286,7 +1255,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowDisplayException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowDisplayInvalidArgumentException(option,argv[i]);
             break;
@@ -1383,6 +1352,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           break;
         if (LocaleCompare("noop",option+1) == 0)
           break;
+        if (LocaleCompare("normalize",option+1) == 0)
+          break;
         ThrowDisplayException(OptionError,"UnrecognizedOption",option);
       }
       case 'p':
@@ -1555,8 +1526,6 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("scenes",option+1) == 0)
           {
-            first_scene=0;
-            last_scene=0;
             if (*option == '+')
               break;
             i++;
@@ -1564,9 +1533,6 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               ThrowDisplayException(OptionError,"MissingArgument",option);
             if (IsSceneGeometry(argv[i],MagickFalse) == MagickFalse)
               ThrowDisplayInvalidArgumentException(option,argv[i]);
-            first_scene=atol(argv[i]);
-            last_scene=first_scene;
-            (void) sscanf(argv[i],"%ld-%ld",&first_scene,&last_scene);
             break;
           }
         if (LocaleCompare("seed",option+1) == 0)
@@ -1824,11 +1790,12 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             if (IsPathAccessible(resource_info.write_filename) != MagickFalse)
               {
                 char
-                  answer[2];
+                  answer[2],
+                  *p;
 
                 (void) fprintf(stderr,"Overwrite %s? ",
                   resource_info.write_filename);
-                (void) fgets(answer,(int) sizeof(answer),stdin);
+                p=fgets(answer,(int) sizeof(answer),stdin);
                 if (((*answer != 'y') && (*answer != 'Y')))
                   return(MagickFalse);
               }
@@ -1858,7 +1825,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
 #else
   (void) ThrowMagickException(exception,GetMagickModule(),MissingDelegateError,
     "DelegateLibrarySupportNotBuiltIn","`%s' (X11)",image_info->filename);
-  DisplayUsage();
+  (void) DisplayUsage();
   return(MagickFalse);
 #endif
 }
