@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -55,7 +55,7 @@ extern WandExport GravityType
 
 extern WandExport Image
   *MagickDestroyImage(Image *),
-  *GetImageFromMagickWand(MagickWand *);
+  *GetImageFromMagickWand(const MagickWand *);
 
 extern WandExport ImageType
   MagickGetImageType(MagickWand *);
@@ -81,7 +81,6 @@ extern WandExport MagickBooleanType
   MagickAddNoiseImage(MagickWand *,const NoiseType),
   MagickAddNoiseImageChannel(MagickWand *,const ChannelType,const NoiseType),
   MagickAffineTransformImage(MagickWand *,const DrawingWand *),
-  MagickAffinityImage(MagickWand *,const MagickWand *,const DitherMethod),
   MagickAnnotateImage(MagickWand *,const DrawingWand *,const double,
     const double,const double,const char *),
   MagickAnimateImages(MagickWand *,const char *),
@@ -132,6 +131,9 @@ extern WandExport MagickBooleanType
   MagickEvaluateImage(MagickWand *,const MagickEvaluateOperator,const double),
   MagickEvaluateImageChannel(MagickWand *,const ChannelType,
     const MagickEvaluateOperator,const double),
+  MagickExportImagePixels(MagickWand *,const long,const long,
+    const unsigned long,const unsigned long,const char *,const StorageType,
+    void *),
   MagickExtentImage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
   MagickFlipImage(MagickWand *),
@@ -163,8 +165,6 @@ extern WandExport MagickBooleanType
   MagickGetImagePage(MagickWand *,unsigned long *,unsigned long *,long *,
     long *),
   MagickGetImagePixelColor(MagickWand *,const long,const long,PixelWand *),
-  MagickGetImagePixels(MagickWand *,const long,const long,const unsigned long,
-    const unsigned long,const char *,const StorageType,void *),
   MagickGetImageRange(MagickWand *,double *,double *),
   MagickGetImageRedPrimary(MagickWand *,double *,double *),
   MagickGetImageResolution(MagickWand *,double *,double *),
@@ -172,6 +172,9 @@ extern WandExport MagickBooleanType
   MagickHasNextImage(MagickWand *),
   MagickHasPreviousImage(MagickWand *),
   MagickImplodeImage(MagickWand *,const double),
+  MagickImportImagePixels(MagickWand *,const long,const long,
+    const unsigned long,const unsigned long,const char *,const StorageType,
+    const void *),
   MagickLabelImage(MagickWand *,const char *),
   MagickLevelImage(MagickWand *,const double,const double,const double),
   MagickLevelImageChannel(MagickWand *,const ChannelType,const double,
@@ -227,9 +230,11 @@ extern WandExport MagickBooleanType
   MagickReadImageFile(MagickWand *,FILE *),
   MagickRecolorImage(MagickWand *,const unsigned long,const double *),
   MagickReduceNoiseImage(MagickWand *,const double),
+  MagickRemapImage(MagickWand *,const MagickWand *,const DitherMethod),
   MagickRemoveImage(MagickWand *),
   MagickResampleImage(MagickWand *,const double,const double,const FilterTypes,
     const double),
+  MagickResetImagePage(MagickWand *,const char *),
   MagickResizeImage(MagickWand *,const unsigned long,const unsigned long,
     const FilterTypes,const double),
   MagickRollImage(MagickWand *,const long,const long),
@@ -265,8 +270,7 @@ extern WandExport MagickBooleanType
   MagickSetImageGravity(MagickWand *,const GravityType),
   MagickSetImageFormat(MagickWand *,const char *),
   MagickSetImageInterlaceScheme(MagickWand *,const InterlaceType),
-  MagickSetImageInterpolateMethod(MagickWand *,
-    const InterpolatePixelMethod),
+  MagickSetImageInterpolateMethod(MagickWand *,const InterpolatePixelMethod),
   MagickSetImageIterations(MagickWand *,const unsigned long),
   MagickSetImageMatte(MagickWand *,const MagickBooleanType),
   MagickSetImageMatteColor(MagickWand *,const PixelWand *),
@@ -274,7 +278,6 @@ extern WandExport MagickBooleanType
   MagickSetImageOrientation(MagickWand *,const OrientationType),
   MagickSetImagePage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
-  MagickResetImagePage(MagickWand *,const char *),
   MagickSetImagePixels(MagickWand *,const long,const long,const unsigned long,
     const unsigned long,const char *,const StorageType,const void *),
   MagickSetImageRedPrimary(MagickWand *,const double,const double),
@@ -285,8 +288,8 @@ extern WandExport MagickBooleanType
   MagickSetImageType(MagickWand *,const ImageType),
   MagickSetImageUnits(MagickWand *,const ResolutionType),
   MagickSetImageWhitePoint(MagickWand *,const double,const double),
-  MagickShadeImage(MagickWand *,const MagickBooleanType,const double,const 
-    double),
+  MagickShadeImage(MagickWand *,const MagickBooleanType,const double,
+    const double),
   MagickShadowImage(MagickWand *,const double,const double,const long,
     const long),
   MagickSharpenImage(MagickWand *,const double,const double),
@@ -300,6 +303,8 @@ extern WandExport MagickBooleanType
     const MagickBooleanType,const double,const double),
   MagickSketchImage(MagickWand *,const double,const double,const double),
   MagickSolarizeImage(MagickWand *,const double),
+  MagickSparseColorImage(MagickWand *,const ChannelType,const SparseColorMethod,
+    const unsigned long,const double *),
   MagickSpliceImage(MagickWand *,const unsigned long,const unsigned long,
     const long,const long),
   MagickSpreadImage(MagickWand *,const double),
@@ -389,7 +394,8 @@ extern WandExport unsigned long
   MagickGetNumberImages(MagickWand *);
 
 extern WandExport VirtualPixelMethod
-  MagickGetImageVirtualPixelMethod(MagickWand *);
+  MagickGetImageVirtualPixelMethod(MagickWand *),
+  MagickSetImageVirtualPixelMethod(MagickWand *,const VirtualPixelMethod);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
