@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -28,7 +28,8 @@ extern WandExport MagickBooleanType
   MogrifyImageInfo(ImageInfo *,const int,const char **,ExceptionInfo *),
   MogrifyImageList(ImageInfo *,const int,const char **,Image **,
     ExceptionInfo *),
-  MogrifyImages(ImageInfo *,const int,const char **,Image **,ExceptionInfo *);
+  MogrifyImages(ImageInfo *,const MagickBooleanType,const int,const char **,
+    Image **,ExceptionInfo *);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
